;$Id: shapelets_amoeba.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      SHAPELETS_CALC_CHI2
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE: 
;      Computes the chi2 difference between the input and the reconstructed
;      image for a given value of beta (w) and n_max
;
; INPUTS: 
;      pstamp - pstamp structure from shapelets_image2pstamp.pro
;      w    - shapelet scale size beta
;      nmax - maximum order of shapelet decomposition
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS: 
;      Reduced chi^2 is returned
;
; OPTIONAL OUTPUTS: 
;      decomp - the shapelet decomposition structure is returned
;      recomp - a reconstructed image of the shapelet model
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier

function shapelets_calc_chi2, PSTAMP, BETA, N_MAX, $
                              X0=x0,               $
                              DECOMP=decomp,       $
                              RECOMP=recomp,       $
                              PSF=psf,             $
                              SKY=sky

COMPILE_OPT idl2, HIDDEN

; Should always have had to specify at least the object's centroid
if not keyword_set(x0) then begin
  x0=size(pstamp.image,/dimensions)/2.
  message,"Need to set centroid!"
endif

; Decompose image into shapelets
shapelets_decomp,pstamp.image,beta,n_max,decomp,recomp,x0=x0,noise=pstamp.noise,$
  /ls,/over,/integrate,psf=psf,sky=sky

; Return chi^2 of fit
return,decomp.chisq[1]

end




; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      SHAPELETS_AMOEBA_CHI2
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE: 
;      Compute Chi^2(beta) in the format required for the amoeba
;      minimisation procudure shapelets_1d_amoeba.pro. This version also 
;      finds the optimal centroid, x0.
;
; INPUTS: 
;      BETA    - Shapelet scale size beta
; 
; COMMON VARIABLES:
;      PSTAMP  - Object structure from shapelets_extract_pstamp.pro
;      X0      - Position of the centre of the shapelet basis functions
;      NMAX    - Value of n_max to use
;      FLAG    - Flag for decomposition
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      Reduced chi^2(beta) is returned.
;      Updates position of the centre of the shapelet basis functions
;      in the common block. Also possibly alters the flag.
;
; MODIFICATION HISTORY:
;      Apr 05 - Bug in flagging spotted by Stephane Paulin-Henriksson
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier


function shapelets_amoeba_chi2,BETA,PSF=psf,SKY=sky,VERBOSE=verbose

COMPILE_OPT idl2, HIDDEN

common shapelets_betax0_common,pstamp_1,nmax_1,x0_1,hist,th_min_geom_1,flag_1

; Compute geometrical limits for th_max and th_min
shapelets_geometric_contstraints, pstamp_1, th_min_1, th_max_geom, x0=x0

; Compute the shapelet decomposition's th_min and th_max
shapelets_theta_minmax, beta, nmax_1, theta

; Are these going to be allowed?
if (beta le 0.) or (theta[1] gt th_max_geom) or (theta[0] lt th_min_geom_1) then begin; No
  c2=!values.f_infinity     ; Bounce of walls of allowed n_max vs beta
  flag_1=flag_1>1           ;  space by setting chi^2 to infinity.
  xc=x0_1                   ; Keep centroid the same.
endif else begin                                                            ; Yes
  
  ; Make a shapelet decomposition with the new parameters and calculate
  ;    its chi^2 value.
  c2=shapelets_calc_chi2(pstamp_1,beta,nmax_1,x0=x0_1,decomp=decomp,psf=psf,sky=sky)
  
  ; Set centre of basis functions for next iteration to the location
  ;   of the current decomposition's first moments.
  ;xc=1.5*shapelets_centroid(decomp)-1.0*x0_1
  xc=shapelets_centroid(decomp)
  
  ; If this is then going to extend outside the postage stamp, don't move it!
  shapelets_geometric_contstraints, pstamp_1, th_min_geom_1, th_max_geom, x0=xc
  if theta[1] gt th_max_geom then begin
    xc=x0_1
    flag_1=flag_1>5
    message,"Centroid wandered, pushing model off postage stamp",/info,noprint=1-verbose
  endif
 
endelse

; Update the history records
hist.i=hist.i+1
hist.beta[hist.i]=beta
hist.nmax[hist.i]=nmax_1
hist.x0[*,hist.i]=x0_1
hist.chi2[hist.i]=c2

;Write to screen
message," n_max, beta, chi^2, x_c:"+$
  string(nmax_1)+string(beta)+string(c2)+string(x0_1[0])+string(x0_1[1]),$
  /info,noprint=1-verbose

; Move shapelet centre to current centroid
x0_1=xc

; Set a flag if this moves the outside of the shapelet decomposition
;   off the edge of the postage stamp
npix_x=1+pstamp_1.im_ran[3]-pstamp_1.im_ran[2]
npix_y=1+pstamp_1.im_ran[1]-pstamp_1.im_ran[0]
th_max=beta*sqrt(nmax_1+1)-1 ; -1 for good luck: allow a little leeway!
if min(xc) lt th_max or xc[0] gt npix_x-th_max or xc[1] gt npix_y-th_max then flag_1=flag_1>5

return,c2

end



; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      SHAPELETS_AMOTRY
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Extrapolates by a factor fac through the face of the simplex, across
;      from the high point, tries it and replaces the high point if the new
;      point is better.
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Adapted from Numerical Recipies by Alexandre Refregier

function shapelets_amotry,P,Y,FUNC,ILO,IHI,FAC,PTRY, $
                          PSF=psf,                   $
                          SKY=sky,                   $
                          VERBOSE=verbose

COMPILE_OPT idl2, HIDDEN

ptry=p[ilo]+fac*(p[ihi]-p[ilo])
ytry=call_function(func,ptry,psf=psf,sky=sky,verbose=verbose)
if ytry lt y[ihi] then begin
  y[ihi]=ytry
  p[ihi]=ptry
endif
return,ytry
end






; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;      SHAPELETS_AMOEBA
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Minimise a function f(x). This is a one dimensional version
;      of Numerical Recipies' "Amoeba". Later specialised to 
;
; INPUTS:
;      FTOL            - Fractional tolerance to be achieved in the function.
;      STARTING POINT  - Simplex, or p0 and scale, must be given (see below)
;
; INPUTS REQUIRED FROM SHAPELETS_FOCUS_BETAX0:
;      PSTAMP          - Postage stamp structure of the image.
;      NMAX            - Maximum shapelet order at which to attempt
;                        optimisation.
;
; OPTIONAL INPUTS:
;      FUNCTION_NAME   - A string containing the name of the
;                        function to be minimized (default='func')
;      SIMPLEX=[X0,X1] - Starting simplex. If not set, scale and p0 
;                        must be used.
;      P0              - Starting point. Must be used with scale. (beta).
;      SCALE           - Scale for the starting point. Must be used with p0.
;      FUNCTION_VALUE  - On exit contains the function values at the
;                        simplex points. The first element contains the 
;                        function minimum. (output only)
;      MAX_ITER        - Maximum number of calls to func (default=100)
;                        note that ncalls can exceed nmax by a few because of
;                        internal juggling to get the best fit.
;
; OUTPUTS:
;      None as such.
; 
; OPTIONAL OUTPUTS:
;      FOCUS           - Updated history of search.
;      FLAG            - Updated success flag.
;      DECOMP          - Shapelet decomposition of image as a decomp structure.
;      RECOMP          - Repixellated version of shapelet model.
;      FUNCTION_VALUE  - The function values at the final simplex points. 
;                        The first element contains the function minimum.
;      SIMPLEX         - The corresponding final positions of the amoeba's legs.
;      NCALLS          - Total number of steps taken by amoeba.
;
; MODIFICATION HISTORY:
;      Mar 05 - Bug fixed by RM to break out of iterative loop as soon as converged.
;      Jan 05 - Common block and hist structure now internal to this routine, RM.
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier
;-

pro shapelets_amoeba,FTOL,PSTAMP,NMAX,              $
                     FUNCTION_NAME=function_name,   $
                     X0=x0,                         $
                     NCALLS=ncalls,                 $
                     FOCUS=focus,                   $
                     ITER_MAX=iter_max,             $
                     P0=p0,                         $
                     SCALE=scale,                   $
                     SIMPLEX=simplex,               $
                     FUNCTION_VALUE=function_value, $
                     VERBOSE=verbose,               $
                     FLAG=flag,                     $
                     PSF=psf,                       $
                     SKY=sky,                       $
                     TH_MIN=th_min,                 $
                     HIST=hist,                     $
                     DECOMP=decomp,                 $
                     RECOMP=recomp

COMPILE_OPT idl2

common shapelets_betax0_common,$
       pstamp_common,nmax_common,x0_common,hist_common,th_min_common,flag_common


; declarations
npts=2           ; number of points in the simplex
if not keyword_set(iter_max) then iter_max=100  ; maximum number of iterations allowed
if not keyword_set(function_name) then function_name='function_name'
if not keyword_set(th_min) then if keyword_set(focus) then th_min=focus.th_min else th_min=0.2
if not keyword_set(x0) then if keyword_set(pstamp) then x0=[pstamp.xo,pstamp.yo] else x0=[0,0]
if not keyword_set(nmax) then if keyword_set(focus) then nmax=focus.n_max else nmax=0
if not keyword_set(flag) then if keyword_set(focus) then flag=focus.flag else flag=0

; initialise common block
pstamp_common=pstamp
x0_common=x0
nmax_common=nmax
th_min_common=th_min
flag_common=flag
hist_common={i:-1,                   $
	   	   beta:fltarr(iter_max+2),$
	  	   nmax:intarr(iter_max+2),$
	  	   chi2:fltarr(iter_max+2),$
       	   x0:fltarr(2,iter_max+2)}


; Initialise simplex
if keyword_set(simplex) then begin
  p=simplex
endif else begin
  if (keyword_set(scale) and keyword_set(p0)) then begin
    p=p0+[0.,scale]  ; if not specified set starting simplex using p0 and scale
  endif else message,'Simplex or starting point and scale MUST be specified!'
endelse


; Initialise function at simplex points, looking back at focus struct if available
y=fltarr(npts)
if keyword_set(focus) then begin
  y[0]=focus.chisq 
  done_previously=1
endif else done_previously=0
for i=done_previously,npts-1 do y[i]=call_function(function_name,p[i],psf=psf,sky=sky,verbose=verbose)
ncalls=npts


; Iterate by moving the simplex
converged=0B
while ncalls lt iter_max and not converged do begin

  ; Sort simplex
  ilo=0 & ihi=1
  if y[0] gt y[1] then begin
    ilo=1 & ihi=0
  endif

  ; Test for tolerance
  d=total(abs(y))
  if d ne 0 then rtol=abs(y[ihi]-y[ilo])*2./d else rtol=abs(ftol)/2.
  if rtol lt abs(ftol) then begin  ; done?
    if keyword_set(simplex) then simplex=[p[ilo],p[ihi]]
    if keyword_set(function_value) then function_value=[y[ilo],y[ihi]]
    converged=1B
    break
  endif

  ; First reflect about lowest point
  ytry=shapelets_amotry(p,y,function_name,ilo,ihi,-1.0,ptry,psf=psf,sky=sky,verbose=verbose)
  ncalls=ncalls+1

  ; Is new point lower than lowest point?
  if ytry le y[ilo] then begin  ; try further expansion if so
    ytry=shapelets_amotry(p,y,function_name,ilo,ihi,2.0,ptry,psf=psf,sky=sky,verbose=verbose) 
    ncalls=ncalls+1
  endif else begin  ; if not, try contraction about lowest point
    ysave=y[ihi]
    ytry=shapelets_amotry(p,y,function_name,ilo,ihi,0.5,ptry,psf=psf,sky=sky,verbose=verbose) 
    ncalls=ncalls+1
    if ytry ge ysave then begin   ; if latest point is worse then contract
                                  ; about lowest point
      p[ilo]=p[ilo]-.5*(p[ilo]-p[ihi])
      y[ilo]=ytry
    endif
  endelse

endwhile


; In case the convergence was not achieved with iter_max steps
if not converged then begin
  message,"Amoeba failed to converge within "+strtrim(string(iter_max),2)+" steps",$
          /info,noprint=1-verbose
  flag_common=flag_common>4
  ilo=0 & ihi=1    ; sort simplex one last time
  if y[0] gt y[1] then begin
    ilo=1 & ihi=0
  endif
endif


; Gather additional data to return to calling routine.
simplex=[p[ilo],p[ihi]]
function_value=[y[ilo],y[ihi]]
x0=x0_common
flag=flag_common


; Make one last decomposition at the best beta and x0
; (could really just look back over last couple of steps and find best one,
; especially because, at a new centroid, this decomposition could be worse!)
shapelets_decomp,pstamp.image,p[ilo],nmax_common,decomp,recomp,$
  x0=x0,/ls,/over,noise=pstamp.noise,/integrate,psf=psf
; Should really call like this, to keep the amoeba routine general
;y[i]=call_function(function_name,p[i],psf=psf,sky=sky,verbose=verbose)
if keyword_set(printit) then message,"n_max, beta, chi^2, x_c:"+$
  string(nmax)+string(beta_min)+string(decomp.chisq[1])+string(x0[0])+string(x0[1]),/info
hist_common.i=hist_common.i+1
hist_common.beta[hist_common.i]=p[ilo]
hist_common.nmax[hist_common.i]=nmax_common
hist_common.x0[*,hist_common.i]=x0
hist_common.chi2[hist_common.i]=decomp.chisq[1]


; If we are keeping an external record of progress, update the focus structure
if keyword_set(focus) then begin
  focus_new={name:focus.name,type:"focus"}
  names=tag_names(focus)
  for i=0,n_tags(focus)-1 do begin
    case strupcase(names[i]) of
      "NAME":
      "TYPE":
      "FLAG":          focus_new=create_struct(focus_new,"flag",flag_common)
      "BETA":          focus_new=create_struct(focus_new,"beta",p[ilo])
      "X0":            focus_new=create_struct(focus_new,"x0",x0_common)
      "CHISQ":         focus_new=create_struct(focus_new,"chisq",y[ilo]) ;???
      "N_ITERATIONS":  focus_new=create_struct(focus_new,"n_iterations",focus.n_iterations+hist_common.i+1)
      "HISTORY":       focus_new=create_struct(focus_new,"history",[focus.history,"beta/centroid optimised"])
      "BETA_HISTORY":  focus_new=create_struct(focus_new,"beta_history",[focus.beta_history,hist_common.beta[0:hist_common.i]])
      "N_MAX_HISTORY": focus_new=create_struct(focus_new,"n_max_history",[focus.n_max_history,hist_common.nmax[0:hist_common.i]])
      "X0_HISTORY":    focus_new=create_struct(focus_new,"x0_history",transpose([transpose(focus.x0_history),transpose(hist_common.x0[*,0:hist_common.i])]))
      "CHISQ_HISTORY": focus_new=create_struct(focus_new,"chisq_history",[focus.chisq_history,hist_common.chi2[0:hist_common.i]])
      else: focus_new=create_struct(focus_new,names[i],focus.(i))
    endcase
  endfor
  focus=focus_new
endif

end
