;$Id: shapelets_focus.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;       SHAPELETS_FOCUS
;
; PURPOSE:
;       Find the optimal beta, n_max and centroid for a shapelet
;       decomposition via an iterative search throughout the space spanned
;       by these (meta-)variables. See the paper Polar shapelets, by
;       Massey & Refregier (2004) for more details.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       pstamp - Postage stamp structure containing the image (and other
;                parameters) around a single object. This can be created with
;                shapelets_image2pstamp.pro
;
; OPTIONAL INPUTS:
;       BETA_GUESS  - Guess starting point for beta (DEFAULT: guesses itself) 
;       N_MAX_GUESS - Guess starting point for n_max (DEFAULT: guesses itself)
;       TOL_CHISQ   - Fractional tolerance for Chi^2 minimisation
;       TOL_BETA    - Fractional tolerance for finding beta
;       CHISQ0      - Target value of reduced Chi^2 to achieve (DEFAULT: 1.0)
;       TH_MIN_GEOM - Minimum geometrical scale theta_min (DEFAULT: 0.5 pixels) 
;       N_MAX_RANGE - Range of allowed n_max values (DEFAULT: [2,12])
;	   FIXED_BETA  - Fixed value of beta that should be used for decomposition.
;       PSF         - A Cartesian decomp structure representing the local PSF.
;                     The shapelet model of the galaxy will be deconvolved from
;                     this.
;
; KEYWORD PARAMETERS:
;       SILENT     - Operate silently.
;       VERBOSE    - Operate noisily.
;       SKY        - 1: fit sky background with a constant value around object.
;                    2: fit sky gradient with a plane around object.
;                    DEFAULT: no sky subtraction.
;       NON1       - a_01 and a_10 are forced to be zero, Kuijken-like.
;
; OUTPUTS:
;       DECOMP     - Decomposition structure containing the shapelet
;                    coefficients with the optimal beta, n_max and centroid
;                    parameters.
;       FOCUS      - IDL structure tracing the search on the beta-n_max plane.
;
; OPTIONAL OUTPUTS:
;       RECOMP     - Recomposed image using optimal shapelet parameters.
;
; CALLING SEQUENCE:
;       shapelets_focus,pstamp,focus,decomp
;
; TO DO:
;       Initialise focus & hist structures here. And add hist to common block...
;
;       Should check through history list at end to find chisq closest to 1.
;       On occasion, we have gone past this and stopped at something else.
;       Will need to store chi-squareds in history structure to implement this!
;
; NOTES:
;       MEANINGS OF FLAG:
;       0: OK
;       1: has bounced into theta_min or max wall at some point
;       2: n_max_max reached - shapelets incompletely represent object
;       3: chisq>1 but converged (in n_max search) by flatness limit
;       4: did not converge
;       5: centroid off edge
;       6: fatal error
;       NB it can only be one of these. If duplicated, highest number takes
;          priority.
;
; MODIFICATION HISTORY:
;       Apr 05 - NON1 keyword added by RM.
;       Apr 05 - Guess used as starting n_max by Stephane Paulin-Henriksson.
;       Nov 04 - PSF passed to guess_nmax_beta routine by RM.
;       Aug 04 - Fixed_beta and max_loops keywords added by RM.
;       Jul 04 - Header updated to reflect new additional options by RM.
;       May 04 - Tidied up by RM.
;       Apr 02 - PSF deconvolution implemented by R. Massey.
;       Apr 02 - Written by A. Refregier.
;-

pro shapelets_focus, pstamp, focus, decomp, recomp,                         $
                     PSF=psf, SKY=sky, NON1=non1,                           $
                     BETA_GUESS=beta_guess, N_MAX_GUESS=n_max_guess,        $
                     CHISQ0=chisq0, TOL_CHISQ=tol_chisq, TOL_BETA=tol_beta, $
                     N_MAX_RANGE=n_max_range, FIXED_BETA=fixed_beta,        $
                     TH_MIN_GEOM=th_min_geom, MAX_LOOPS=max_loops,          $
                     VERBOSE=verbose, SILENT=silent

COMPILE_OPT idl2

; Initial declarations
if not keyword_set(tol_beta) then tol_beta=1e-2
if not keyword_set(tol_chisq) then tol_chisq=sqrt(2./n_elements(pstamp.image)) ; rms for reduced Chi^2
if not keyword_set(chisq0) then chisq0=1.0+tol_chisq
if not keyword_set(n_max_range) then n_max_range=[2,12]
if not keyword_set(max_loops) then max_loops=20
if not keyword_set(th_min_geom) then th_min_geom=0.2


; Guess starting points (unless specified by hand)
shapelets_guess_nmaxbeta,pstamp,guess,th_min_geom=th_min_geom*1.05,psf=psf
       ; 1.05 factor present to really ensure that guess is not in forbidden area
if not keyword_set(beta_guess) then beta_guess=guess.beta
;if not keyword_set(n_max_guess) then n_max_guess=n_max_range[0]>guess.nmax<n_max_range[1]
if not keyword_set(n_max_guess) then n_max_guess=2 ; Ignore guess.n_max and start at low n for a quick beta search
if not keyword_set(x0_guess) then x0_guess=guess.x0


; Try a first attempt at the fit using this position
if keyword_set(fixed_beta) then beta=fixed_beta else beta=beta_guess
n_max=n_max_range[0]>n_max_guess<n_max_range[1] ; Ignore n_max guess and start at low n for a quick beta search
x0=x0_guess
if keyword_set(verbose) then message,"Guessing n_max, beta, x_c:     "+$
  string(n_max)+string(beta)+"             "+string(x0[0])+string(x0[1]),/info
shapelets_decomp,pstamp.image,beta,n_max,decomp,recomp,noise=pstamp.noise,$
                 x0=x0,/ls,/over,/integrate,psf=psf,sky=sky


; Initialise focus structure to contain the history of this search
focus={name:pstamp.name,                                      $
       type:"focus",                                          $
       history:strarr(1),                                     $
       beta:beta,                                             $
       n_max:n_max,                                           $
       x0:x0,                                                 $
       chisq:decomp.chisq[1],                                 $
       chisq0:chisq0,                                         $
       tol_chisq:tol_chisq,                                   $
       th_min:th_min_geom,                                    $
       n_max_guess:n_max_guess,                               $
       beta_guess:beta_guess,                                 $
       x0_guess:x0_guess,                                     $
       n_iterations:1,                                        $
       beta_history:beta,                                     $
       n_max_history:n_max,                                   $
       x0_history:x0,                                         $
       chisq_history:decomp.chisq[1],                         $
       flag:0,                                                $
       flag_interpret:["OK",                                                                           $ ; 0
                       "Bounced off geometrical constraints",                                          $ ; 1
                       "Maximum n_max reached - shapelets may incompletely represent object",          $ ; 2
                       "Converged by flatness limit - shapelets may incompletely represent object",    $ ; 3
                       "Failed to converge",                                                           $ ; 4
                       "Centroid wandered, pushing basis functions off the edge of the postage stamp", $ ; 5
                       "Fatal crash during focus routine"]}                                              ; 6

  
; Try to find a better beta
shapelets_focus_betax0,pstamp,n_max,focus,decomp,recomp, $
  beta_guess=beta,x0_guess=x0,                           $
  th_min=th_min_geom,tol=1e-2,                           $
  psf=psf,sky=sky,verbose=verbose,/append
x0=focus.x0
beta=focus.beta
if keyword_set(fixed_beta) then beta=fixed_beta 


; Alternate n_max-finder and beta-finder until tolerance is reached
found=0
iloop=0
while not found do begin
    
  ; Try to find a better n_max
  n_max_old=n_max
  shapelets_focus_nmax,pstamp,beta,focus,decomp,recomp,x0=x0,chisq0=chisq0,$
         n_max_range=n_max_range,$
	    psf=psf,verbose=verbose,sky=sky,/append
  if focus.flag eq 6 then break    ; about to crash. quit before it gets there
  n_max=focus.n_max
  if n_max_old eq n_max then found=1 ; exit if n_max has not changed  
  if focus.flag eq 3 then found=1  ; exit if we've found a plateau 
       ; this helps low S/N and I THINK its OK about falling off edge, but...

  if keyword_set(fixed_beta) then break
  

  ; Try to find a better beta
  beta_old=beta
  if not found then begin   
    shapelets_focus_betax0,pstamp,n_max,focus,decomp,recomp,beta_guess=beta,x0_guess=x0,$
       /append,psf=psf,verbose=verbose,sky=sky,tol=tol_beta
    beta=focus.beta
    x0=focus.x0
    if abs(beta-beta_old) le tol_beta then found=1  ; exit if beta has not changed  
  endif
  if focus.flag eq 5 then begin
    message,"Centroid wandered, pushing model off postage stamp",/info,noprint=1-verbose
    found=1  ; if its fallen off edge just quit now
  endif

  ; Limit the number of loops
  iloop=iloop+1
  if iloop gt max_loops then begin
    focus.flag=(focus.flag>4)
    break
  endif
  
endwhile

; Do one final decomposition if we require coeffs with n=1 to be zero
if keyword_set(non1) then begin
  shapelets_decomp,pstamp.image,$
    focus.beta,focus.n_max,decomp,recomp,x0=focus.x0,/ls,/over,$
    noise=pstamp.noise,/integrate,psf=psf,sky=sky,non1=non1
  focus.chisq=decomp.chisq[1]
endif

; Print final used step to screen, without repreating the decomposition
if keyword_set(verbose) then message,"       n_max, beta, chi^2, x_c:"+$
  string(focus.n_max)+string(focus.beta)+string(focus.chisq)+$
  string(focus.x0[0])+string(focus.x0[1]),/info

; Print result to screen
message,'Flags: ('+strtrim(pstamp.flag,1)+','+strtrim(focus.flag,1)+'), '+$
        'n_max='+strtrim(focus.n_max,1)+', chi^2='+strtrim(focus.chisq,1),/info,noprint=silent


end


