;$Id: shapelets_focus_betax0.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;       SHAPELETS_FOCUS_BETAX0
;
; PURPOSE:
;       Find the optimal beta and centroid x0 for the decomposition of an
;       image by minimising Chi^2. This is done using a 1-dimensional Amoeba
;       search. The maximum shapelet order nmax is assumed to be known.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       PSTAMP     - IDL postage stamp structure extracted from a large image
;                    via shapelets_image2pstamp.pro
;       NMAX       - Maximum shapelet order for decomposition
;
; OPTIONAL INPUTS:
;       X0_GUESS   - Guess for centroid (default:pstamp.x0)
;       BETA_GUESS - Initial guess for beta (default: estimated)
;       ITER_MAX   - Maximum number of iterations (default:30)
;       TOL        - Chi^2 tolerance for Amoeba search (default=1e-3)
;       APPEND     - Append the history file to the current
;                    shapelets_focus structure
;       TH_MIN     - Minimum scale theta_min (default:.5 pixels)
;       FOCUS      - Structure containing the prior history of the search
;       SCALE_FACT - Controls the size of the amoeba's steps
;
; KEYWORD PARAMETERS:
;       verbose    - Verbose output to screen
;
; OUTPUTS:
;       DECOMP     - Decomposition at optimal beta and x0
;       FOCUS      - Structure containing the history of the search
;
; TO DO:
;       Check length of history structure after iteration. Or just append
;       during normal operation, rather than initialising n_h thus.
;
; MODIFICATION HISTORY:
;       Jan 05     - Optimisation & bugs fixed in centroid iteration by RM
;       May 04     - Sky background fitting added by RM
;       Apr 02     - PSF deconvolution added by Richard Massey
;       Mar 02     - Written by Alexandre Refregier
;-

pro shapelets_focus_betax0, PSTAMP, NMAX, FOCUS, DECOMP, RECOMP,$
                            BETA_GUESS=beta_guess,        $
                            X0_GUESS=x0_guess,            $
                            APPEND=append,                $
                            ITER_MAX=iter_max,            $
                            TOL=tol,                      $
                            TH_MIN=th_min,                $
                            SCALE_FACT=scale_fact,        $
                            PSF=psf,                      $
                            SKY=sky,                      $
                            VERBOSE=verbose

COMPILE_OPT idl2

; Initial declarations
if not keyword_set(iter_max) then iter_max=30
if not keyword_set(tol) then if keyword_set(focus) then tol=focus.tol_chisq else tol=sqrt(2./n_data) ; rms for reduced Chi^2
if not keyword_set(th_min) then if keyword_set(focus) then th_min=focus.th_min else th_min=0.2
if not keyword_set(scale_fact) then scale_fact=4.3
if keyword_set(append) and not keyword_set(focus) then append=0B ; Nothing to append to!
if not keyword_set(append) then flag=0 else flag=focus.flag
if not keyword_set(verbose) then verbose=0


; Guess starting points for beta and x0 if not provided
; (Could be more intelligent here, since n_max is a required input).
shapelets_guess_nmaxbeta,pstamp,guess
if not keyword_set(beta_guess) then begin
shapelets_guess_nmaxbeta,pstamp,guess
  beta_guess=guess.beta
  message,'Guessing beta of '+strtrim(string(beta_guess),2),/info,noprint=1-verbose
endif
if not keyword_set(x0_guess) then begin
  x0_guess=guess.x0
  message,'Guessing centroid of ('+strtrim(string(x0_guess[0]),2)+','+$
          strtrim(string(x0_guess[1]),2)+')',/info,noprint=1-verbose
endif 


; Create focus structure if one doesn't already exist
if not keyword_set(focus) or not keyword_set(append) then begin
  ; Have a first attempt at decomposition, using the guesses
  shapelets_decomp,pstamp.image,beta_guess,nmax,decomp,recomp,x0=x0_guess,$
                   noise=pstamp.noise,/ls,/over,/integrate,psf=psf,sky=sky
  focus={name:pstamp.name,                                      $
         type:"focus",                                          $
         history:"Created during beta/centroid search",         $
         beta:beta_guess,                                       $
         n_max:nmax,                                            $
         x0:x0_guess,                                           $
         chisq:decomp.chisq[1],                                 $
         chisq0:1+tol,                                          $
         tol_chisq:tol,                                         $
         th_min:th_min,                                         $
         n_max_guess:nmax,                                      $
         beta_guess:beta_guess,                                 $
         x0_guess:x0_guess,                                     $
         n_iterations:1,                                        $
         beta_history:beta_guess,                               $
         n_max_history:nmax,                                    $
         x0_history:x0_guess,                                   $
         chisq_history:decomp.chisq[1],                         $
         flag:0,                                                $
         flag_interpret:["OK",                                                                           $ ; 0
                         "Bounced off geometrical constraints",                                          $ ; 1
                         "Maximum n_max reached - shapelets may incompletely represent object",          $ ; 2
                         "Converged by flatness limit - shapelets may incompletely represent object",    $ ; 3
                         "Failed to converge",                                                           $ ; 4
                         "Centroid wandered, pushing basis functions off the edge of the postage stamp", $ ; 5
                         "Fatal crash during focus routine"]}                                              ; 6
endif


; Set scale for amoeba search
beta_max=min([x0_guess[0],pstamp.im_ran[1]-pstamp.im_ran[0]-x0_guess[0],$ 
            x0_guess[1],pstamp.im_ran[3]-pstamp.im_ran[2]-x0_guess[1]])$  ; th_max
		  /sqrt(nmax+1)
beta_min=(th_min*sqrt(nmax+1))
direction=sign(-2.*beta_guess+beta_max+beta_min) ; But which is it closer to? 
scale=direction*beta_guess/scale_fact            ; Go in opposite direction!
message,"Finding beta that minimises chi^2",/info,noprint=1-verbose


; Optimise beta and x0 by minimizing chi^2
shapelets_amoeba,TOL,PSTAMP,NMAX,$
                 FUNCTION_NAME='shapelets_amoeba_chi2',$
                 FOCUS=focus,$
                 ITER_MAX=iter_max,$
                 TH_MIN=th_min,$
                 X0=x0_guess,$
                 P0=beta_guess,$
                 SCALE=scale,$
                 FUNCTION_VALUE=function_value,$
                 SIMPLEX=simplex,$
                 PSF=psf,$
                 SKY=sky,$
                 VERBOSE=verbose


; Catch errors if amoeba failed to converge
;if focus.flag ge 4 then 
;  ...
;endif


end
