;$Id: shapelets_focus_nmax.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;       SHAPELETS_FOCUS_NMAX
;
; PURPOSE:
;       Find the optimal n_max for the decomposition of an image by exploring
;       different values until Chi^2 is equal to a set value Chi2_0.
;       The shapelet scale beta is assumed to be known.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       PSTAMP   - Object structure from shapelets_extract_pstamp.pro
;       BETA     - Shapelet scale for decomposition
;
; OPTIONAL INPUTS:
;       CHI20       - Value of Chi^2 to achieve (DEFAULT: 1+tol)
;       X0          - Centroid for decomposition (DEFAULT: pstamp.x0)
;       N_MAX_RANGE - Range of n_max value to explore (DEFAULT: [2,12])
;       TOL_CHI2    - Tolerance for Chi^2 flatness (DEFAULT: 1sigma for Chi^2)
;       TH_MIN_GEOM - Minimum scale theta_min (DEFAULT: 0.5 pixels) 
;       FOCUS       - Structure containing the history of the search
;
; KEYWORD PARAMETERS:
;       APPEND   - Append the history file to the current focus structure
;
; OUTPUTS:
;       DECOMP   - Decomposition at optimal beta and x0
;       RECOMP   - Recomposed pixellated image of the optimal model
;       FOCUS    - Updated structure containing the history of the search
;
; MODIFICATION HISTORY:
;       Apr 05 - Iteration stopped if chi^2 is getting worse by RM
;       Apr 05 - Iteration forced to stop at n_max Stephane Paulin-Henriksson
;       Jan 05 - Tidied up to avoid awkward history structure by RM 
;       Jan 05 - Beta adjustments to follow geometric constraints by RM 
;       Dec 04 - Bug fixed in final search around nearby values of n_max by RM
;       May 04 - Sky background fitting added by RM
;       Apr 02 - PSF deconvolution added by Richard Massey
;       Mar 02 - Written by Alexandre Refregier
;-

pro shapelets_focus_nmax,pstamp,beta,focus,decomp,recomp,chisq0=chisq0,x0=x0,$
  n_max_range=n_max_range,append=append,tol_chisq=tol,$
  th_min_geom=th_min_geom,psf=psf,sky=sky,verbose=verbose

COMPILE_OPT idl2


; Initial declarations
n_data=n_elements(pstamp.image)
if not keyword_set(tol) then if keyword_set(focus) then tol=focus.tol_chisq else tol=sqrt(2./n_data) ; rms for reduced Chi^2
if not keyword_set(chi20) then if keyword_set(focus) then chi20=focus.chisq0 else chi20=1+tol
if keyword_set(append) and not keyword_set(focus) then append=0B ; Nothing to append to!
;message,"Using chi^2 tolerance of "+strtrim(string(tol),2),/info,noprint=1-verbose
message,"Finding n_max to achieve chi^2 of at least "+strtrim(string(chi20),2),/info,noprint=1-verbose


; Determine starting points for iteration
shapelets_guess_nmaxbeta, pstamp, guess, th_min_geom=th_min_geom, X0=decomp.x
if not keyword_set(x0) then if keyword_set(focus) then x0=focus.x0 else x0=guess.x0
if not keyword_set(beta) then if keyword_set(focus) then beta=focus.beta else beta=guess.beta
if not keyword_set(n_max_range) then n_max_range=[2,12]
if max(n_max_range mod 2) eq 1 then message,'Peculiar choice of an odd value in the n_max limits',/info,noprint=1-verbose
if not keyword_set(th_min_geom) then if keyword_set(focus) then th_min_geom=focus.th_min_geom else th_min_geom=0.2
if not keyword_set(append) then flag=0 else flag=focus.flag


; Compute allowed range of n_max given the size of the image and the pixel size
shapelets_geometric_contstraints, pstamp, th_min_geom, th_max, x0=x0
n_max_geom=fix(min([(th_max/beta)^2-1.,(beta/th_min_geom)^2-1.]))
n_max_range2=[n_max_range[0],min([n_max_range[1],n_max_geom])>n_max_range[0]]
if n_max_range2[0] gt 2 then message,"Peculiar starting choice of n_max. There is currently no mechanism to decrease this value!",/info,noprint=1-verbose

; Build internal history structure
history_n_iter=0
history_beta=0.
history_n_max=0
history_chisq=0.
history_x0=fltarr(1,2)


; Initialise some flags to track the progress of our iteration
converged=0B
flat=0B
out=0B
bounce=0B
worsening=0B


; Try several n_max's starting from the lowest allowed value 
n_max=n_max_range2[0]-2
decomp_new=1.
recomp_new=1.
c2=!values.f_infinity
while not converged and not flat and not out and not worsening do begin

  n_max=n_max+2
  beta_min=th_min_geom*sqrt(n_max+1)
  beta_max=th_max/sqrt(n_max+1)
  
  if (n_max gt n_max_range[1]) then begin   ; out of range?
    
    n_max=n_max-2
    bounce=1B
    out=1B
  
  endif else begin
  
    if (n_max gt n_max_range2[1]) or (beta lt beta_min) or (beta gt beta_max) then begin  ; bounce obliquely off th_min_geom/max_geom: nudge beta 
      bounce=1B
      beta=beta_min>beta<beta_max
    endif
    
    decomp_old=decomp_new
    recomp_old=recomp_new
    c2_old=c2
    shapelets_decomp,pstamp.image,beta,n_max,decomp_new,recomp_new,x0=x0,/ls,/over,$
      noise=pstamp.noise,/integrate,psf=psf,sky=sky
    c2=decomp_new.chisq[1]
       

    ; update history file
    history_n_iter=history_n_iter+1
    history_beta=[history_beta,beta]
    history_n_max=[history_n_max,n_max]
    history_chisq=[history_chisq,c2]
    history_x0=[history_x0,transpose(x0)]
    if keyword_set(verbose) then message,"  n_max, beta, chi^2, x_c:"+$
      string(n_max)+string(beta)+string(c2)+string(x0[0])+string(x0[1]),/info


    ; is this a good value? 
    if c2 lt chi20 then begin
      converged=1B
    endif else if c2 ge c2_old then begin ; Are we getting worse (e.g. for small pstamps)
      worsening=1B
    endif else begin
      if n_max ge n_max_range2[0]+2 then begin
        if abs(c2-history_chisq[history_n_iter-1]) lt tol then begin 
          flat=1B
          message,'Chi^2 flatness criterion reached',/info,noprint=1-verbose
        endif
      endif
    endelse

  
  endelse ; out?

endwhile


; decide which of the neighbouring values is optimal
if bounce then flag=flag>1  ; did we bounce off the geometrical constraints at any point?
if out then begin           ; did we reach n_max_geom?

  ; Update flags
  flag=flag>2
  if keyword_set(verbose) then message,'n_max reached maximum',/info,noprint=1-verbose

  ; Are n_max_geom and n_max both even/both odd?
  if ((n_max_range2[1] mod 2) eq (n_max mod 2)) or (n_max eq n_max_range[1]) then begin

    n_max_opt=n_max     ; yes: keep last value tried
    c2_opt=c2
    decomp=decomp_new
    recomp=recomp_new

  endif else begin      ; no: evaluate n_max+1=n_max_geom 
    
    ; Store previous decomposition as we may come back to this
    decomp_old=decomp_new
    recomp_old=recomp_new
    c2_old=c2
    n_max_old=n_max
    n_max=n_max+1
    
    ; Nudge beta by bouncing obliquely off th_min_geom/max_geom if necessary (flag already higher than 1, so no need to update that)
    beta_min=th_min_geom*sqrt(n_max+1)
    beta_max=th_max/sqrt(n_max+1)
    if (beta_min gt beta_min) then c2=!values.f_infinity else begin 
      beta=beta_min>beta<beta_max
      shapelets_decomp,pstamp.image,beta,n_max,decomp_new,recomp_new,x0=x0,/ls,/over,$
        noise=pstamp.noise,/integrate,psf=psf,sky=sky
      c2=decomp_new.chisq[1]
    endelse

    ; Update history list
    history_n_iter=history_n_iter+1
    history_beta=[history_beta,beta]
    history_n_max=[history_n_max,n_max]
    history_chisq=[history_chisq,c2]
    history_x0=[history_x0,transpose(x0)]
    if keyword_set(verbose) then message,"  n_max, beta, chi^2, x_c:"+$
      string(n_max)+string(beta)+string(c2)+string(x0[0])+string(x0[1]),/info
 
    if c2 lt chi20 then begin  ; is new value better than old one?
      c2_opt=c2_old  ; yes: revert to old decomposition
      n_max_opt=n_max_old
      decomp=decomp_old
      recomp=recomp_old
      history_n_iter=history_n_iter+1
      history_beta=[history_beta,beta]
      history_n_max=[history_n_max,n_max-1]
      history_chisq=[history_chisq,c2_opt]
      history_x0=[history_x0,transpose(x0)]
    endif else begin
      c2_opt=c2      ; no: keep new decomposition
      n_max_opt=n_max
      decomp=decomp_new
      recomp=recomp_new
    endelse
    
  endelse
endif else begin    ; Still within range of valid n_max
  if flat and not converged then begin  ; did we stop because of flatness?
    message,'Chi^2 flatness criterion reached',/info,noprint=1-verbose
    flag=flag>3
    c2_opt=c2         ; yes: then keep last value
    n_max_opt=n_max
    decomp=decomp_new
    recomp=recomp_new
  endif else begin    ; no: then pick best value from 3 nearest neighbors
    n_max_mid=n_max-1
    n_max_a=[n_max-2,n_max-1,n_max]
    if n_max-1 lt n_max_range[0] then begin ; bounce off minimum n_max constraint
      bounce=1B
      c2_mid=!values.f_infinity
      flag=flag>1
      recomp_mid=1.
    endif else begin
      shapelets_decomp,pstamp.image,beta,n_max_mid,decomp_mid,recomp_mid,x0=x0,/ls,$
        noise=pstamp.noise,/integrate,psf=psf,sky=sky
      c2_mid=decomp_mid.chisq[1]
    endelse
    message,"  n_max, beta, chi^2, x_c:"+$
      string(n_max_mid)+string(beta)+string(c2_mid)+string(x0[0])+string(x0[1]),/info,noprint=1-verbose
    history_n_iter=history_n_iter+1
    history_beta=[history_beta,beta]
    history_n_max=[history_n_max,n_max_mid]
    history_chisq=[history_chisq,c2_mid]
    history_x0=[history_x0,transpose(x0)]    
    if history_n_iter ge 3 then c2_old=history_chisq[history_n_iter-2] else c2_old=!values.f_infinity
    if n_max_a[0] lt n_max_range[0] then c2_old=!values.f_infinity
    c2_a=[c2_old,c2_mid,c2]
    ;dc2_opt=min(abs(c2_a-chi20),opt)
    dc2_opt=min(abs(c2_a-1),opt)
    n_max_opt=n_max_a[opt]
    c2_opt=c2_a[opt]
    case opt of
      0: begin
           decomp=decomp_old
           recomp=recomp_old
           history_n_iter=history_n_iter+1
           history_beta=[history_beta,beta]
           history_n_max=[history_n_max,n_max_opt]
           history_chisq=[history_chisq,c2_opt]
           history_x0=[history_x0,transpose(x0)]
         end
      1: begin
           decomp=decomp_mid
           recomp=recomp_mid
         end
      2: begin
           decomp=decomp_new
           recomp=recomp_new
           history_n_iter=history_n_iter+1
           history_beta=[history_beta,beta]
           history_n_max=[history_n_max,n_max_opt]
           history_chisq=[history_chisq,c2_opt]
           history_x0=[history_x0,transpose(x0)]
         end
    endcase
  endelse   ; flat?
endelse

if decomp.n_max ne n_max_opt then begin
  message,'Error in decomp coefficient assignement',/info,noprint=1-verbose
  flag=flag>6
endif


; Update focus structure
if not keyword_set(focus) or not keyword_set(append) then begin
  focus={name:pstamp.name,                                      $
         type:"focus",                                          $
         history:"Created during n_max search",                 $
         beta:beta,                                             $
         n_max:n_max_opt,                                       $
         x0:x0,                                                 $
         chisq:decomp.chisq[1],                                 $
         chisq0:chisq0,                                         $
         tol_chisq:tol_chisq,                                   $
         th_min:th_min_geom,                                    $
         n_max_guess:guess.nmax,                                $
         beta_guess:guess.beta,                                 $
         x0_guess:guess.x0,                                     $
         n_iterations:history_n_iter,                           $
         beta_history:history_beta[1:*],                        $
         n_max_history:history_n_max[1:*],                      $
         x0_history:transpose(history_x0[1:*,*]),               $
         chisq_history:history_chisq[1:*],                      $
         flag:flag,                                             $
         flag_interpret:["OK",                                                                           $ ; 0
                         "Bounced off geometrical constraints",                                          $ ; 1
                         "Maximum n_max reached - shapelets may incompletely represent object",          $ ; 2
                         "Converged by flatness limit - shapelets may incompletely represent object",    $ ; 3
                         "Failed to converge",                                                           $ ; 4
                         "Centroid wandered, pushing basis functions off the edge of the postage stamp", $ ; 5
                         "Fatal crash during focus routine"]}                                              ; 6
endif else begin
  focus_new={name:focus.name,type:"focus"}
  names=tag_names(focus)
  for i=0,n_tags(focus)-1 do begin
    case strupcase(names[i]) of
      "NAME":
      "TYPE":
      "BETA":          focus_new=create_struct(focus_new,"beta",beta)
      "N_MAX":         focus_new=create_struct(focus_new,"n_max",n_max_opt)
      "N_ITERATIONS":  focus_new=create_struct(focus_new,"n_iterations",focus.n_iterations+history_n_iter)
      "HISTORY":       focus_new=create_struct(focus_new,"history",[focus.history,"n_max optimised"])
      "BETA_HISTORY":  focus_new=create_struct(focus_new,"beta_history",[focus.beta_history,history_beta[1:*]])
      "N_MAX_HISTORY": focus_new=create_struct(focus_new,"n_max_history",[focus.n_max_history,history_n_max[1:*]])
      "X0_HISTORY":    focus_new=create_struct(focus_new,"x0_history",transpose([transpose(focus.x0_history),history_x0[1:*,*]]))
      "CHISQ_HISTORY": focus_new=create_struct(focus_new,"chisq_history",[focus.chisq_history,history_chisq[1:*]])
      else: focus_new=create_struct(focus_new,names[i],focus.(i))
    endcase
  endfor
  focus=focus_new
endelse

end




