;$Id: shapelets_plot_focus.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;      SHAPELETS_IMAGE_PROFILE
;
; CATEGORY:
;      Subroutine of shapelets_plot_focus.pro.
;
; PURPOSE:
;      Computes the radial profile of an image array.
;
; INPUTS:
;      IM - 2D image array.
;
; OPTIONAL INPUTS:
;      X0 - Object's centre (DEFAULT: centre of the array)
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      PROFILE
;
; MODIFICATION HISTORY:
;      Apr 05 - Modified by Richard Massey.
;      Mar 02 - Modified by AR to be more general 
;      Jul 99 - Written by A. Refregier
;-

function shapelets_image_profile, im, x0=x0

COMPILE_OPT idl2, HIDDEN

; declarations
si=size(im)
n1=si[1] & n2=si[2]
if not keyword_set(x0) then begin    ; set object center
   x0=[n1/2.,n2/2.]             
endif
n_th=15              ; number of theta bins

; define angular bins - linear spacing
th_ran = [0., sqrt(2.)*float(min([n1,n2])/2.)]
dth = (th_ran[1]-th_ran[0])/float(n_th)
th_l = findgen(n_th)*dth+th_ran[0]
th_m = th_l+.5*dth
th_h = th_l+dth

; define angular bins - log spacing
th_ran_log = [1., sqrt(2.)*float(min([n1,n2])/2.)]
dlogth =  (alog10(th_ran_log[1])-alog10(th_ran_log[0]))/float(n_th)
;th_l = findgen(n_th)*dth+th_ran_log[0]
;th_m = th_l+.5*dth
;th_h = th_l+dth
logth_l = findgen(n_th)*dlogth+alog10(th_ran_log[0])
logth_m = logth_l+.5*dlogth 

; define arrays
ith = fltarr(n_th)
ith_log = fltarr(n_th)
nth = lonarr(n_th)
nth_log = lonarr(n_th)
thbin=fltarr(n1,n2)
logthbin=fltarr(n1,n2)
; compute profile
for i=0, n1-1 do begin
   for j=0, n2-1 do begin
      th = sqrt(float(i-x0[0])^2+float(j-x0[1])^2)
      thi = fix((th-th_ran[0])/dth)
      thbin[i,j]=thi
      thi_log = fix((alog10(th)-alog10(th_ran_log[0]))/dlogth)
      logthbin[i,j]=thi_log      
      if thi lt n_th-1 and thi ge 0 then begin
        ith[thi] = ith[thi]+im[i,j]
        nth[thi] = nth[thi]+1l
      endif   
      if thi_log lt n_th-1 and thi_log ge 0 then begin
        ith_log[thi_log] = ith_log[thi_log]+im[i,j]
        nth_log[thi_log] = nth_log[thi_log]+1L
      endif 
     endfor
endfor
gd=where(nth gt 0)
ith = ith[gd]/float(nth[gd])
gd_log=where(nth_log gt 0)
ith_log = ith_log[gd_log]/float(nth_log[gd_log])


; store in structure
profile={r:th_m[gd],f:ith[gd],r_log:10.^(logth_m[gd_log]),$
  f_log:ith_log[gd_log]}
return,profile

end



; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;       SHAPELETS_PLOT_FOCUS
; 
; PURPOSE:
;       Plot objects and their decomposition
;
; CATEGORY:
;       Shapelets/plotting.
;
; INPUTS:
;       PSTAMP   - Postage stamp structure.
;       DECOMP   - Shapelet decomp structure of final decomposition.
;
; OPTIONAL INPUTS:
;       IMAGE_IN - Second input image to plot (eg. noise free simulated image). 
;       SCAT_IN  - Input sextractor catalog to plot input ellipse as well.
;       FOCUS    - Focus history structure.
;       PSF      - PSF for convolution, as a decomp structure type.
;
; KEYWORDS:
;       None.
;
; OUTPUTS:
;       Several plots are drawn to STDOUT.
;
; MODIFICATION HISTORY:
;       Apr 05 - Updated by RM.
;       Jan 05 - idl2 compatability ([]s, etc) implemented by RM.
;       Jan 05 - Color scale ranges normalised, and PSF keyword added by AR.
;       Nov 04 - Window positions generalised to my screen by Richard Massey.
;       Jul 04 - Modified by AR to allow more flexibility in indexing.
;       Aug 03 - Written by Alexandre Refregier.
;-

pro shapelets_plot_focus, PSTAMP, DECOMP,    $
                          PSF=psf,           $
                          IMAGE_IN=image_in, $
                          SCAT_IN=scat_in,   $
                          FOCUS=focus

COMPILE_OPT idl2

; Initial declarations
dist_max=5.               ; Matching radius for input scat comparison [pixels]
; Set up display size for AR's great big monitor
;dsize=[1400,1200]
; Set up display size for RJM's more modest display
;dsize=[1248,1024]
; Set up display size for AR's even larger monitor
;dsize=[1550,1200]
; Let's end this bickering! Just ask IDL for the display's resolution...
device,get_screen_size=dsize
if keyword_set(psf) then n_windows=4 else n_windows=3
wborder=[5,20]            ; Number of pixels used by window manager as border around [L/R/B,T] of windows
csize=0.09+0.03*n_windows ; Fractional size of colour bar
charsize=1.3              ; Size of label text, realtive to IDL default
wsize=[(dsize[0])/n_windows-2*wborder[0],$                   ; Window width 
       (dsize[0])/n_windows*(1+csize)-wborder[0]-wborder[1]] ; Window height
c_ran=[min(pstamp.image),max(pstamp.image)]                  ; Colour scale range
flag_pstamp=['OK','','nearby object','severe overlap','near edge',$
             'FWHM=0','no backgd','object masked out']       ; Short meanings
flag_focus=['OK','!7h!6!imin!n-!7h!6!imax!n bounce','max n!imax!n reached',$
            'converged by flatness','failed to converge',$   ; Short meanings
            'centroid off edge','fatal error']


;
; Plot the original image
;
window,0,title="Original object",$
  xsize=wsize[0],ysize=wsize[1],$
  xpos=0,        ypos=dsize[1]-wsize[1]
shapelets_plot_pstamp,pstamp,title='Original object',/mask,cran=c_ran,csize=csize


;
; Plot the reconstructed image
;
shapelets_recomp,decomp,recomp
window,1,title="Shapelets reconstruction",$
  xsize=wsize[0],            ysize=wsize[1],$
  xpos=wsize[0]+2*wborder[0],ypos=dsize[1]-wsize[1]
shapelets_plot_image,recomp,/fr,/col,title='Reconstructed object',cran=c_ran,csize=csize
plt_ellipse,pstamp.xo,pstamp.yo,$                 ; Sextractor ellipse
    pstamp.a,pstamp.b,(pstamp.theta+90.)/!radeg,lines=2
if keyword_set(focus) then begin ; plot focus history if available
  ;oplot,[focus.x0_guess[0]],[focus.x0_guess[1]],psym=1     ; guess
  oplot,focus.x0_history[0,*],focus.x0_history[1,*]         ; whole history
  oplot,[focus.x0[0]],[focus.x0[1]],psym=4,symsize=sqrt(2.) ; final 
endif
; Plot unweighted ellipticity
q=shapelets_quadrupole(decomp,flux=flux)
a=sqrt((q[0,0]+q[1,1]+sqrt((q[0,0]-q[1,1])^2+4.*q[0,1]^2))/flux)  ; principal axes [pixels]
b=sqrt((q[0,0]+q[1,1]-sqrt((q[0,0]-q[1,1])^2+4.*q[0,1]^2))/flux)
alpha=.5*atan(2.*q[0,1],q[0,0]-q[1,1])   ; position angle, counter-clockwise from x-axis
plt_ellipse,decomp.x[0],decomp.x[1],a,b,alpha-!pi/2.
centroid=shapelets_centroid(decomp)
; Plot centre of light
oplot,[centroid[0]],[centroid[1]],psym=7,symsize=3,color=0


;
; Plot histogram of pixel values
;
window,4,title='Histogram',$
    xsize=wsize[0],ysize=wsize[1]*2/3,$ 
    xpos=0,        ypos=dsize[1]-wsize[1]*5/3-wborder[0]-wborder[1]
!p.region=0
gd=where(pstamp.seg ne -1)
im_ran=c_ran
binsize=(im_ran[1]-im_ran[0])/50
plothist,pstamp.image[gd],bin=binsize,$
   xran=c_ran,/xsty,$
   xtitle='pixel value',ytitle='No of pixels',title='Postage stamp histogram'
plothist,pstamp.image[where(pstamp.seg eq 0)],lines=2,/over,bin=binsize
;plothist,pstamp.image(where(pstamp.seg gt 0)),lines=3,/over,bin=binsize
oplot,pstamp.back_mean_local*[1.,1.],[0.,1e6],lines=2
oplot,pstamp.back_mean_local-pstamp.back_rms_local*[1.,1.],[0.,1e6],lines=2
oplot,pstamp.back_mean_local+pstamp.back_rms_local*[1.,1.],[0.,1e6],lines=2
oplot,pstamp.back_mean_local-pstamp.back_rms_ext*[1.,1.],[0.,1e6],lines=1
oplot,pstamp.back_mean_local+pstamp.back_rms_ext*[1.,1.],[0.,1e6],lines=1
xyouts,.9,.85,/norm,charsize=charsize,align=1,$
        'loc back ave= '+string(pstamp.back_mean_local,format='(e8.2)')+'!c'+$
        'loc back rms= '+string(pstamp.back_rms_local,format='(e8.2)')+'!c'+$
        'ext back rms= '+string(pstamp.back_rms_ext,format='(e8.2)')


;
; Plot focus history if available
;
if keyword_set(focus) then begin
  case n_windows of
    3: xsize=wsize[0]
    4: xsize=2*wsize[0]+2*wborder[0]
  endcase
  window,3,title="Focus history",$
    xsize=xsize,                   ysize=wsize[1]*2/3,$
    xpos=2*(wsize[0]+2*wborder[0]),ypos=dsize[1]-wsize[1]*5/3-wborder[0]-wborder[1]
  th_max=focus.beta*sqrt(focus.n_max+1.)
  th_min=focus.beta/sqrt(focus.n_max+1.)
  th_maxmax=abs(min([focus.x0[0],$  ; imposed by image boundaries
       pstamp.im_ran[1]-pstamp.im_ran[0]-focus.x0[0],$ 
       focus.x0[1],pstamp.im_ran[3]-pstamp.im_ran[2]-focus.x0[1]]))
  beta_ran=[0.,th_maxmax]
  n_max_ran=[0,max(focus.n_max_history)+2]
  !p.region=0
  plot,[0,0],/nodata,yran=beta_ran,xran=n_max_ran,$
    xtitle='!6n!dmax!n',ytitle='!7b!6',title='Focus history'
  if focus.n_iterations gt 1 then begin
    oplot,focus.n_max_history,focus.beta_history
    oplot,focus.n_max_history,focus.beta_history,psym=1
  endif
  usersym, cos(2*!pi*findgen(21)/20), sin(2*!pi*findgen(21)/20)
  oplot,[focus.n_max_guess<(n_max_ran[1])],[focus.beta_guess],psym=8,symsize=3
  oplot,[focus.n_max],[focus.beta],psym=7,symsize=3
  flux=shapelets_flux(decomp,/error)
  snr=flux[0]/flux[1]
  xyouts,.9,.85,/normal,charsize=charsize,align=1,$
       '!7b!6='+string(focus.beta,format='(f6.2)')+', '+$
       'n!imax!n='+strtrim(string(focus.n_max),2)+'!c'+$
       '!7v!6!e2!n='+strtrim(string(focus.chisq),2)+', '+$
       'snr='+strtrim(string(snr),2)+'!c'+$
       'pstamp: '+string(flag_pstamp[pstamp.flag])+'!c'+$
       'focus: '+focus.flag_interpret[focus.flag]
  ; plot geometrical constraints
  nn_max=xgen(0.,n_max_ran[1]+4)
  oplot,nn_max,focus.th_min*sqrt(nn_max+1.),lines=3
  oplot,nn_max,th_maxmax/sqrt(nn_max+1.),lines=3
endif


;
; Plot reconvolved image if deconvolution was applied
;
if keyword_set(psf) then begin
  decomp_r=shapelets_convolve(decomp,psf)
  decomp_r.x=decomp.x
  decomp_r.n_pixels=decomp.n_pixels
  shapelets_recomp,decomp_r,recomp_psf
  window,7,title="Reconvolved reconstruction",$
    xsize=wsize[0],                ysize=wsize[1],$
    xpos=3*(wsize[0]+2*wborder[0]),ypos=dsize[1]-wsize[1]
  shapelets_plot_image,recomp_psf,/fr,/col,title='Reconvolved Reconstruction',cran=c_ran,csize=csize
  plt_ellipse,pstamp.xo,pstamp.yo,$                 ; Sextractor ellipse
      pstamp.a,pstamp.b,(pstamp.theta+90.)/!radeg,lines=2
endif



;
; Plot object profile
;
prof=shapelets_image_profile(pstamp.image,x0=[pstamp.XO,pstamp.YO])
window,6,title='Profile',$
    xsize=wsize[0],            ysize=wsize[1]*2/3,$
    xpos=wsize[0]+2*wborder[0],ypos=dsize[1]-wsize[1]*5/3-wborder[0]-wborder[1]
!p.region=0
r=xgen(0.,max(prof.r))
fsh_r=shapelets_profile(decomp,r)
profile_ylog=1B
if profile_ylog then begin
  plot,prof.r,prof.f,yran=[max(fsh_r)*1e-4,max(fsh_r)],/ystyle,psym=6,$
    xtitle='r (pixels)',ytitle='I(r)',title='Profile',/ylog
endif else begin
  plot,prof.r,prof.f,yran=[c_ran[0],max(fsh_r)],/ystyle,psym=6,$
    xtitle='r (pixels)',ytitle='I(r)',title='Profile'
endelse
oplot,[0,1000],pstamp.back_mean_local*[1.,1.],lines=2
oplot,[0,1000],pstamp.back_mean_local-pstamp.back_rms_local*[1.,1.],lines=1
oplot,[0,1000],pstamp.back_mean_local+pstamp.back_rms_local*[1.,1.],lines=1
oplot,r,fsh_r,lines=0
if keyword_set(image_in) then begin   ; plot input porfile if available
  bit_ref=image_in.image[pstamp.im_ran[0]:pstamp.im_ran[1],$
                       pstamp.im_ran[2]:pstamp.im_ran[3]] 
  ;cmp_mom,bit_ref,mom
  mom=shapelets_image_moments(bit_ref)
  prof_in=shapelets_image_profile(bit_ref,x0=mom.xc)
  oplot,prof_in.r,prof_in.f,psym=1
endif
if keyword_set(psf) then begin
   oplot,r,shapelets_profile(decomp_r,r),lines=3
   oplot,r,shapelets_profile(psf,r)*$
         (shapelets_profile(decomp_r,0))[0]/(shapelets_profile(psf,0))[0],lines=4
endif



;
; Plot residual map
;
if keyword_set(psf) then residual=pstamp.image-recomp_psf $
    else residual=pstamp.image-recomp
window,2,title='Residuals',$
  xsize=wsize[0],                ysize=wsize[1],$
  xpos=2*(wsize[0]+2*wborder[0]),ypos=dsize[1]-wsize[1]
shapelets_plot_image,residual,/fr,/col,title='Residuals',csize=csize



;
; Plot input image if available
;
if keyword_set(image_in) then begin
  
  ; plot input image
  window,5,title='Input image',$
    xsize=wsize[0],                ysize=wsize[1],$
    xpos=2*(wsize[0]+2*wborder[0]),ypos=dsize[1]-2*wsize[1]-wborder[0]-wborder[1]
  bit_ref=image_in.image[pstamp.im_ran[0]:pstamp.im_ran[1],$
                       pstamp.im_ran[2]:pstamp.im_ran[3]]
  shapelets_plot_image,bit_ref,cran=c_ran,/frame,/col,csize=csize,$
    title='Input image'
  plt_ellipse,decomp.x[0],decomp.x[1],dmom.a,dmom.b,dmom.alpha-!pi/2.
  
  ; compute image moments after removing neighboring sources
  othpix= where((pstamp.seg ne 0) and (pstamp.seg ne pstamp.sexid),nothpix)  
  if nothpix gt 0 then  bit_ref[othpix]=0.
  ;cmp_mom,bit_ref,mom                  ; Alexandre's old version
  mom=shapelets_image_moments(bit_ref)  ; Richard's standardised routine name
  plt_ellipse,mom.xc[0],mom.xc[1],mom.a,mom.b,mom.alpha-!pi/2.,lines=1
  oplot,[mom.xc[0]],[mom.xc[1]],psym=1
  
  ; plot input parameter ellipse if input scat catalog is provided
  if keyword_set(scat_in) then begin
    centroid=shapelets_centroid(decomp)+[pstamp.im_ran[0],pstamp.im_ran[2]]
    dist_in=sqrt((scat_in.x[*,0]-centroid[0])^2+$
               (scat_in.x[*,1]-centroid[1])^2)
    dist=min(dist_in,match)
    plt_ellipse,scat_in.x[match,0]-pstamp.im_ran[0],$
              scat_in.x[match,1]-pstamp.im_ran[2],$
              scat_in.a[match]*2.,scat_in.b[match]*2.,$
              scat_in.theta[match]/!radeg-!pi/2.,lines=3
  endif
endif 

end
