pro shapelets_plot_image, image,	      $
                          FRAME=frame,        $
                          COLBAR=colbar,      $
                          CBAR=cbar,	      $
                          CRANGE=crange,      $
                          CLOG=clog,	      $
                          TITLE=title,        $
                          XTITLE=xtitle,      $
                          YTITLE=ytitle,      $
                          CTITLE=ctitle,      $
                          INVERSE=inverse,    $
                          SCALABLE=scalable,  $
                          NOERASE=noerase,    $
                          CSIZE=csize,        $
                          ISOTROPIC=isotropic

;$Id: shapelets_plot_image.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_PLOT_IMAGE
;
; PURPOSE:
;      Draws a 2D pixellated image.
;
; CATEGORY:
;      Plotting.
;
; PURPOSE:
;      Plot an image scaled to the current coordinate frame.
;      Further plotting (such as contours) can be performed over the
;      resulting plot. Optionally, an annotated color bar can be drawn at
;      the top. The 'scalable' keyword must be invoked when outputing to a
;      postscript file.
;
; NOTES:
;      !p.region=0 can be used to reset the window after using this
;      routine
;
; INPUTS:
;      IMAGE    - shapelets image structre OR a 2D image array
;
; OPTIONAL INPUTS:
;      CRANGE	- range of a for colors (default: [min(a),max(a)])
;      TITLE	- title for the plot
;      X,YTITLE - titles for the frame
;      CTITLE	- title for the color bar
;      CSIZE	- vertical size of the color bar (0-1, default:.12)
;
; KEYWORD PARAMETERS:
;      NOERASE  - supress erasing whatever was previously in the window.
;      CBAR	- draw color bar. Also works with colbar.
;      FRAME	- draw a frame with pixel index limits;
;       	  to change limits set frame=[x0,x1,y0,y1].
;       	  set frame=-1 to supress the frame while keeping
;       	  the image within it.
;      INVERSE  - invert the color coding.
;      SCALABLE - use scalable pixels.
;       	  DEFAULT: nonscalable used with xterm or win device, scalable
;       	  used with postscript device.
;      ISOTROPIC- force scales on x and y axes to be identical.
;      CLOG	- make colour scale logarithmic.
;
; OUTPUTS:
;      Plot of the scaled image with, optionally, a coordinate frame
;      and a color bar.
;
; MODIFICATION HISTORY:
;      Apr 05 - Plotting of 1D arrays and logarithmic color scale added by RM.
;      Mar 05 - Input generalised to include shapelet image structures by RM.
;      Jul 04 - Automatic detection of device type (and adoption of scalable
;               pixels where relevant) by RM.
;      Nov 02 - Isotropic keyword added by Richard Massey. Still tempreamental!
;      Aug 00 - Plotting over the full plotting region allowed by AR.
;    3 Sep 97 - Active changes for the color bar rangeadded by AR.
;   14 Oct 97 - Plotting of a color bar and flexible frame labels added by AR.
;    7 Jul 97 - Written by Alexandre Refregier.
;-

; Parse input array
if shapelets_structure_type(image,message=message,/silent) then begin
  if image.type eq"image" then a=float(image.image) else message,"You need to input an image!"
endif else begin
  a=float(image)
  n_dimensions=size(a,/n_dim)
  ; Hack to get around the fact that tvscl can't handle 1D arrays direction
  if n_dimensions eq 1 then a=reform(a,n_elements(a),1)#[1,1]
  if n_dimensions gt 2 then message,"Input variable is "+strtrim(string(n_dimensions),2)+"D!"
endelse
if n_elements(uniq(a)) eq 1 then message,"WARNING: All pixels in the image have the same value!",/info

; Remember how graphics device was initially set up
pregion=!p.region

; Figure out what kind of plot we are making
; (if we're plotting to postscript, the pixels should be scalable)
if n_elements(scalable) eq 0 then $
  if strupcase(!d.name) eq "PS" then scalable=1 else scalable=0

; Convert image to a logarithmic colour scale
if keyword_set(clog) then begin
  orders_of_magnitude=1./10^(2>clog)
  a=alog10(abs(a-min(a)+orders_of_magnitude*max(a)))
endif

; Set colour range
if not keyword_set(crange) then crange=[min(a),max(a)]

; Draw color bar
if keyword_set(colbar) or keyword_set(cbar) then begin
  shapelets_plot_colourbar,crange,clog=clog,scalable=scalable,inverse=inverse,title=ctitle,$
    csize=csize
  noerase=1
endif

; Set up the coordinate frame
nx=n_elements(a[*,0])
ny=n_elements(a[0,*])
plot,[0],[0],/nodata,xstyle=12,ystyle=12,noerase=noerase

; Map image to color indices
if not keyword_set(crange) then begin
  if keyword_set(inverse) then b=-a else b=a
  b_ran=[min(b),max(b)]
  b=(b-b_ran[0])/(b_ran[1]-b_ran[0])*(!d.table_size-1)    ; this is equivalent to tvscl
endif else begin
  if keyword_set(inverse) then begin    
    b=(1.-((a>crange[0]<crange[1])-crange[0])/(crange[1]-crange[0]))*(!d.table_size-1)
  endif else begin
    b=((a>crange[0]<crange[1])-crange[0])/(crange[1]-crange[0])*(!d.table_size-1)
  endelse
endelse

; Plot image scaled to the frame size
if not keyword_set(scalable) then begin
; non-scalable pixel case (to be used for xterm device)
  if keyword_set(frame) then begin
    px=!x.window*!d.x_vsize     ; position of window in device pixels
    py=!y.window*!d.y_vsize
  endif else begin
    px=!x.region*!d.x_vsize     ; position of window in device pixels
    py=!y.region*!d.y_vsize    
  endelse
  sx=abs(px[1] - px[0] +1)      ; desired size of image in pixels
  sy=abs(py[1] - py[0] +1)
  if keyword_set(isotropic) then begin
    message,"WARNING: Isotropic plotting still temperamental!",/info
    smin=sx<sy
    nmax=float(nx>ny)
    !p.region=[ !p.region[0], $
	  	      !p.region[1], $
  	           !p.region[0]+(!p.region[2]-!p.region[0])*nx/nmax, $
	  	      !p.region[1]+(!p.region[3]-!p.region[1])*ny/nmax ]
    sx=smin*(nx/nmax)
    sy=smin*(ny/nmax)
  endif
  tv,congrid(b,sx,sy),px[0],py[0]
endif else begin
; scalable pixel case (to be used for ps device)
  if keyword_set(isotropic) then message,"WARNING: Isotropic plotting not yet implemented for scalable pixels!",/info
  if keyword_set(frame) then begin  
    tv,b,$
      !x.window[0],!y.window[0],$
      xsize=!x.window[1]-!x.window[0],$
      ysize=!y.window[1]-!y.window[0],/norm
  endif else begin
    tv,b,$
      !x.region[0],!y.region[0],$
      xsize=!x.region[1]-!x.region[0],$
      ysize=!y.region[1]-!y.region[0],/norm 
  endelse   
endelse	

; draw frame if requested
if keyword_set(frame) then begin
  fsupress=0
  if n_elements(frame) eq 4 then begin
    xr=[frame[0],frame[1]] & yr=[frame[2],frame[3]]
  endif else begin
    if frame eq -1 then begin
      fsupress=1 
    endif else begin
      xr=[0.,nx] & yr=[0.,ny]
    endelse
  endelse
  if fsupress eq 0 then $
    plot,[0],[0],/nodata,/noerase,/xstyle,/ystyle,$
      xrange=xr,yrange=yr,$
      title=title,xtitle=xtitle,ytitle=ytitle
endif

; Return graphics device to initial set up
if not keyword_set(isotropic) then !p.region=pregion

end


