function shapelets_polar_matrix, n_max,              $
                                 CALCULATE=calculate,$
                                 C2P=c2p,            $
                                 P2C=p2c,            $
                                 INVERSE=inverse,    $
                                 N1=n1,              $
                                 N2=n2

;$Id: shapelets_c2p.pro, v2$
;
; Copyright  2005 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public Licence as published
; by the Free Software Foundation; either version 2 of the Licence, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public Licence for more details.
;
; You should have received a copy of the GNU General Public Licence
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
;
;+
; NAME:
;      SHAPELETS_POLAR_MATRIX
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Calculates a matrix to convert Cartesian shapelet coefficients to their
;      equivalent polar shapelet representation.
;
; INPUTS:
;       N_MAX     - The maximum order used in the shapelet expansion.
;
; OPTIONAL INPUTS:
;       N1 and N2 - The shapelet coefficients are rearranged and stored in a
;                   1D array, even if the basis functions are 2D. This 
;                   simplifies many tasks, including the conversion from
;                   Cartesian to polar shapelets, which then has a 2D matrix.
;                   These vectors specify the order in which the shapelet 
;                   coefficients are stored. For polar shapelets, we assume the
;                   convention nl=n2 and nr=n1. If this is not set, the default
;                   order is assumed. Note that this is much faster, because 
;                   the matrix can then be read from a pre-calculated look-up
;                   table.
;
; KEYWORD PARAMETERS:
;       CALCULATE - By default, the matrix is just read in from a 
;                   pre-calculated look-up table. Setting this option forces 
;                   the routine to calculate the matrix anew.
;       C2P       - Calculate the matrix to convert from Cartesian to polar
;                   shapelet representation. This is the default option.
;       P2C       - Calculate the matrix to convert from polar to Cartesian
;                   shapelet representation.
;       INVERT    - A synonym for P2C (works interchangeably).
;
; OUTPUTS:
;       MATRIX    - The requested conversion matrix. It is independent of beta  
;                   etc, so once it has been calculated, simply use e.g.
;                   polar_coeffs=matrix#cartesian_coeffs
;
; NOTES:
;      A vector of polar shapelet coefficients may be indexed using:
;        shapelets_make_nvec,n_max,n_r,n_l,n_coeffs
;        n=n_r+n_l
;        m=n_r-n_l
;      or
;        shapelets_make_nvec,n_max,n,m,n_coeffs,/POLAR
;
; MODIFICATION HISTORY:
;      Jul 05 - Noticed should be using la_invert to invert complex matrices.
;      Apr 05 - Faster calculation & look-up tables (using invert) added by RM.
;      Feb 02 - Written by Richard Massey.
;-

COMPILE_OPT idl2

; Work out what n_max was and set up index variables
if keyword_set(n1) and keyword_set(n2) then begin
  n_max=max(n1+n2)
  n_coeffs=n_elements(n1)<n_elements(n2)
  calculate=1B
endif else begin
  shapelets_make_nvec, n_max, n1, n2, n_coeffs   
endelse

; Decide whether we want the forward or the reverse transformation
if keyword_set(c2p) or (not keyword_set(p2c) and not keyword_set(inverse)) then begin
  ; Create matrix to convert Cartesian shapelets into polar shapelets

  ; Initialise empty array
  matrix=complexarr(n_coeffs,n_coeffs)
  
  if n_max gt 20 or keyword_set(calculate) then begin
    ; Old code to do calculate the matrix. It explicitly follows the notation in 
    ; published papers, and is very clear. However, its nested for loops are slow.
    ;for i=0,n_coeffs-1 do begin    ; Cartesian
    ;  for j=0,n_coeffs-1 do begin  ; Polar
    ;
    ;    ; Get into form of eqn(69) in Paper I
    ;    n_r=n1[j]
    ;    n_l=n2[j]
    ;    n_1=n1[i]
    ;    n_2=n2[i]
    ;
    ;    if n_1+n_2 eq n_r+n_l then begin
    ;
    ;      factor=complex(0.,0.)
    ;      good=0
    ;      for n_rd=0,n_r do begin
    ;        for n_ld=0,n_l do begin
    ;          if n_rd+n_ld eq n_1 then begin
    ;            good=good+1
    ;            factor = factor                $
    ;              + complex(0.,1.)^(n_ld-n_rd) $
    ;              * comb(n_r,n_rd)             $
    ;              * comb(n_l,n_ld)
    ;          endif
    ;        endfor
    ;      endfor
    ;
    ;      matrix[j,i] =               $
    ;          2.^(-0.5*(n_r+n_l))     $
    ;        * complex(0,1.)^(n_r-n_l)  $
    ;        * sqrt( factorial(n_1)    $
    ;             *  factorial(n_2)    $
    ;             /  factorial(n_r)    $
    ;             /  factorial(n_l)  ) $
    ;        * factor
    ;
    ;    endif
    ;
    ;  endfor
    ;endfor  
    
    ; Faster new alternative
    for i=0,n_coeffs-1 do begin    ; Cartesian shapelet coefficients
      ; Only states with the same n get mixed
      same_n=where(n1+n2 eq n1[i]+n2[i],n_same_n)
      for j=0,n_same_n-1 do begin  ; Polar shapelets coefficients
        ; Get into form of eqn(69) in Paper I
        n_r=n1[same_n[j]]
        n_l=n2[same_n[j]]
        n_1=n1[i]
        n_2=n2[i]
        n_ld=replicate(1,n_r+1)#indgen(n_l+1)
        n_rd=indgen(n_r+1)#replicate(1,n_l+1)
        nonzero=where(n_rd+n_ld eq n_1,n_nonzero)
        if n_nonzero gt 0 then $
        matrix[same_n[j],i] = 2.^(-0.5*(n_r+n_l))         $
                                * complex(0,1)^(n_r-n_l)  $
                                * sqrt( factorial(n_1)    $
                                     *  factorial(n_2)    $
                                     /  factorial(n_r)    $
                                     /  factorial(n_l)  ) $
                                * total(complex(0.,1.)^(n_ld[nonzero]-n_rd[nonzero]) $
                                     * comb(n_r,n_rd[nonzero]) * comb(n_l,n_ld[nonzero]) )
      endfor
    endfor
  
  endif else begin
    ; Use look-up table
    switch n_max of
      
      20: begin
            matrix[210:230,210:230]=[$
            [complex(+9.765625E-04,+0.000000E+00),complex(-4.367320E-03,-0.000000E+00),complex(+1.346099E-02,+0.000000E+00),complex(-3.297254E-02,-0.000000E+00),complex(+6.797464E-02,+0.000000E+00),complex(-1.215967E-01,-0.000000E+00),complex(+1.922613E-01,+0.000000E+00),complex(-2.718986E-01,-0.000000E+00),complex(+3.466040E-01,+0.000000E+00),complex(-4.002239E-01,-0.000000E+00),complex(+4.197583E-01,+0.000000E+00),complex(-4.002239E-01,+0.000000E+00),complex(+3.466040E-01,+0.000000E+00),complex(-2.718986E-01,+0.000000E+00),complex(+1.922613E-01,+0.000000E+00),complex(-1.215967E-01,+0.000000E+00),complex(+6.797464E-02,+0.000000E+00),complex(-3.297254E-02,+0.000000E+00),complex(+1.346099E-02,+0.000000E+00),complex(-4.367320E-03,+0.000000E+00),complex(+9.765625E-04,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.367320E-03),complex(+0.000000E+00,-1.757812E-02),complex(+0.000000E+00,+4.815948E-02),complex(+0.000000E+00,-1.032204E-01),complex(+0.000000E+00,+1.823951E-01),complex(+0.000000E+00,-2.718986E-01),complex(+0.000000E+00,+3.439275E-01),complex(+0.000000E+00,-3.647902E-01),complex(+0.000000E+00,+3.100121E-01),complex(+0.000000E+00,-1.789856E-01),0,complex(+0.000000E+00,+1.789856E-01),complex(+0.000000E+00,-3.100121E-01),complex(+0.000000E+00,+3.647902E-01),complex(+0.000000E+00,-3.439275E-01),complex(+0.000000E+00,+2.718986E-01),complex(+0.000000E+00,-1.823951E-01),complex(+0.000000E+00,+1.032204E-01),complex(+0.000000E+00,-4.815948E-02),complex(+0.000000E+00,+1.757812E-02),complex(+0.000000E+00,-4.367320E-03)],$
            [complex(-1.346099E-02,+0.000000E+00),complex(+4.815948E-02,+0.000000E+00),complex(-1.152344E-01,+0.000000E+00),complex(+2.105030E-01,+0.000000E+00),complex(-3.057467E-01,+0.000000E+00),complex(+3.528622E-01,+0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(+1.578048E-01,+0.000000E+00),complex(+5.029060E-02,+0.000000E+00),complex(-2.322823E-01,-0.000000E+00),complex(+3.045247E-01,+0.000000E+00),complex(-2.322823E-01,+0.000000E+00),complex(+5.029060E-02,+0.000000E+00),complex(+1.578048E-01,-0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(+3.528622E-01,-0.000000E+00),complex(-3.057467E-01,+0.000000E+00),complex(+2.105030E-01,-0.000000E+00),complex(-1.152344E-01,+0.000000E+00),complex(+4.815948E-02,-0.000000E+00),complex(-1.346099E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.297254E-02),complex(-0.000000E+00,+1.032204E-01),complex(+0.000000E+00,-2.105030E-01),complex(-0.000000E+00,+3.144531E-01),complex(+0.000000E+00,-3.462765E-01),complex(-0.000000E+00,+2.520969E-01),complex(+0.000000E+00,-4.555431E-02),complex(+0.000000E+00,-1.771647E-01),complex(+0.000000E+00,+2.874347E-01),complex(+0.000000E+00,-2.133649E-01),0,complex(+0.000000E+00,+2.133649E-01),complex(+0.000000E+00,-2.874347E-01),complex(+0.000000E+00,+1.771647E-01),complex(+0.000000E+00,+4.555431E-02),complex(-0.000000E+00,-2.520969E-01),complex(+0.000000E+00,+3.462765E-01),complex(-0.000000E+00,-3.144531E-01),complex(+0.000000E+00,+2.105030E-01),complex(-0.000000E+00,-1.032204E-01),complex(+0.000000E+00,+3.297254E-02)],$
            [complex(+6.797464E-02,+0.000000E+00),complex(-1.823951E-01,-0.000000E+00),complex(+3.057467E-01,+0.000000E+00),complex(-3.462765E-01,-0.000000E+00),complex(+2.314453E-01,+0.000000E+00),complex(+8.734641E-03,+0.000000E+00),complex(-2.292573E-01,+0.000000E+00),complex(+2.695312E-01,+0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(-1.552458E-01,-0.000000E+00),complex(+2.713719E-01,+0.000000E+00),complex(-1.552458E-01,+0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(+2.695312E-01,-0.000000E+00),complex(-2.292573E-01,+0.000000E+00),complex(+8.734641E-03,-0.000000E+00),complex(+2.314453E-01,+0.000000E+00),complex(-3.462765E-01,+0.000000E+00),complex(+3.057467E-01,+0.000000E+00),complex(-1.823951E-01,+0.000000E+00),complex(+6.797464E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.215967E-01),complex(+0.000000E+00,-2.718986E-01),complex(+0.000000E+00,+3.528622E-01),complex(+0.000000E+00,-2.520969E-01),complex(+0.000000E+00,-8.734641E-03),complex(-0.000000E+00,+2.421875E-01),complex(+0.000000E+00,-2.470529E-01),complex(-0.000000E+00,+1.746928E-02),complex(+0.000000E+00,+2.226905E-01),complex(+0.000000E+00,-2.314268E-01),0,complex(+0.000000E+00,+2.314268E-01),complex(+0.000000E+00,-2.226905E-01),complex(-0.000000E+00,-1.746928E-02),complex(+0.000000E+00,+2.470529E-01),complex(-0.000000E+00,-2.421875E-01),complex(+0.000000E+00,+8.734641E-03),complex(+0.000000E+00,+2.520969E-01),complex(+0.000000E+00,-3.528622E-01),complex(+0.000000E+00,+2.718986E-01),complex(+0.000000E+00,-1.215967E-01)],$
            [complex(-1.922613E-01,+0.000000E+00),complex(+3.439275E-01,+0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(+4.555431E-02,+0.000000E+00),complex(+2.292573E-01,+0.000000E+00),complex(-2.470529E-01,-0.000000E+00),complex(-7.812500E-03,+0.000000E+00),complex(+2.430680E-01,+0.000000E+00),complex(-1.830944E-01,+0.000000E+00),complex(-9.757809E-02,-0.000000E+00),complex(+2.558519E-01,+0.000000E+00),complex(-9.757809E-02,+0.000000E+00),complex(-1.830944E-01,+0.000000E+00),complex(+2.430680E-01,-0.000000E+00),complex(-7.812500E-03,+0.000000E+00),complex(-2.470529E-01,+0.000000E+00),complex(+2.292573E-01,+0.000000E+00),complex(+4.555431E-02,-0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(+3.439275E-01,-0.000000E+00),complex(-1.922613E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.718986E-01),complex(-0.000000E+00,+3.647902E-01),complex(+0.000000E+00,-1.578048E-01),complex(+0.000000E+00,-1.771647E-01),complex(+0.000000E+00,+2.695312E-01),complex(+0.000000E+00,-1.746928E-02),complex(+0.000000E+00,-2.430680E-01),complex(-0.000000E+00,+1.640625E-01),complex(+0.000000E+00,+1.394263E-01),complex(+0.000000E+00,-2.414935E-01),0,complex(+0.000000E+00,+2.414935E-01),complex(+0.000000E+00,-1.394263E-01),complex(-0.000000E+00,-1.640625E-01),complex(+0.000000E+00,+2.430680E-01),complex(+0.000000E+00,+1.746928E-02),complex(+0.000000E+00,-2.695312E-01),complex(+0.000000E+00,+1.771647E-01),complex(+0.000000E+00,+1.578048E-01),complex(-0.000000E+00,-3.647902E-01),complex(+0.000000E+00,+2.718986E-01)],$
            [complex(+3.466040E-01,+0.000000E+00),complex(-3.100121E-01,-0.000000E+00),complex(-5.029060E-02,+0.000000E+00),complex(+2.874347E-01,+0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(-2.226905E-01,-0.000000E+00),complex(+1.830944E-01,+0.000000E+00),complex(+1.394263E-01,+0.000000E+00),complex(-2.324219E-01,+0.000000E+00),complex(-4.736076E-02,-0.000000E+00),complex(+2.483619E-01,+0.000000E+00),complex(-4.736076E-02,+0.000000E+00),complex(-2.324219E-01,+0.000000E+00),complex(+1.394263E-01,-0.000000E+00),complex(+1.830944E-01,+0.000000E+00),complex(-2.226905E-01,+0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(+2.874347E-01,-0.000000E+00),complex(-5.029060E-02,+0.000000E+00),complex(-3.100121E-01,+0.000000E+00),complex(+3.466040E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.002239E-01),complex(+0.000000E+00,-1.789856E-01),complex(+0.000000E+00,-2.322823E-01),complex(-0.000000E+00,+2.133649E-01),complex(+0.000000E+00,+1.552458E-01),complex(+0.000000E+00,-2.314268E-01),complex(+0.000000E+00,-9.757809E-02),complex(-0.000000E+00,+2.414935E-01),complex(+0.000000E+00,+4.736076E-02),complex(+0.000000E+00,-2.460938E-01),0,complex(+0.000000E+00,+2.460938E-01),complex(+0.000000E+00,-4.736076E-02),complex(-0.000000E+00,-2.414935E-01),complex(+0.000000E+00,+9.757809E-02),complex(+0.000000E+00,+2.314268E-01),complex(+0.000000E+00,-1.552458E-01),complex(-0.000000E+00,-2.133649E-01),complex(+0.000000E+00,+2.322823E-01),complex(+0.000000E+00,+1.789856E-01),complex(+0.000000E+00,-4.002239E-01)],$
            [complex(-4.197583E-01,+0.000000E+00),0,complex(+3.045247E-01,+0.000000E+00),0,complex(-2.713719E-01,+0.000000E+00),0,complex(+2.558519E-01,+0.000000E+00),0,complex(-2.483619E-01,+0.000000E+00),0,complex(+2.460938E-01,+0.000000E+00),0,complex(-2.483619E-01,+0.000000E+00),0,complex(+2.558519E-01,+0.000000E+00),0,complex(-2.713719E-01,+0.000000E+00),0,complex(+3.045247E-01,+0.000000E+00),0,complex(-4.197583E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.002239E-01),complex(+0.000000E+00,-1.789856E-01),complex(+0.000000E+00,+2.322823E-01),complex(-0.000000E+00,+2.133649E-01),complex(+0.000000E+00,-1.552458E-01),complex(+0.000000E+00,-2.314268E-01),complex(+0.000000E+00,+9.757809E-02),complex(-0.000000E+00,+2.414935E-01),complex(+0.000000E+00,-4.736076E-02),complex(+0.000000E+00,-2.460938E-01),0,complex(+0.000000E+00,+2.460938E-01),complex(+0.000000E+00,+4.736076E-02),complex(-0.000000E+00,-2.414935E-01),complex(+0.000000E+00,-9.757809E-02),complex(+0.000000E+00,+2.314268E-01),complex(+0.000000E+00,+1.552458E-01),complex(-0.000000E+00,-2.133649E-01),complex(+0.000000E+00,-2.322823E-01),complex(+0.000000E+00,+1.789856E-01),complex(+0.000000E+00,+4.002239E-01)],$
            [complex(+3.466040E-01,+0.000000E+00),complex(+3.100121E-01,+0.000000E+00),complex(-5.029060E-02,+0.000000E+00),complex(-2.874347E-01,-0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(+2.226905E-01,+0.000000E+00),complex(+1.830944E-01,+0.000000E+00),complex(-1.394263E-01,-0.000000E+00),complex(-2.324219E-01,+0.000000E+00),complex(+4.736076E-02,+0.000000E+00),complex(+2.483619E-01,+0.000000E+00),complex(+4.736076E-02,-0.000000E+00),complex(-2.324219E-01,+0.000000E+00),complex(-1.394263E-01,+0.000000E+00),complex(+1.830944E-01,+0.000000E+00),complex(+2.226905E-01,-0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(-2.874347E-01,+0.000000E+00),complex(-5.029060E-02,+0.000000E+00),complex(+3.100121E-01,-0.000000E+00),complex(+3.466040E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.718986E-01),complex(-0.000000E+00,+3.647902E-01),complex(+0.000000E+00,+1.578048E-01),complex(+0.000000E+00,-1.771647E-01),complex(+0.000000E+00,-2.695312E-01),complex(+0.000000E+00,-1.746928E-02),complex(+0.000000E+00,+2.430680E-01),complex(-0.000000E+00,+1.640625E-01),complex(+0.000000E+00,-1.394263E-01),complex(+0.000000E+00,-2.414935E-01),0,complex(+0.000000E+00,+2.414935E-01),complex(+0.000000E+00,+1.394263E-01),complex(-0.000000E+00,-1.640625E-01),complex(+0.000000E+00,-2.430680E-01),complex(+0.000000E+00,+1.746928E-02),complex(+0.000000E+00,+2.695312E-01),complex(+0.000000E+00,+1.771647E-01),complex(+0.000000E+00,-1.578048E-01),complex(-0.000000E+00,-3.647902E-01),complex(+0.000000E+00,-2.718986E-01)],$
            [complex(-1.922613E-01,+0.000000E+00),complex(-3.439275E-01,-0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(-4.555431E-02,-0.000000E+00),complex(+2.292573E-01,+0.000000E+00),complex(+2.470529E-01,+0.000000E+00),complex(-7.812500E-03,+0.000000E+00),complex(-2.430680E-01,-0.000000E+00),complex(-1.830944E-01,+0.000000E+00),complex(+9.757809E-02,+0.000000E+00),complex(+2.558519E-01,+0.000000E+00),complex(+9.757809E-02,-0.000000E+00),complex(-1.830944E-01,+0.000000E+00),complex(-2.430680E-01,+0.000000E+00),complex(-7.812500E-03,+0.000000E+00),complex(+2.470529E-01,-0.000000E+00),complex(+2.292573E-01,+0.000000E+00),complex(-4.555431E-02,+0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(-3.439275E-01,+0.000000E+00),complex(-1.922613E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.215967E-01),complex(+0.000000E+00,-2.718986E-01),complex(+0.000000E+00,-3.528622E-01),complex(+0.000000E+00,-2.520969E-01),complex(+0.000000E+00,+8.734641E-03),complex(-0.000000E+00,+2.421875E-01),complex(+0.000000E+00,+2.470529E-01),complex(-0.000000E+00,+1.746928E-02),complex(+0.000000E+00,-2.226905E-01),complex(+0.000000E+00,-2.314268E-01),0,complex(+0.000000E+00,+2.314268E-01),complex(+0.000000E+00,+2.226905E-01),complex(-0.000000E+00,-1.746928E-02),complex(+0.000000E+00,-2.470529E-01),complex(-0.000000E+00,-2.421875E-01),complex(+0.000000E+00,-8.734641E-03),complex(+0.000000E+00,+2.520969E-01),complex(+0.000000E+00,+3.528622E-01),complex(+0.000000E+00,+2.718986E-01),complex(+0.000000E+00,+1.215967E-01)],$
            [complex(+6.797464E-02,+0.000000E+00),complex(+1.823951E-01,+0.000000E+00),complex(+3.057467E-01,+0.000000E+00),complex(+3.462765E-01,+0.000000E+00),complex(+2.314453E-01,+0.000000E+00),complex(-8.734641E-03,-0.000000E+00),complex(-2.292573E-01,+0.000000E+00),complex(-2.695312E-01,-0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(+1.552458E-01,+0.000000E+00),complex(+2.713719E-01,+0.000000E+00),complex(+1.552458E-01,-0.000000E+00),complex(-9.461071E-02,+0.000000E+00),complex(-2.695312E-01,+0.000000E+00),complex(-2.292573E-01,+0.000000E+00),complex(-8.734641E-03,+0.000000E+00),complex(+2.314453E-01,+0.000000E+00),complex(+3.462765E-01,-0.000000E+00),complex(+3.057467E-01,+0.000000E+00),complex(+1.823951E-01,-0.000000E+00),complex(+6.797464E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+3.297254E-02),complex(-0.000000E+00,+1.032204E-01),complex(+0.000000E+00,+2.105030E-01),complex(-0.000000E+00,+3.144531E-01),complex(+0.000000E+00,+3.462765E-01),complex(-0.000000E+00,+2.520969E-01),complex(+0.000000E+00,+4.555431E-02),complex(+0.000000E+00,-1.771647E-01),complex(+0.000000E+00,-2.874347E-01),complex(+0.000000E+00,-2.133649E-01),0,complex(+0.000000E+00,+2.133649E-01),complex(+0.000000E+00,+2.874347E-01),complex(+0.000000E+00,+1.771647E-01),complex(+0.000000E+00,-4.555431E-02),complex(-0.000000E+00,-2.520969E-01),complex(+0.000000E+00,-3.462765E-01),complex(-0.000000E+00,-3.144531E-01),complex(+0.000000E+00,-2.105030E-01),complex(-0.000000E+00,-1.032204E-01),complex(+0.000000E+00,-3.297254E-02)],$
            [complex(-1.346099E-02,+0.000000E+00),complex(-4.815948E-02,-0.000000E+00),complex(-1.152344E-01,+0.000000E+00),complex(-2.105030E-01,-0.000000E+00),complex(-3.057467E-01,+0.000000E+00),complex(-3.528622E-01,-0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(-1.578048E-01,-0.000000E+00),complex(+5.029060E-02,+0.000000E+00),complex(+2.322823E-01,+0.000000E+00),complex(+3.045247E-01,+0.000000E+00),complex(+2.322823E-01,-0.000000E+00),complex(+5.029060E-02,+0.000000E+00),complex(-1.578048E-01,+0.000000E+00),complex(-3.068582E-01,+0.000000E+00),complex(-3.528622E-01,+0.000000E+00),complex(-3.057467E-01,+0.000000E+00),complex(-2.105030E-01,+0.000000E+00),complex(-1.152344E-01,+0.000000E+00),complex(-4.815948E-02,+0.000000E+00),complex(-1.346099E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.367320E-03),complex(+0.000000E+00,-1.757812E-02),complex(+0.000000E+00,-4.815948E-02),complex(+0.000000E+00,-1.032204E-01),complex(+0.000000E+00,-1.823951E-01),complex(+0.000000E+00,-2.718986E-01),complex(+0.000000E+00,-3.439275E-01),complex(+0.000000E+00,-3.647902E-01),complex(+0.000000E+00,-3.100121E-01),complex(+0.000000E+00,-1.789856E-01),0,complex(+0.000000E+00,+1.789856E-01),complex(+0.000000E+00,+3.100121E-01),complex(+0.000000E+00,+3.647902E-01),complex(+0.000000E+00,+3.439275E-01),complex(+0.000000E+00,+2.718986E-01),complex(+0.000000E+00,+1.823951E-01),complex(+0.000000E+00,+1.032204E-01),complex(+0.000000E+00,+4.815948E-02),complex(+0.000000E+00,+1.757812E-02),complex(+0.000000E+00,+4.367320E-03)],$
            [complex(+9.765625E-04,+0.000000E+00),complex(+4.367320E-03,+0.000000E+00),complex(+1.346099E-02,+0.000000E+00),complex(+3.297254E-02,+0.000000E+00),complex(+6.797464E-02,+0.000000E+00),complex(+1.215967E-01,+0.000000E+00),complex(+1.922613E-01,+0.000000E+00),complex(+2.718986E-01,+0.000000E+00),complex(+3.466040E-01,+0.000000E+00),complex(+4.002239E-01,+0.000000E+00),complex(+4.197583E-01,+0.000000E+00),complex(+4.002239E-01,-0.000000E+00),complex(+3.466040E-01,+0.000000E+00),complex(+2.718986E-01,-0.000000E+00),complex(+1.922613E-01,+0.000000E+00),complex(+1.215967E-01,-0.000000E+00),complex(+6.797464E-02,+0.000000E+00),complex(+3.297254E-02,-0.000000E+00),complex(+1.346099E-02,+0.000000E+00),complex(+4.367320E-03,-0.000000E+00),complex(+9.765625E-04,+0.000000E+00)]$
            ]
          end

      19: begin
            matrix[190:209,190:209]=[$
            [complex(-0.000000E+00,+1.381068E-03),complex(+0.000000E+00,-6.019935E-03),complex(-0.000000E+00,+1.805981E-02),complex(+0.000000E+00,-4.299094E-02),complex(-0.000000E+00,+8.598188E-02),complex(+0.000000E+00,-1.489250E-01),complex(-0.000000E+00,+2.274867E-01),complex(+0.000000E+00,-3.100121E-01),complex(-0.000000E+00,+3.796857E-01),complex(+0.000000E+00,-4.197583E-01),complex(+0.000000E+00,+4.197583E-01),complex(+0.000000E+00,-3.796857E-01),complex(+0.000000E+00,+3.100121E-01),complex(+0.000000E+00,-2.274867E-01),complex(+0.000000E+00,+1.489250E-01),complex(+0.000000E+00,-8.598188E-02),complex(+0.000000E+00,+4.299094E-02),complex(+0.000000E+00,-1.805981E-02),complex(+0.000000E+00,+6.019935E-03),complex(+0.000000E+00,-1.381068E-03)],$
            [complex(-6.019935E-03,+0.000000E+00),complex(+2.347815E-02,+0.000000E+00),complex(-6.214806E-02,+0.000000E+00),complex(+1.282164E-01,+0.000000E+00),complex(-2.169815E-01,+0.000000E+00),complex(+3.074916E-01,+0.000000E+00),complex(-3.653231E-01,+0.000000E+00),complex(+3.556082E-01,+0.000000E+00),complex(-2.613176E-01,+0.000000E+00),complex(+9.629916E-02,+0.000000E+00),complex(+9.629916E-02,+0.000000E+00),complex(-2.613176E-01,+0.000000E+00),complex(+3.556082E-01,+0.000000E+00),complex(-3.653231E-01,+0.000000E+00),complex(+3.074916E-01,+0.000000E+00),complex(-2.169815E-01,+0.000000E+00),complex(+1.282164E-01,+0.000000E+00),complex(-6.214806E-02,+0.000000E+00),complex(+2.347815E-02,+0.000000E+00),complex(-6.019935E-03,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.805981E-02),complex(+0.000000E+00,+6.214806E-02),complex(+0.000000E+00,-1.422500E-01),complex(+0.000000E+00,+2.465699E-01),complex(+0.000000E+00,-3.353351E-01),complex(+0.000000E+00,+3.530459E-01),complex(+0.000000E+00,-2.609451E-01),complex(+0.000000E+00,+7.112165E-02),complex(-0.000000E+00,+1.451765E-01),complex(+0.000000E+00,-2.888975E-01),complex(+0.000000E+00,+2.888975E-01),complex(+0.000000E+00,-1.451765E-01),complex(-0.000000E+00,-7.112165E-02),complex(+0.000000E+00,+2.609451E-01),complex(-0.000000E+00,-3.530459E-01),complex(+0.000000E+00,+3.353351E-01),complex(-0.000000E+00,-2.465699E-01),complex(+0.000000E+00,+1.422500E-01),complex(-0.000000E+00,-6.214806E-02),complex(+0.000000E+00,+1.805981E-02)],$
            [complex(+4.299094E-02,+0.000000E+00),complex(-1.282164E-01,-0.000000E+00),complex(+2.465699E-01,+0.000000E+00),complex(-3.411238E-01,-0.000000E+00),complex(+3.342184E-01,+0.000000E+00),complex(-1.865822E-01,-0.000000E+00),complex(-5.115547E-02,+0.000000E+00),complex(+2.489756E-01,+0.000000E+00),complex(-2.805370E-01,+0.000000E+00),complex(+1.213612E-01,+0.000000E+00),complex(+1.213612E-01,+0.000000E+00),complex(-2.805370E-01,+0.000000E+00),complex(+2.489756E-01,+0.000000E+00),complex(-5.115547E-02,+0.000000E+00),complex(-1.865822E-01,+0.000000E+00),complex(+3.342184E-01,-0.000000E+00),complex(-3.411238E-01,+0.000000E+00),complex(+2.465699E-01,-0.000000E+00),complex(-1.282164E-01,+0.000000E+00),complex(+4.299094E-02,-0.000000E+00)],$
            [complex(-0.000000E+00,+8.598188E-02),complex(+0.000000E+00,-2.169815E-01),complex(-0.000000E+00,+3.353351E-01),complex(+0.000000E+00,-3.342184E-01),complex(-0.000000E+00,+1.602039E-01),complex(+0.000000E+00,+1.052515E-01),complex(+0.000000E+00,-2.777011E-01),complex(+0.000000E+00,+2.190985E-01),complex(-0.000000E+00,+2.439452E-02),complex(+0.000000E+00,-2.427224E-01),complex(+0.000000E+00,+2.427224E-01),complex(+0.000000E+00,-2.439452E-02),complex(-0.000000E+00,-2.190985E-01),complex(+0.000000E+00,+2.777011E-01),complex(-0.000000E+00,-1.052515E-01),complex(+0.000000E+00,-1.602039E-01),complex(+0.000000E+00,+3.342184E-01),complex(+0.000000E+00,-3.353351E-01),complex(+0.000000E+00,+2.169815E-01),complex(+0.000000E+00,-8.598188E-02)],$
            [complex(-1.489250E-01,+0.000000E+00),complex(+3.074916E-01,+0.000000E+00),complex(-3.530459E-01,+0.000000E+00),complex(+1.865822E-01,+0.000000E+00),complex(+1.052515E-01,+0.000000E+00),complex(-2.817379E-01,-0.000000E+00),complex(+1.772078E-01,+0.000000E+00),complex(+1.034972E-01,+0.000000E+00),complex(-2.675995E-01,+0.000000E+00),complex(+1.401359E-01,+0.000000E+00),complex(+1.401359E-01,+0.000000E+00),complex(-2.675995E-01,+0.000000E+00),complex(+1.034972E-01,+0.000000E+00),complex(+1.772078E-01,-0.000000E+00),complex(-2.817379E-01,+0.000000E+00),complex(+1.052515E-01,-0.000000E+00),complex(+1.865822E-01,+0.000000E+00),complex(-3.530459E-01,+0.000000E+00),complex(+3.074916E-01,+0.000000E+00),complex(-1.489250E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.274867E-01),complex(+0.000000E+00,+3.653231E-01),complex(+0.000000E+00,-2.609451E-01),complex(+0.000000E+00,-5.115547E-02),complex(-0.000000E+00,+2.777011E-01),complex(+0.000000E+00,-1.772078E-01),complex(+0.000000E+00,-1.270583E-01),complex(+0.000000E+00,+2.634910E-01),complex(+0.000000E+00,-6.454184E-02),complex(+0.000000E+00,-2.140611E-01),complex(+0.000000E+00,+2.140611E-01),complex(+0.000000E+00,+6.454184E-02),complex(-0.000000E+00,-2.634910E-01),complex(+0.000000E+00,+1.270583E-01),complex(+0.000000E+00,+1.772078E-01),complex(+0.000000E+00,-2.777011E-01),complex(+0.000000E+00,+5.115547E-02),complex(+0.000000E+00,+2.609451E-01),complex(-0.000000E+00,-3.653231E-01),complex(+0.000000E+00,+2.274867E-01)],$
            [complex(+3.100121E-01,+0.000000E+00),complex(-3.556082E-01,-0.000000E+00),complex(+7.112165E-02,+0.000000E+00),complex(+2.489756E-01,+0.000000E+00),complex(-2.190985E-01,+0.000000E+00),complex(-1.034972E-01,-0.000000E+00),complex(+2.634910E-01,+0.000000E+00),complex(-3.866990E-02,-0.000000E+00),complex(-2.368038E-01,+0.000000E+00),complex(+1.570779E-01,+0.000000E+00),complex(+1.570779E-01,+0.000000E+00),complex(-2.368038E-01,+0.000000E+00),complex(-3.866990E-02,+0.000000E+00),complex(+2.634910E-01,-0.000000E+00),complex(-1.034972E-01,+0.000000E+00),complex(-2.190985E-01,+0.000000E+00),complex(+2.489756E-01,+0.000000E+00),complex(+7.112165E-02,-0.000000E+00),complex(-3.556082E-01,+0.000000E+00),complex(+3.100121E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.796857E-01),complex(+0.000000E+00,-2.613176E-01),complex(+0.000000E+00,-1.451765E-01),complex(+0.000000E+00,+2.805370E-01),complex(-0.000000E+00,+2.439452E-02),complex(+0.000000E+00,-2.675995E-01),complex(-0.000000E+00,+6.454184E-02),complex(+0.000000E+00,+2.368038E-01),complex(+0.000000E+00,-1.353447E-01),complex(+0.000000E+00,-1.923803E-01),complex(+0.000000E+00,+1.923803E-01),complex(+0.000000E+00,+1.353447E-01),complex(-0.000000E+00,-2.368038E-01),complex(+0.000000E+00,-6.454184E-02),complex(+0.000000E+00,+2.675995E-01),complex(+0.000000E+00,-2.439452E-02),complex(-0.000000E+00,-2.805370E-01),complex(+0.000000E+00,+1.451765E-01),complex(+0.000000E+00,+2.613176E-01),complex(+0.000000E+00,-3.796857E-01)],$
            [complex(-4.197583E-01,+0.000000E+00),complex(+9.629916E-02,+0.000000E+00),complex(+2.888975E-01,+0.000000E+00),complex(-1.213612E-01,-0.000000E+00),complex(-2.427224E-01,+0.000000E+00),complex(+1.401359E-01,+0.000000E+00),complex(+2.140611E-01,+0.000000E+00),complex(-1.570779E-01,-0.000000E+00),complex(-1.923803E-01,+0.000000E+00),complex(+1.740146E-01,+0.000000E+00),complex(+1.740146E-01,+0.000000E+00),complex(-1.923803E-01,+0.000000E+00),complex(-1.570779E-01,+0.000000E+00),complex(+2.140611E-01,-0.000000E+00),complex(+1.401359E-01,+0.000000E+00),complex(-2.427224E-01,+0.000000E+00),complex(-1.213612E-01,+0.000000E+00),complex(+2.888975E-01,-0.000000E+00),complex(+9.629916E-02,+0.000000E+00),complex(-4.197583E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.197583E-01),complex(+0.000000E+00,-9.629916E-02),complex(-0.000000E+00,+2.888975E-01),complex(+0.000000E+00,+1.213612E-01),complex(+0.000000E+00,-2.427224E-01),complex(+0.000000E+00,-1.401359E-01),complex(-0.000000E+00,+2.140611E-01),complex(+0.000000E+00,+1.570779E-01),complex(+0.000000E+00,-1.923803E-01),complex(+0.000000E+00,-1.740146E-01),complex(+0.000000E+00,+1.740146E-01),complex(+0.000000E+00,+1.923803E-01),complex(-0.000000E+00,-1.570779E-01),complex(+0.000000E+00,-2.140611E-01),complex(+0.000000E+00,+1.401359E-01),complex(+0.000000E+00,+2.427224E-01),complex(-0.000000E+00,-1.213612E-01),complex(+0.000000E+00,-2.888975E-01),complex(+0.000000E+00,+9.629916E-02),complex(+0.000000E+00,+4.197583E-01)],$
            [complex(+3.796857E-01,+0.000000E+00),complex(+2.613176E-01,+0.000000E+00),complex(-1.451765E-01,+0.000000E+00),complex(-2.805370E-01,-0.000000E+00),complex(+2.439452E-02,+0.000000E+00),complex(+2.675995E-01,+0.000000E+00),complex(+6.454184E-02,+0.000000E+00),complex(-2.368038E-01,-0.000000E+00),complex(-1.353447E-01,+0.000000E+00),complex(+1.923803E-01,+0.000000E+00),complex(+1.923803E-01,+0.000000E+00),complex(-1.353447E-01,+0.000000E+00),complex(-2.368038E-01,+0.000000E+00),complex(+6.454184E-02,-0.000000E+00),complex(+2.675995E-01,+0.000000E+00),complex(+2.439452E-02,-0.000000E+00),complex(-2.805370E-01,+0.000000E+00),complex(-1.451765E-01,+0.000000E+00),complex(+2.613176E-01,+0.000000E+00),complex(+3.796857E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.100121E-01),complex(+0.000000E+00,+3.556082E-01),complex(-0.000000E+00,+7.112165E-02),complex(+0.000000E+00,-2.489756E-01),complex(+0.000000E+00,-2.190985E-01),complex(+0.000000E+00,+1.034972E-01),complex(-0.000000E+00,+2.634910E-01),complex(+0.000000E+00,+3.866990E-02),complex(+0.000000E+00,-2.368038E-01),complex(+0.000000E+00,-1.570779E-01),complex(+0.000000E+00,+1.570779E-01),complex(+0.000000E+00,+2.368038E-01),complex(-0.000000E+00,-3.866990E-02),complex(+0.000000E+00,-2.634910E-01),complex(-0.000000E+00,-1.034972E-01),complex(+0.000000E+00,+2.190985E-01),complex(+0.000000E+00,+2.489756E-01),complex(+0.000000E+00,-7.112165E-02),complex(-0.000000E+00,-3.556082E-01),complex(+0.000000E+00,-3.100121E-01)],$
            [complex(-2.274867E-01,+0.000000E+00),complex(-3.653231E-01,-0.000000E+00),complex(-2.609451E-01,+0.000000E+00),complex(+5.115547E-02,+0.000000E+00),complex(+2.777011E-01,+0.000000E+00),complex(+1.772078E-01,+0.000000E+00),complex(-1.270583E-01,+0.000000E+00),complex(-2.634910E-01,-0.000000E+00),complex(-6.454184E-02,+0.000000E+00),complex(+2.140611E-01,+0.000000E+00),complex(+2.140611E-01,+0.000000E+00),complex(-6.454184E-02,+0.000000E+00),complex(-2.634910E-01,+0.000000E+00),complex(-1.270583E-01,+0.000000E+00),complex(+1.772078E-01,+0.000000E+00),complex(+2.777011E-01,-0.000000E+00),complex(+5.115547E-02,+0.000000E+00),complex(-2.609451E-01,+0.000000E+00),complex(-3.653231E-01,+0.000000E+00),complex(-2.274867E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.489250E-01),complex(+0.000000E+00,-3.074916E-01),complex(+0.000000E+00,-3.530459E-01),complex(+0.000000E+00,-1.865822E-01),complex(-0.000000E+00,+1.052515E-01),complex(+0.000000E+00,+2.817379E-01),complex(-0.000000E+00,+1.772078E-01),complex(+0.000000E+00,-1.034972E-01),complex(+0.000000E+00,-2.675995E-01),complex(+0.000000E+00,-1.401359E-01),complex(+0.000000E+00,+1.401359E-01),complex(+0.000000E+00,+2.675995E-01),complex(+0.000000E+00,+1.034972E-01),complex(+0.000000E+00,-1.772078E-01),complex(-0.000000E+00,-2.817379E-01),complex(+0.000000E+00,-1.052515E-01),complex(+0.000000E+00,+1.865822E-01),complex(+0.000000E+00,+3.530459E-01),complex(+0.000000E+00,+3.074916E-01),complex(+0.000000E+00,+1.489250E-01)],$
            [complex(+8.598188E-02,+0.000000E+00),complex(+2.169815E-01,+0.000000E+00),complex(+3.353351E-01,+0.000000E+00),complex(+3.342184E-01,+0.000000E+00),complex(+1.602039E-01,+0.000000E+00),complex(-1.052515E-01,-0.000000E+00),complex(-2.777011E-01,+0.000000E+00),complex(-2.190985E-01,-0.000000E+00),complex(+2.439452E-02,+0.000000E+00),complex(+2.427224E-01,+0.000000E+00),complex(+2.427224E-01,+0.000000E+00),complex(+2.439452E-02,-0.000000E+00),complex(-2.190985E-01,+0.000000E+00),complex(-2.777011E-01,+0.000000E+00),complex(-1.052515E-01,+0.000000E+00),complex(+1.602039E-01,-0.000000E+00),complex(+3.342184E-01,+0.000000E+00),complex(+3.353351E-01,-0.000000E+00),complex(+2.169815E-01,+0.000000E+00),complex(+8.598188E-02,-0.000000E+00)],$
            [complex(-0.000000E+00,+4.299094E-02),complex(+0.000000E+00,+1.282164E-01),complex(-0.000000E+00,+2.465699E-01),complex(+0.000000E+00,+3.411238E-01),complex(-0.000000E+00,+3.342184E-01),complex(+0.000000E+00,+1.865822E-01),complex(+0.000000E+00,-5.115547E-02),complex(+0.000000E+00,-2.489756E-01),complex(+0.000000E+00,-2.805370E-01),complex(+0.000000E+00,-1.213612E-01),complex(+0.000000E+00,+1.213612E-01),complex(+0.000000E+00,+2.805370E-01),complex(+0.000000E+00,+2.489756E-01),complex(+0.000000E+00,+5.115547E-02),complex(-0.000000E+00,-1.865822E-01),complex(+0.000000E+00,-3.342184E-01),complex(-0.000000E+00,-3.411238E-01),complex(+0.000000E+00,-2.465699E-01),complex(-0.000000E+00,-1.282164E-01),complex(+0.000000E+00,-4.299094E-02)],$
            [complex(-1.805981E-02,+0.000000E+00),complex(-6.214806E-02,-0.000000E+00),complex(-1.422500E-01,+0.000000E+00),complex(-2.465699E-01,-0.000000E+00),complex(-3.353351E-01,+0.000000E+00),complex(-3.530459E-01,-0.000000E+00),complex(-2.609451E-01,+0.000000E+00),complex(-7.112165E-02,-0.000000E+00),complex(+1.451765E-01,+0.000000E+00),complex(+2.888975E-01,+0.000000E+00),complex(+2.888975E-01,+0.000000E+00),complex(+1.451765E-01,-0.000000E+00),complex(-7.112165E-02,+0.000000E+00),complex(-2.609451E-01,+0.000000E+00),complex(-3.530459E-01,+0.000000E+00),complex(-3.353351E-01,+0.000000E+00),complex(-2.465699E-01,+0.000000E+00),complex(-1.422500E-01,+0.000000E+00),complex(-6.214806E-02,+0.000000E+00),complex(-1.805981E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-6.019935E-03),complex(+0.000000E+00,-2.347815E-02),complex(+0.000000E+00,-6.214806E-02),complex(+0.000000E+00,-1.282164E-01),complex(+0.000000E+00,-2.169815E-01),complex(+0.000000E+00,-3.074916E-01),complex(+0.000000E+00,-3.653231E-01),complex(+0.000000E+00,-3.556082E-01),complex(+0.000000E+00,-2.613176E-01),complex(+0.000000E+00,-9.629916E-02),complex(+0.000000E+00,+9.629916E-02),complex(+0.000000E+00,+2.613176E-01),complex(+0.000000E+00,+3.556082E-01),complex(+0.000000E+00,+3.653231E-01),complex(+0.000000E+00,+3.074916E-01),complex(+0.000000E+00,+2.169815E-01),complex(+0.000000E+00,+1.282164E-01),complex(+0.000000E+00,+6.214806E-02),complex(+0.000000E+00,+2.347815E-02),complex(+0.000000E+00,+6.019935E-03)],$
            [complex(+1.381068E-03,+0.000000E+00),complex(+6.019935E-03,+0.000000E+00),complex(+1.805981E-02,+0.000000E+00),complex(+4.299094E-02,+0.000000E+00),complex(+8.598188E-02,+0.000000E+00),complex(+1.489250E-01,+0.000000E+00),complex(+2.274867E-01,+0.000000E+00),complex(+3.100121E-01,+0.000000E+00),complex(+3.796857E-01,+0.000000E+00),complex(+4.197583E-01,+0.000000E+00),complex(+4.197583E-01,+0.000000E+00),complex(+3.796857E-01,-0.000000E+00),complex(+3.100121E-01,+0.000000E+00),complex(+2.274867E-01,-0.000000E+00),complex(+1.489250E-01,+0.000000E+00),complex(+8.598188E-02,-0.000000E+00),complex(+4.299094E-02,+0.000000E+00),complex(+1.805981E-02,-0.000000E+00),complex(+6.019935E-03,+0.000000E+00),complex(+1.381068E-03,-0.000000E+00)]$
            ]
          end

      18: begin
            matrix[171:189,171:189]=[$
            [complex(-1.953125E-03,-0.000000E+00),complex(+8.286407E-03,+0.000000E+00),complex(-2.415882E-02,-0.000000E+00),complex(+5.579241E-02,+0.000000E+00),complex(-1.080415E-01,-0.000000E+00),complex(+1.807881E-01,+0.000000E+00),complex(-2.661128E-01,-0.000000E+00),complex(+3.484235E-01,+0.000000E+00),complex(-4.085627E-01,-0.000000E+00),complex(+4.306630E-01,+0.000000E+00),complex(-4.085627E-01,+0.000000E+00),complex(+3.484235E-01,+0.000000E+00),complex(-2.661128E-01,+0.000000E+00),complex(+1.807881E-01,+0.000000E+00),complex(-1.080415E-01,+0.000000E+00),complex(+5.579241E-02,+0.000000E+00),complex(-2.415882E-02,+0.000000E+00),complex(+8.286407E-03,+0.000000E+00),complex(-1.953125E-03,+0.000000E+00)],$
            [complex(+0.000000E+00,-8.286407E-03),complex(+0.000000E+00,+3.125000E-02),complex(+0.000000E+00,-7.972004E-02),complex(+0.000000E+00,+1.578048E-01),complex(+0.000000E+00,-2.546563E-01),complex(+0.000000E+00,+3.408973E-01),complex(+0.000000E+00,-3.763404E-01),complex(+0.000000E+00,+3.284968E-01),complex(+0.000000E+00,-1.925983E-01),0,complex(+0.000000E+00,+1.925983E-01),complex(+0.000000E+00,-3.284968E-01),complex(+0.000000E+00,+3.763404E-01),complex(+0.000000E+00,-3.408973E-01),complex(+0.000000E+00,+2.546563E-01),complex(+0.000000E+00,-1.578048E-01),complex(+0.000000E+00,+7.972004E-02),complex(+0.000000E+00,-3.125000E-02),complex(+0.000000E+00,+8.286407E-03)],$
            [complex(+2.415882E-02,+0.000000E+00),complex(-7.972004E-02,+0.000000E+00),complex(+1.738281E-01,+0.000000E+00),complex(-2.841646E-01,+0.000000E+00),complex(+3.581203E-01,+0.000000E+00),complex(-3.361645E-01,+0.000000E+00),complex(+1.936255E-01,+0.000000E+00),complex(+2.816837E-02,+0.000000E+00),complex(-2.312124E-01,-0.000000E+00),complex(+3.133533E-01,+0.000000E+00),complex(-2.312124E-01,+0.000000E+00),complex(+2.816837E-02,+0.000000E+00),complex(+1.936255E-01,-0.000000E+00),complex(-3.361645E-01,+0.000000E+00),complex(+3.581203E-01,-0.000000E+00),complex(-2.841646E-01,+0.000000E+00),complex(+1.738281E-01,-0.000000E+00),complex(-7.972004E-02,+0.000000E+00),complex(+2.415882E-02,-0.000000E+00)],$
            [complex(-0.000000E+00,+5.579241E-02),complex(+0.000000E+00,-1.578048E-01),complex(-0.000000E+00,+2.841646E-01),complex(+0.000000E+00,-3.593750E-01),complex(-0.000000E+00,+3.025768E-01),complex(+0.000000E+00,-1.012616E-01),complex(+0.000000E+00,-1.490530E-01),complex(+0.000000E+00,+2.927343E-01),complex(+0.000000E+00,-2.288409E-01),0,complex(+0.000000E+00,+2.288409E-01),complex(+0.000000E+00,-2.927343E-01),complex(+0.000000E+00,+1.490530E-01),complex(+0.000000E+00,+1.012616E-01),complex(-0.000000E+00,-3.025768E-01),complex(+0.000000E+00,+3.593750E-01),complex(-0.000000E+00,-2.841646E-01),complex(+0.000000E+00,+1.578048E-01),complex(-0.000000E+00,-5.579241E-02)],$
            [complex(-1.080415E-01,-0.000000E+00),complex(+2.546563E-01,+0.000000E+00),complex(-3.581203E-01,-0.000000E+00),complex(+3.025768E-01,+0.000000E+00),complex(-7.031250E-02,-0.000000E+00),complex(-1.960922E-01,+0.000000E+00),complex(+2.886399E-01,+0.000000E+00),complex(-1.259728E-01,+0.000000E+00),complex(-1.477162E-01,-0.000000E+00),complex(+2.802717E-01,+0.000000E+00),complex(-1.477162E-01,+0.000000E+00),complex(-1.259728E-01,+0.000000E+00),complex(+2.886399E-01,-0.000000E+00),complex(-1.960922E-01,+0.000000E+00),complex(-7.031250E-02,+0.000000E+00),complex(+3.025768E-01,+0.000000E+00),complex(-3.581203E-01,+0.000000E+00),complex(+2.546563E-01,+0.000000E+00),complex(-1.080415E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.807881E-01),complex(+0.000000E+00,+3.408973E-01),complex(+0.000000E+00,-3.361645E-01),complex(+0.000000E+00,+1.012616E-01),complex(-0.000000E+00,+1.960922E-01),complex(+0.000000E+00,-2.812500E-01),complex(-0.000000E+00,+6.899813E-02),complex(+0.000000E+00,+2.107928E-01),complex(+0.000000E+00,-2.471764E-01),0,complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,-2.107928E-01),complex(-0.000000E+00,-6.899813E-02),complex(+0.000000E+00,+2.812500E-01),complex(-0.000000E+00,-1.960922E-01),complex(+0.000000E+00,-1.012616E-01),complex(+0.000000E+00,+3.361645E-01),complex(+0.000000E+00,-3.408973E-01),complex(+0.000000E+00,+1.807881E-01)],$
            [complex(+2.661128E-01,+0.000000E+00),complex(-3.763404E-01,+0.000000E+00),complex(+1.936255E-01,+0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(-2.886399E-01,-0.000000E+00),complex(+6.899813E-02,+0.000000E+00),complex(+2.265625E-01,+0.000000E+00),complex(-2.148082E-01,+0.000000E+00),complex(-8.396166E-02,-0.000000E+00),complex(+2.655101E-01,+0.000000E+00),complex(-8.396166E-02,+0.000000E+00),complex(-2.148082E-01,+0.000000E+00),complex(+2.265625E-01,-0.000000E+00),complex(+6.899813E-02,+0.000000E+00),complex(-2.886399E-01,+0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(+1.936255E-01,-0.000000E+00),complex(-3.763404E-01,+0.000000E+00),complex(+2.661128E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.484235E-01),complex(+0.000000E+00,-3.284968E-01),complex(+0.000000E+00,-2.816837E-02),complex(+0.000000E+00,+2.927343E-01),complex(+0.000000E+00,-1.259728E-01),complex(+0.000000E+00,-2.107928E-01),complex(-0.000000E+00,+2.148082E-01),complex(+0.000000E+00,+1.093750E-01),complex(+0.000000E+00,-2.565071E-01),0,complex(+0.000000E+00,+2.565071E-01),complex(+0.000000E+00,-1.093750E-01),complex(-0.000000E+00,-2.148082E-01),complex(+0.000000E+00,+2.107928E-01),complex(+0.000000E+00,+1.259728E-01),complex(+0.000000E+00,-2.927343E-01),complex(+0.000000E+00,+2.816837E-02),complex(+0.000000E+00,+3.284968E-01),complex(-0.000000E+00,-3.484235E-01)],$
            [complex(-4.085627E-01,-0.000000E+00),complex(+1.925983E-01,+0.000000E+00),complex(+2.312124E-01,+0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(-1.477162E-01,-0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(+8.396166E-02,+0.000000E+00),complex(-2.565071E-01,+0.000000E+00),complex(-2.734375E-02,-0.000000E+00),complex(+2.594056E-01,+0.000000E+00),complex(-2.734375E-02,+0.000000E+00),complex(-2.565071E-01,+0.000000E+00),complex(+8.396166E-02,-0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(-1.477162E-01,+0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(+2.312124E-01,-0.000000E+00),complex(+1.925983E-01,+0.000000E+00),complex(-4.085627E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.306630E-01),0,complex(-0.000000E+00,+3.133533E-01),0,complex(+0.000000E+00,-2.802717E-01),0,complex(-0.000000E+00,+2.655101E-01),0,complex(+0.000000E+00,-2.594056E-01),0,complex(+0.000000E+00,+2.594056E-01),0,complex(-0.000000E+00,-2.655101E-01),0,complex(+0.000000E+00,+2.802717E-01),0,complex(-0.000000E+00,-3.133533E-01),0,complex(+0.000000E+00,+4.306630E-01)],$
            [complex(+4.085627E-01,+0.000000E+00),complex(+1.925983E-01,+0.000000E+00),complex(-2.312124E-01,-0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(+1.477162E-01,+0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(-8.396166E-02,-0.000000E+00),complex(-2.565071E-01,+0.000000E+00),complex(+2.734375E-02,+0.000000E+00),complex(+2.594056E-01,+0.000000E+00),complex(+2.734375E-02,-0.000000E+00),complex(-2.565071E-01,+0.000000E+00),complex(-8.396166E-02,+0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(+1.477162E-01,-0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(-2.312124E-01,+0.000000E+00),complex(+1.925983E-01,+0.000000E+00),complex(+4.085627E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.484235E-01),complex(+0.000000E+00,+3.284968E-01),complex(+0.000000E+00,-2.816837E-02),complex(+0.000000E+00,-2.927343E-01),complex(+0.000000E+00,-1.259728E-01),complex(+0.000000E+00,+2.107928E-01),complex(-0.000000E+00,+2.148082E-01),complex(+0.000000E+00,-1.093750E-01),complex(+0.000000E+00,-2.565071E-01),0,complex(+0.000000E+00,+2.565071E-01),complex(+0.000000E+00,+1.093750E-01),complex(-0.000000E+00,-2.148082E-01),complex(+0.000000E+00,-2.107928E-01),complex(+0.000000E+00,+1.259728E-01),complex(+0.000000E+00,+2.927343E-01),complex(+0.000000E+00,+2.816837E-02),complex(+0.000000E+00,-3.284968E-01),complex(-0.000000E+00,-3.484235E-01)],$
            [complex(-2.661128E-01,-0.000000E+00),complex(-3.763404E-01,+0.000000E+00),complex(-1.936255E-01,-0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(+2.886399E-01,+0.000000E+00),complex(+6.899813E-02,+0.000000E+00),complex(-2.265625E-01,-0.000000E+00),complex(-2.148082E-01,+0.000000E+00),complex(+8.396166E-02,+0.000000E+00),complex(+2.655101E-01,+0.000000E+00),complex(+8.396166E-02,-0.000000E+00),complex(-2.148082E-01,+0.000000E+00),complex(-2.265625E-01,+0.000000E+00),complex(+6.899813E-02,+0.000000E+00),complex(+2.886399E-01,-0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(-1.936255E-01,+0.000000E+00),complex(-3.763404E-01,+0.000000E+00),complex(-2.661128E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.807881E-01),complex(+0.000000E+00,-3.408973E-01),complex(+0.000000E+00,-3.361645E-01),complex(+0.000000E+00,-1.012616E-01),complex(-0.000000E+00,+1.960922E-01),complex(+0.000000E+00,+2.812500E-01),complex(-0.000000E+00,+6.899813E-02),complex(+0.000000E+00,-2.107928E-01),complex(+0.000000E+00,-2.471764E-01),0,complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,+2.107928E-01),complex(-0.000000E+00,-6.899813E-02),complex(+0.000000E+00,-2.812500E-01),complex(-0.000000E+00,-1.960922E-01),complex(+0.000000E+00,+1.012616E-01),complex(+0.000000E+00,+3.361645E-01),complex(+0.000000E+00,+3.408973E-01),complex(+0.000000E+00,+1.807881E-01)],$
            [complex(+1.080415E-01,+0.000000E+00),complex(+2.546563E-01,+0.000000E+00),complex(+3.581203E-01,+0.000000E+00),complex(+3.025768E-01,+0.000000E+00),complex(+7.031250E-02,+0.000000E+00),complex(-1.960922E-01,+0.000000E+00),complex(-2.886399E-01,-0.000000E+00),complex(-1.259728E-01,+0.000000E+00),complex(+1.477162E-01,+0.000000E+00),complex(+2.802717E-01,+0.000000E+00),complex(+1.477162E-01,-0.000000E+00),complex(-1.259728E-01,+0.000000E+00),complex(-2.886399E-01,+0.000000E+00),complex(-1.960922E-01,+0.000000E+00),complex(+7.031250E-02,-0.000000E+00),complex(+3.025768E-01,+0.000000E+00),complex(+3.581203E-01,-0.000000E+00),complex(+2.546563E-01,+0.000000E+00),complex(+1.080415E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+5.579241E-02),complex(+0.000000E+00,+1.578048E-01),complex(-0.000000E+00,+2.841646E-01),complex(+0.000000E+00,+3.593750E-01),complex(-0.000000E+00,+3.025768E-01),complex(+0.000000E+00,+1.012616E-01),complex(+0.000000E+00,-1.490530E-01),complex(+0.000000E+00,-2.927343E-01),complex(+0.000000E+00,-2.288409E-01),0,complex(+0.000000E+00,+2.288409E-01),complex(+0.000000E+00,+2.927343E-01),complex(+0.000000E+00,+1.490530E-01),complex(+0.000000E+00,-1.012616E-01),complex(-0.000000E+00,-3.025768E-01),complex(+0.000000E+00,-3.593750E-01),complex(-0.000000E+00,-2.841646E-01),complex(+0.000000E+00,-1.578048E-01),complex(-0.000000E+00,-5.579241E-02)],$
            [complex(-2.415882E-02,-0.000000E+00),complex(-7.972004E-02,+0.000000E+00),complex(-1.738281E-01,-0.000000E+00),complex(-2.841646E-01,+0.000000E+00),complex(-3.581203E-01,-0.000000E+00),complex(-3.361645E-01,+0.000000E+00),complex(-1.936255E-01,-0.000000E+00),complex(+2.816837E-02,+0.000000E+00),complex(+2.312124E-01,+0.000000E+00),complex(+3.133533E-01,+0.000000E+00),complex(+2.312124E-01,-0.000000E+00),complex(+2.816837E-02,+0.000000E+00),complex(-1.936255E-01,+0.000000E+00),complex(-3.361645E-01,+0.000000E+00),complex(-3.581203E-01,+0.000000E+00),complex(-2.841646E-01,+0.000000E+00),complex(-1.738281E-01,+0.000000E+00),complex(-7.972004E-02,+0.000000E+00),complex(-2.415882E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-8.286407E-03),complex(+0.000000E+00,-3.125000E-02),complex(+0.000000E+00,-7.972004E-02),complex(+0.000000E+00,-1.578048E-01),complex(+0.000000E+00,-2.546563E-01),complex(+0.000000E+00,-3.408973E-01),complex(+0.000000E+00,-3.763404E-01),complex(+0.000000E+00,-3.284968E-01),complex(+0.000000E+00,-1.925983E-01),0,complex(+0.000000E+00,+1.925983E-01),complex(+0.000000E+00,+3.284968E-01),complex(+0.000000E+00,+3.763404E-01),complex(+0.000000E+00,+3.408973E-01),complex(+0.000000E+00,+2.546563E-01),complex(+0.000000E+00,+1.578048E-01),complex(+0.000000E+00,+7.972004E-02),complex(+0.000000E+00,+3.125000E-02),complex(+0.000000E+00,+8.286407E-03)],$
            [complex(+1.953125E-03,+0.000000E+00),complex(+8.286407E-03,+0.000000E+00),complex(+2.415882E-02,+0.000000E+00),complex(+5.579241E-02,+0.000000E+00),complex(+1.080415E-01,+0.000000E+00),complex(+1.807881E-01,+0.000000E+00),complex(+2.661128E-01,+0.000000E+00),complex(+3.484235E-01,+0.000000E+00),complex(+4.085627E-01,+0.000000E+00),complex(+4.306630E-01,+0.000000E+00),complex(+4.085627E-01,-0.000000E+00),complex(+3.484235E-01,+0.000000E+00),complex(+2.661128E-01,-0.000000E+00),complex(+1.807881E-01,+0.000000E+00),complex(+1.080415E-01,-0.000000E+00),complex(+5.579241E-02,+0.000000E+00),complex(+2.415882E-02,-0.000000E+00),complex(+8.286407E-03,+0.000000E+00),complex(+1.953125E-03,-0.000000E+00)]$
            ]
          end

      17: begin
            matrix[153:170,153:170]=[$
            [complex(+0.000000E+00,-2.762136E-03),complex(-0.000000E+00,+1.138858E-02),complex(+0.000000E+00,-3.221176E-02),complex(-0.000000E+00,+7.202769E-02),complex(+0.000000E+00,-1.347515E-01),complex(-0.000000E+00,+2.172802E-01),complex(+0.000000E+00,-3.072806E-01),complex(-0.000000E+00,+3.851967E-01),complex(+0.000000E+00,-4.306630E-01),complex(+0.000000E+00,+4.306630E-01),complex(+0.000000E+00,-3.851967E-01),complex(+0.000000E+00,+3.072806E-01),complex(+0.000000E+00,-2.172802E-01),complex(+0.000000E+00,+1.347515E-01),complex(+0.000000E+00,-7.202769E-02),complex(+0.000000E+00,+3.221176E-02),complex(+0.000000E+00,-1.138858E-02),complex(+0.000000E+00,+2.762136E-03)],$
            [complex(+1.138858E-02,+0.000000E+00),complex(-4.143204E-02,+0.000000E+00),complex(+1.015625E-01,+0.000000E+00),complex(-1.921621E-01,+0.000000E+00),complex(+2.941383E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+3.726325E-01,+0.000000E+00),complex(-2.802717E-01,+0.000000E+00),complex(+1.044511E-01,+0.000000E+00),complex(+1.044511E-01,+0.000000E+00),complex(-2.802717E-01,+0.000000E+00),complex(+3.726325E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+2.941383E-01,+0.000000E+00),complex(-1.921621E-01,+0.000000E+00),complex(+1.015625E-01,+0.000000E+00),complex(-4.143204E-02,+0.000000E+00),complex(+1.138858E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+3.221176E-02),complex(+0.000000E+00,-1.015625E-01),complex(+0.000000E+00,+2.099223E-01),complex(+0.000000E+00,-3.211688E-01),complex(+0.000000E+00,+3.697550E-01),complex(+0.000000E+00,-2.981060E-01),complex(+0.000000E+00,+1.053964E-01),complex(-0.000000E+00,+1.321214E-01),complex(+0.000000E+00,-2.954324E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,-1.321214E-01),complex(-0.000000E+00,-1.053964E-01),complex(+0.000000E+00,+2.981060E-01),complex(-0.000000E+00,-3.697550E-01),complex(+0.000000E+00,+3.211688E-01),complex(-0.000000E+00,-2.099223E-01),complex(+0.000000E+00,+1.015625E-01),complex(-0.000000E+00,-3.221176E-02)],$
            [complex(-7.202769E-02,-0.000000E+00),complex(+1.921621E-01,+0.000000E+00),complex(-3.211688E-01,-0.000000E+00),complex(+3.646019E-01,+0.000000E+00),complex(-2.480392E-01,-0.000000E+00),0,complex(+2.356735E-01,+0.000000E+00),complex(-2.954324E-01,+0.000000E+00),complex(+1.321214E-01,+0.000000E+00),complex(+1.321214E-01,+0.000000E+00),complex(-2.954324E-01,+0.000000E+00),complex(+2.356735E-01,+0.000000E+00),0,complex(-2.480392E-01,+0.000000E+00),complex(+3.646019E-01,-0.000000E+00),complex(-3.211688E-01,+0.000000E+00),complex(+1.921621E-01,-0.000000E+00),complex(-7.202769E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.347515E-01),complex(-0.000000E+00,+2.941383E-01),complex(+0.000000E+00,-3.697550E-01),complex(-0.000000E+00,+2.480392E-01),complex(+0.000000E+00,+3.314563E-02),complex(+0.000000E+00,-2.672286E-01),complex(+0.000000E+00,+2.519456E-01),0,complex(+0.000000E+00,-2.471764E-01),complex(+0.000000E+00,+2.471764E-01),0,complex(-0.000000E+00,-2.519456E-01),complex(+0.000000E+00,+2.672286E-01),complex(-0.000000E+00,-3.314563E-02),complex(+0.000000E+00,-2.480392E-01),complex(+0.000000E+00,+3.697550E-01),complex(+0.000000E+00,-2.941383E-01),complex(+0.000000E+00,+1.347515E-01)],$
            [complex(+2.172802E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+2.981060E-01,+0.000000E+00),0,complex(-2.672286E-01,-0.000000E+00),complex(+2.320194E-01,+0.000000E+00),complex(+6.250000E-02,+0.000000E+00),complex(-2.742176E-01,+0.000000E+00),complex(+1.532923E-01,+0.000000E+00),complex(+1.532923E-01,+0.000000E+00),complex(-2.742176E-01,+0.000000E+00),complex(+6.250000E-02,+0.000000E+00),complex(+2.320194E-01,-0.000000E+00),complex(-2.672286E-01,+0.000000E+00),0,complex(+2.981060E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+2.172802E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+3.072806E-01),complex(+0.000000E+00,-3.726325E-01),complex(+0.000000E+00,+1.053964E-01),complex(-0.000000E+00,+2.356735E-01),complex(+0.000000E+00,-2.519456E-01),complex(+0.000000E+00,-6.250000E-02),complex(+0.000000E+00,+2.762136E-01),complex(+0.000000E+00,-9.695058E-02),complex(+0.000000E+00,-2.167881E-01),complex(+0.000000E+00,+2.167881E-01),complex(+0.000000E+00,+9.695058E-02),complex(-0.000000E+00,-2.762136E-01),complex(+0.000000E+00,+6.250000E-02),complex(+0.000000E+00,+2.519456E-01),complex(+0.000000E+00,-2.356735E-01),complex(-0.000000E+00,-1.053964E-01),complex(+0.000000E+00,+3.726325E-01),complex(-0.000000E+00,-3.072806E-01)],$
            [complex(-3.851967E-01,-0.000000E+00),complex(+2.802717E-01,+0.000000E+00),complex(+1.321214E-01,+0.000000E+00),complex(-2.954324E-01,+0.000000E+00),0,complex(+2.742176E-01,+0.000000E+00),complex(-9.695058E-02,-0.000000E+00),complex(-2.320194E-01,+0.000000E+00),complex(+1.729371E-01,+0.000000E+00),complex(+1.729371E-01,+0.000000E+00),complex(-2.320194E-01,+0.000000E+00),complex(-9.695058E-02,+0.000000E+00),complex(+2.742176E-01,-0.000000E+00),0,complex(-2.954324E-01,+0.000000E+00),complex(+1.321214E-01,+0.000000E+00),complex(+2.802717E-01,-0.000000E+00),complex(-3.851967E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.306630E-01),complex(-0.000000E+00,+1.044511E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,-1.321214E-01),complex(+0.000000E+00,-2.471764E-01),complex(-0.000000E+00,+1.532923E-01),complex(+0.000000E+00,+2.167881E-01),complex(+0.000000E+00,-1.729371E-01),complex(+0.000000E+00,-1.933495E-01),complex(+0.000000E+00,+1.933495E-01),complex(+0.000000E+00,+1.729371E-01),complex(-0.000000E+00,-2.167881E-01),complex(+0.000000E+00,-1.532923E-01),complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,+1.321214E-01),complex(-0.000000E+00,-2.954324E-01),complex(+0.000000E+00,-1.044511E-01),complex(+0.000000E+00,+4.306630E-01)],$
            [complex(+4.306630E-01,+0.000000E+00),complex(+1.044511E-01,+0.000000E+00),complex(-2.954324E-01,-0.000000E+00),complex(-1.321214E-01,+0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(+1.532923E-01,+0.000000E+00),complex(-2.167881E-01,-0.000000E+00),complex(-1.729371E-01,+0.000000E+00),complex(+1.933495E-01,+0.000000E+00),complex(+1.933495E-01,+0.000000E+00),complex(-1.729371E-01,+0.000000E+00),complex(-2.167881E-01,+0.000000E+00),complex(+1.532923E-01,-0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(-1.321214E-01,+0.000000E+00),complex(-2.954324E-01,+0.000000E+00),complex(+1.044511E-01,-0.000000E+00),complex(+4.306630E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+3.851967E-01),complex(-0.000000E+00,+2.802717E-01),complex(+0.000000E+00,-1.321214E-01),complex(+0.000000E+00,-2.954324E-01),0,complex(-0.000000E+00,+2.742176E-01),complex(+0.000000E+00,+9.695058E-02),complex(+0.000000E+00,-2.320194E-01),complex(+0.000000E+00,-1.729371E-01),complex(+0.000000E+00,+1.729371E-01),complex(+0.000000E+00,+2.320194E-01),complex(-0.000000E+00,-9.695058E-02),complex(+0.000000E+00,-2.742176E-01),0,complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,+1.321214E-01),complex(+0.000000E+00,-2.802717E-01),complex(-0.000000E+00,-3.851967E-01)],$
            [complex(-3.072806E-01,-0.000000E+00),complex(-3.726325E-01,+0.000000E+00),complex(-1.053964E-01,-0.000000E+00),complex(+2.356735E-01,+0.000000E+00),complex(+2.519456E-01,+0.000000E+00),complex(-6.250000E-02,+0.000000E+00),complex(-2.762136E-01,-0.000000E+00),complex(-9.695058E-02,+0.000000E+00),complex(+2.167881E-01,+0.000000E+00),complex(+2.167881E-01,+0.000000E+00),complex(-9.695058E-02,+0.000000E+00),complex(-2.762136E-01,+0.000000E+00),complex(-6.250000E-02,+0.000000E+00),complex(+2.519456E-01,+0.000000E+00),complex(+2.356735E-01,-0.000000E+00),complex(-1.053964E-01,+0.000000E+00),complex(-3.726325E-01,+0.000000E+00),complex(-3.072806E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.172802E-01),complex(+0.000000E+00,-3.688873E-01),complex(+0.000000E+00,-2.981060E-01),0,complex(+0.000000E+00,+2.672286E-01),complex(-0.000000E+00,+2.320194E-01),complex(+0.000000E+00,-6.250000E-02),complex(+0.000000E+00,-2.742176E-01),complex(+0.000000E+00,-1.532923E-01),complex(+0.000000E+00,+1.532923E-01),complex(+0.000000E+00,+2.742176E-01),complex(+0.000000E+00,+6.250000E-02),complex(+0.000000E+00,-2.320194E-01),complex(-0.000000E+00,-2.672286E-01),0,complex(+0.000000E+00,+2.981060E-01),complex(+0.000000E+00,+3.688873E-01),complex(+0.000000E+00,+2.172802E-01)],$
            [complex(+1.347515E-01,+0.000000E+00),complex(+2.941383E-01,+0.000000E+00),complex(+3.697550E-01,+0.000000E+00),complex(+2.480392E-01,+0.000000E+00),complex(-3.314563E-02,-0.000000E+00),complex(-2.672286E-01,+0.000000E+00),complex(-2.519456E-01,-0.000000E+00),0,complex(+2.471764E-01,+0.000000E+00),complex(+2.471764E-01,+0.000000E+00),0,complex(-2.519456E-01,+0.000000E+00),complex(-2.672286E-01,+0.000000E+00),complex(-3.314563E-02,+0.000000E+00),complex(+2.480392E-01,-0.000000E+00),complex(+3.697550E-01,+0.000000E+00),complex(+2.941383E-01,-0.000000E+00),complex(+1.347515E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+7.202769E-02),complex(-0.000000E+00,+1.921621E-01),complex(+0.000000E+00,+3.211688E-01),complex(-0.000000E+00,+3.646019E-01),complex(+0.000000E+00,+2.480392E-01),0,complex(+0.000000E+00,-2.356735E-01),complex(+0.000000E+00,-2.954324E-01),complex(+0.000000E+00,-1.321214E-01),complex(+0.000000E+00,+1.321214E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,+2.356735E-01),0,complex(-0.000000E+00,-2.480392E-01),complex(+0.000000E+00,-3.646019E-01),complex(-0.000000E+00,-3.211688E-01),complex(+0.000000E+00,-1.921621E-01),complex(-0.000000E+00,-7.202769E-02)],$
            [complex(-3.221176E-02,-0.000000E+00),complex(-1.015625E-01,+0.000000E+00),complex(-2.099223E-01,-0.000000E+00),complex(-3.211688E-01,+0.000000E+00),complex(-3.697550E-01,-0.000000E+00),complex(-2.981060E-01,+0.000000E+00),complex(-1.053964E-01,-0.000000E+00),complex(+1.321214E-01,+0.000000E+00),complex(+2.954324E-01,+0.000000E+00),complex(+2.954324E-01,+0.000000E+00),complex(+1.321214E-01,-0.000000E+00),complex(-1.053964E-01,+0.000000E+00),complex(-2.981060E-01,+0.000000E+00),complex(-3.697550E-01,+0.000000E+00),complex(-3.211688E-01,+0.000000E+00),complex(-2.099223E-01,+0.000000E+00),complex(-1.015625E-01,+0.000000E+00),complex(-3.221176E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.138858E-02),complex(+0.000000E+00,-4.143204E-02),complex(+0.000000E+00,-1.015625E-01),complex(+0.000000E+00,-1.921621E-01),complex(+0.000000E+00,-2.941383E-01),complex(+0.000000E+00,-3.688873E-01),complex(+0.000000E+00,-3.726325E-01),complex(+0.000000E+00,-2.802717E-01),complex(+0.000000E+00,-1.044511E-01),complex(+0.000000E+00,+1.044511E-01),complex(+0.000000E+00,+2.802717E-01),complex(+0.000000E+00,+3.726325E-01),complex(+0.000000E+00,+3.688873E-01),complex(+0.000000E+00,+2.941383E-01),complex(+0.000000E+00,+1.921621E-01),complex(+0.000000E+00,+1.015625E-01),complex(+0.000000E+00,+4.143204E-02),complex(+0.000000E+00,+1.138858E-02)],$
            [complex(+2.762136E-03,+0.000000E+00),complex(+1.138858E-02,+0.000000E+00),complex(+3.221176E-02,+0.000000E+00),complex(+7.202769E-02,+0.000000E+00),complex(+1.347515E-01,+0.000000E+00),complex(+2.172802E-01,+0.000000E+00),complex(+3.072806E-01,+0.000000E+00),complex(+3.851967E-01,+0.000000E+00),complex(+4.306630E-01,+0.000000E+00),complex(+4.306630E-01,+0.000000E+00),complex(+3.851967E-01,-0.000000E+00),complex(+3.072806E-01,+0.000000E+00),complex(+2.172802E-01,-0.000000E+00),complex(+1.347515E-01,+0.000000E+00),complex(+7.202769E-02,-0.000000E+00),complex(+3.221176E-02,+0.000000E+00),complex(+1.138858E-02,-0.000000E+00),complex(+2.762136E-03,+0.000000E+00)]$
            ]
          end

      16: begin
            matrix[136:152,136:152]=[$
            [complex(+3.906250E-03,+0.000000E+00),complex(-1.562500E-02,-0.000000E+00),complex(+4.279083E-02,+0.000000E+00),complex(-9.243875E-02,-0.000000E+00),complex(+1.666463E-01,+0.000000E+00),complex(-2.581674E-01,-0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(-4.178044E-01,-0.000000E+00),complex(+4.431485E-01,+0.000000E+00),complex(-4.178044E-01,+0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(-2.581674E-01,+0.000000E+00),complex(+1.666463E-01,+0.000000E+00),complex(-9.243875E-02,+0.000000E+00),complex(+4.279083E-02,+0.000000E+00),complex(-1.562500E-02,+0.000000E+00),complex(+3.906250E-03,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.562500E-02),complex(+0.000000E+00,-5.468750E-02),complex(+0.000000E+00,+1.283725E-01),complex(+0.000000E+00,-2.310969E-01),complex(+0.000000E+00,+3.332926E-01),complex(+0.000000E+00,-3.872510E-01),complex(+0.000000E+00,+3.495603E-01),complex(+0.000000E+00,-2.089022E-01),0,complex(+0.000000E+00,+2.089022E-01),complex(+0.000000E+00,-3.495603E-01),complex(+0.000000E+00,+3.872510E-01),complex(+0.000000E+00,-3.332926E-01),complex(+0.000000E+00,+2.310969E-01),complex(+0.000000E+00,-1.283725E-01),complex(+0.000000E+00,+5.468750E-02),complex(+0.000000E+00,-1.562500E-02)],$
            [complex(-4.279083E-02,+0.000000E+00),complex(+1.283725E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+3.544155E-01,+0.000000E+00),complex(-3.651038E-01,+0.000000E+00),complex(+2.356735E-01,+0.000000E+00),0,complex(-2.288409E-01,-0.000000E+00),complex(+3.236299E-01,+0.000000E+00),complex(-2.288409E-01,+0.000000E+00),0,complex(+2.356735E-01,-0.000000E+00),complex(-3.651038E-01,+0.000000E+00),complex(+3.544155E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+1.283725E-01,-0.000000E+00),complex(-4.279083E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-9.243875E-02),complex(-0.000000E+00,+2.310969E-01),complex(+0.000000E+00,-3.544155E-01),complex(-0.000000E+00,+3.515625E-01),complex(+0.000000E+00,-1.690102E-01),complex(+0.000000E+00,-1.090956E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,-2.471764E-01),0,complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,-2.954324E-01),complex(+0.000000E+00,+1.090956E-01),complex(+0.000000E+00,+1.690102E-01),complex(-0.000000E+00,-3.515625E-01),complex(+0.000000E+00,+3.544155E-01),complex(-0.000000E+00,-2.310969E-01),complex(+0.000000E+00,+9.243875E-02)],$
            [complex(+1.666463E-01,+0.000000E+00),complex(-3.332926E-01,-0.000000E+00),complex(+3.651038E-01,+0.000000E+00),complex(-1.690102E-01,-0.000000E+00),complex(-1.406250E-01,+0.000000E+00),complex(+3.025768E-01,+0.000000E+00),complex(-1.638764E-01,+0.000000E+00),complex(-1.371088E-01,-0.000000E+00),complex(+2.908517E-01,+0.000000E+00),complex(-1.371088E-01,+0.000000E+00),complex(-1.638764E-01,+0.000000E+00),complex(+3.025768E-01,-0.000000E+00),complex(-1.406250E-01,+0.000000E+00),complex(-1.690102E-01,+0.000000E+00),complex(+3.651038E-01,+0.000000E+00),complex(-3.332926E-01,+0.000000E+00),complex(+1.666463E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.581674E-01),complex(+0.000000E+00,-3.872510E-01),complex(+0.000000E+00,+2.356735E-01),complex(-0.000000E+00,+1.090956E-01),complex(+0.000000E+00,-3.025768E-01),complex(-0.000000E+00,+1.328125E-01),complex(+0.000000E+00,+1.904072E-01),complex(+0.000000E+00,-2.655101E-01),0,complex(+0.000000E+00,+2.655101E-01),complex(+0.000000E+00,-1.904072E-01),complex(-0.000000E+00,-1.328125E-01),complex(+0.000000E+00,+3.025768E-01),complex(-0.000000E+00,-1.090956E-01),complex(+0.000000E+00,-2.356735E-01),complex(+0.000000E+00,+3.872510E-01),complex(+0.000000E+00,-2.581674E-01)],$
            [complex(-3.495603E-01,+0.000000E+00),complex(+3.495603E-01,+0.000000E+00),0,complex(-2.954324E-01,-0.000000E+00),complex(+1.638764E-01,+0.000000E+00),complex(+1.904072E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-6.536406E-02,-0.000000E+00),complex(+2.773162E-01,+0.000000E+00),complex(-6.536406E-02,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+1.904072E-01,-0.000000E+00),complex(+1.638764E-01,+0.000000E+00),complex(-2.954324E-01,+0.000000E+00),0,complex(+3.495603E-01,-0.000000E+00),complex(-3.495603E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.178044E-01),complex(-0.000000E+00,+2.089022E-01),complex(+0.000000E+00,+2.288409E-01),complex(+0.000000E+00,-2.471764E-01),complex(+0.000000E+00,-1.371088E-01),complex(-0.000000E+00,+2.655101E-01),complex(+0.000000E+00,+6.536406E-02),complex(+0.000000E+00,-2.734375E-01),0,complex(+0.000000E+00,+2.734375E-01),complex(+0.000000E+00,-6.536406E-02),complex(-0.000000E+00,-2.655101E-01),complex(+0.000000E+00,+1.371088E-01),complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,-2.288409E-01),complex(-0.000000E+00,-2.089022E-01),complex(+0.000000E+00,+4.178044E-01)],$
            [complex(+4.431485E-01,+0.000000E+00),0,complex(-3.236299E-01,+0.000000E+00),0,complex(+2.908517E-01,+0.000000E+00),0,complex(-2.773162E-01,+0.000000E+00),0,complex(+2.734375E-01,+0.000000E+00),0,complex(-2.773162E-01,+0.000000E+00),0,complex(+2.908517E-01,+0.000000E+00),0,complex(-3.236299E-01,+0.000000E+00),0,complex(+4.431485E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.178044E-01),complex(-0.000000E+00,+2.089022E-01),complex(+0.000000E+00,-2.288409E-01),complex(+0.000000E+00,-2.471764E-01),complex(+0.000000E+00,+1.371088E-01),complex(-0.000000E+00,+2.655101E-01),complex(+0.000000E+00,-6.536406E-02),complex(+0.000000E+00,-2.734375E-01),0,complex(+0.000000E+00,+2.734375E-01),complex(+0.000000E+00,+6.536406E-02),complex(-0.000000E+00,-2.655101E-01),complex(+0.000000E+00,-1.371088E-01),complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,+2.288409E-01),complex(-0.000000E+00,-2.089022E-01),complex(+0.000000E+00,-4.178044E-01)],$
            [complex(-3.495603E-01,+0.000000E+00),complex(-3.495603E-01,-0.000000E+00),0,complex(+2.954324E-01,+0.000000E+00),complex(+1.638764E-01,+0.000000E+00),complex(-1.904072E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+6.536406E-02,+0.000000E+00),complex(+2.773162E-01,+0.000000E+00),complex(+6.536406E-02,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-1.904072E-01,+0.000000E+00),complex(+1.638764E-01,+0.000000E+00),complex(+2.954324E-01,-0.000000E+00),0,complex(-3.495603E-01,+0.000000E+00),complex(-3.495603E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.581674E-01),complex(+0.000000E+00,-3.872510E-01),complex(+0.000000E+00,-2.356735E-01),complex(-0.000000E+00,+1.090956E-01),complex(+0.000000E+00,+3.025768E-01),complex(-0.000000E+00,+1.328125E-01),complex(+0.000000E+00,-1.904072E-01),complex(+0.000000E+00,-2.655101E-01),0,complex(+0.000000E+00,+2.655101E-01),complex(+0.000000E+00,+1.904072E-01),complex(-0.000000E+00,-1.328125E-01),complex(+0.000000E+00,-3.025768E-01),complex(-0.000000E+00,-1.090956E-01),complex(+0.000000E+00,+2.356735E-01),complex(+0.000000E+00,+3.872510E-01),complex(+0.000000E+00,+2.581674E-01)],$
            [complex(+1.666463E-01,+0.000000E+00),complex(+3.332926E-01,+0.000000E+00),complex(+3.651038E-01,+0.000000E+00),complex(+1.690102E-01,+0.000000E+00),complex(-1.406250E-01,+0.000000E+00),complex(-3.025768E-01,-0.000000E+00),complex(-1.638764E-01,+0.000000E+00),complex(+1.371088E-01,+0.000000E+00),complex(+2.908517E-01,+0.000000E+00),complex(+1.371088E-01,-0.000000E+00),complex(-1.638764E-01,+0.000000E+00),complex(-3.025768E-01,+0.000000E+00),complex(-1.406250E-01,+0.000000E+00),complex(+1.690102E-01,-0.000000E+00),complex(+3.651038E-01,+0.000000E+00),complex(+3.332926E-01,-0.000000E+00),complex(+1.666463E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+9.243875E-02),complex(-0.000000E+00,+2.310969E-01),complex(+0.000000E+00,+3.544155E-01),complex(-0.000000E+00,+3.515625E-01),complex(+0.000000E+00,+1.690102E-01),complex(+0.000000E+00,-1.090956E-01),complex(+0.000000E+00,-2.954324E-01),complex(+0.000000E+00,-2.471764E-01),0,complex(+0.000000E+00,+2.471764E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,+1.090956E-01),complex(+0.000000E+00,-1.690102E-01),complex(-0.000000E+00,-3.515625E-01),complex(+0.000000E+00,-3.544155E-01),complex(-0.000000E+00,-2.310969E-01),complex(+0.000000E+00,-9.243875E-02)],$
            [complex(-4.279083E-02,+0.000000E+00),complex(-1.283725E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-3.544155E-01,-0.000000E+00),complex(-3.651038E-01,+0.000000E+00),complex(-2.356735E-01,-0.000000E+00),0,complex(+2.288409E-01,+0.000000E+00),complex(+3.236299E-01,+0.000000E+00),complex(+2.288409E-01,-0.000000E+00),0,complex(-2.356735E-01,+0.000000E+00),complex(-3.651038E-01,+0.000000E+00),complex(-3.544155E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-1.283725E-01,+0.000000E+00),complex(-4.279083E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.562500E-02),complex(+0.000000E+00,-5.468750E-02),complex(+0.000000E+00,-1.283725E-01),complex(+0.000000E+00,-2.310969E-01),complex(+0.000000E+00,-3.332926E-01),complex(+0.000000E+00,-3.872510E-01),complex(+0.000000E+00,-3.495603E-01),complex(+0.000000E+00,-2.089022E-01),0,complex(+0.000000E+00,+2.089022E-01),complex(+0.000000E+00,+3.495603E-01),complex(+0.000000E+00,+3.872510E-01),complex(+0.000000E+00,+3.332926E-01),complex(+0.000000E+00,+2.310969E-01),complex(+0.000000E+00,+1.283725E-01),complex(+0.000000E+00,+5.468750E-02),complex(+0.000000E+00,+1.562500E-02)],$
            [complex(+3.906250E-03,+0.000000E+00),complex(+1.562500E-02,+0.000000E+00),complex(+4.279083E-02,+0.000000E+00),complex(+9.243875E-02,+0.000000E+00),complex(+1.666463E-01,+0.000000E+00),complex(+2.581674E-01,+0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(+4.178044E-01,+0.000000E+00),complex(+4.431485E-01,+0.000000E+00),complex(+4.178044E-01,-0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(+2.581674E-01,-0.000000E+00),complex(+1.666463E-01,+0.000000E+00),complex(+9.243875E-02,-0.000000E+00),complex(+4.279083E-02,+0.000000E+00),complex(+1.562500E-02,-0.000000E+00),complex(+3.906250E-03,+0.000000E+00)]$
            ]
          end

      15: begin
            matrix[120:135,120:135]=[$
            [complex(-0.000000E+00,+5.524272E-03),complex(+0.000000E+00,-2.139541E-02),complex(-0.000000E+00,+5.660694E-02),complex(+0.000000E+00,-1.178367E-01),complex(-0.000000E+00,+2.040992E-01),complex(+0.000000E+00,-3.027281E-01),complex(-0.000000E+00,+3.908203E-01),complex(+0.000000E+00,-4.431485E-01),complex(+0.000000E+00,+4.431485E-01),complex(+0.000000E+00,-3.908203E-01),complex(+0.000000E+00,+3.027281E-01),complex(+0.000000E+00,-2.040992E-01),complex(+0.000000E+00,+1.178367E-01),complex(+0.000000E+00,-5.660694E-02),complex(+0.000000E+00,+2.139541E-02),complex(+0.000000E+00,-5.524272E-03)],$
            [complex(-2.139541E-02,+0.000000E+00),complex(+7.181553E-02,+0.000000E+00),complex(-1.607743E-01,+0.000000E+00),complex(+2.738279E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+3.908203E-01,+0.000000E+00),complex(-3.027281E-01,+0.000000E+00),complex(+1.144205E-01,+0.000000E+00),complex(+1.144205E-01,+0.000000E+00),complex(-3.027281E-01,+0.000000E+00),complex(+3.908203E-01,+0.000000E+00),complex(-3.688873E-01,+0.000000E+00),complex(+2.738279E-01,+0.000000E+00),complex(-1.607743E-01,+0.000000E+00),complex(+7.181553E-02,+0.000000E+00),complex(-2.139541E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-5.660694E-02),complex(+0.000000E+00,+1.607743E-01),complex(+0.000000E+00,-2.927864E-01),complex(+0.000000E+00,+3.794897E-01),complex(+0.000000E+00,-3.386067E-01),complex(+0.000000E+00,+1.477162E-01),complex(-0.000000E+00,+1.144205E-01),complex(+0.000000E+00,-3.027281E-01),complex(+0.000000E+00,+3.027281E-01),complex(+0.000000E+00,-1.144205E-01),complex(-0.000000E+00,-1.477162E-01),complex(+0.000000E+00,+3.386067E-01),complex(-0.000000E+00,-3.794897E-01),complex(+0.000000E+00,+2.927864E-01),complex(-0.000000E+00,-1.607743E-01),complex(+0.000000E+00,+5.660694E-02)],$
            [complex(+1.178367E-01,+0.000000E+00),complex(-2.738279E-01,-0.000000E+00),complex(+3.794897E-01,+0.000000E+00),complex(-3.148835E-01,-0.000000E+00),complex(+6.697823E-02,+0.000000E+00),complex(+2.128817E-01,+0.000000E+00),complex(-3.114729E-01,+0.000000E+00),complex(+1.454259E-01,+0.000000E+00),complex(+1.454259E-01,+0.000000E+00),complex(-3.114729E-01,+0.000000E+00),complex(+2.128817E-01,+0.000000E+00),complex(+6.697823E-02,-0.000000E+00),complex(-3.148835E-01,+0.000000E+00),complex(+3.794897E-01,-0.000000E+00),complex(-2.738279E-01,+0.000000E+00),complex(+1.178367E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+2.040992E-01),complex(+0.000000E+00,-3.688873E-01),complex(-0.000000E+00,+3.386067E-01),complex(+0.000000E+00,-6.697823E-02),complex(+0.000000E+00,-2.375437E-01),complex(+0.000000E+00,+2.867837E-01),complex(+0.000000E+00,-3.173453E-02),complex(+0.000000E+00,-2.518850E-01),complex(+0.000000E+00,+2.518850E-01),complex(+0.000000E+00,+3.173453E-02),complex(-0.000000E+00,-2.867837E-01),complex(+0.000000E+00,+2.375437E-01),complex(+0.000000E+00,+6.697823E-02),complex(+0.000000E+00,-3.386067E-01),complex(+0.000000E+00,+3.688873E-01),complex(+0.000000E+00,-2.040992E-01)],$
            [complex(-3.027281E-01,+0.000000E+00),complex(+3.908203E-01,+0.000000E+00),complex(-1.477162E-01,+0.000000E+00),complex(-2.128817E-01,-0.000000E+00),complex(+2.867837E-01,+0.000000E+00),complex(+5.524272E-03,+0.000000E+00),complex(-2.781404E-01,+0.000000E+00),complex(+1.698208E-01,+0.000000E+00),complex(+1.698208E-01,+0.000000E+00),complex(-2.781404E-01,+0.000000E+00),complex(+5.524272E-03,+0.000000E+00),complex(+2.867837E-01,-0.000000E+00),complex(-2.128817E-01,+0.000000E+00),complex(-1.477162E-01,+0.000000E+00),complex(+3.908203E-01,+0.000000E+00),complex(-3.027281E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.908203E-01),complex(+0.000000E+00,+3.027281E-01),complex(-0.000000E+00,+1.144205E-01),complex(+0.000000E+00,-3.114729E-01),complex(-0.000000E+00,+3.173453E-02),complex(+0.000000E+00,+2.781404E-01),complex(+0.000000E+00,-1.381068E-01),complex(+0.000000E+00,-2.192377E-01),complex(+0.000000E+00,+2.192377E-01),complex(+0.000000E+00,+1.381068E-01),complex(-0.000000E+00,-2.781404E-01),complex(+0.000000E+00,-3.173453E-02),complex(+0.000000E+00,+3.114729E-01),complex(+0.000000E+00,-1.144205E-01),complex(-0.000000E+00,-3.027281E-01),complex(+0.000000E+00,+3.908203E-01)],$
            [complex(+4.431485E-01,+0.000000E+00),complex(-1.144205E-01,-0.000000E+00),complex(-3.027281E-01,+0.000000E+00),complex(+1.454259E-01,+0.000000E+00),complex(+2.518850E-01,+0.000000E+00),complex(-1.698208E-01,-0.000000E+00),complex(-2.192377E-01,+0.000000E+00),complex(+1.933495E-01,+0.000000E+00),complex(+1.933495E-01,+0.000000E+00),complex(-2.192377E-01,+0.000000E+00),complex(-1.698208E-01,+0.000000E+00),complex(+2.518850E-01,-0.000000E+00),complex(+1.454259E-01,+0.000000E+00),complex(-3.027281E-01,+0.000000E+00),complex(-1.144205E-01,+0.000000E+00),complex(+4.431485E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+4.431485E-01),complex(+0.000000E+00,+1.144205E-01),complex(+0.000000E+00,-3.027281E-01),complex(+0.000000E+00,-1.454259E-01),complex(-0.000000E+00,+2.518850E-01),complex(+0.000000E+00,+1.698208E-01),complex(+0.000000E+00,-2.192377E-01),complex(+0.000000E+00,-1.933495E-01),complex(+0.000000E+00,+1.933495E-01),complex(+0.000000E+00,+2.192377E-01),complex(-0.000000E+00,-1.698208E-01),complex(+0.000000E+00,-2.518850E-01),complex(+0.000000E+00,+1.454259E-01),complex(+0.000000E+00,+3.027281E-01),complex(-0.000000E+00,-1.144205E-01),complex(+0.000000E+00,-4.431485E-01)],$
            [complex(-3.908203E-01,+0.000000E+00),complex(-3.027281E-01,-0.000000E+00),complex(+1.144205E-01,+0.000000E+00),complex(+3.114729E-01,+0.000000E+00),complex(+3.173453E-02,+0.000000E+00),complex(-2.781404E-01,-0.000000E+00),complex(-1.381068E-01,+0.000000E+00),complex(+2.192377E-01,+0.000000E+00),complex(+2.192377E-01,+0.000000E+00),complex(-1.381068E-01,+0.000000E+00),complex(-2.781404E-01,+0.000000E+00),complex(+3.173453E-02,-0.000000E+00),complex(+3.114729E-01,+0.000000E+00),complex(+1.144205E-01,-0.000000E+00),complex(-3.027281E-01,+0.000000E+00),complex(-3.908203E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.027281E-01),complex(+0.000000E+00,-3.908203E-01),complex(+0.000000E+00,-1.477162E-01),complex(+0.000000E+00,+2.128817E-01),complex(-0.000000E+00,+2.867837E-01),complex(+0.000000E+00,-5.524272E-03),complex(+0.000000E+00,-2.781404E-01),complex(+0.000000E+00,-1.698208E-01),complex(+0.000000E+00,+1.698208E-01),complex(+0.000000E+00,+2.781404E-01),complex(+0.000000E+00,+5.524272E-03),complex(+0.000000E+00,-2.867837E-01),complex(-0.000000E+00,-2.128817E-01),complex(+0.000000E+00,+1.477162E-01),complex(+0.000000E+00,+3.908203E-01),complex(+0.000000E+00,+3.027281E-01)],$
            [complex(+2.040992E-01,+0.000000E+00),complex(+3.688873E-01,+0.000000E+00),complex(+3.386067E-01,+0.000000E+00),complex(+6.697823E-02,+0.000000E+00),complex(-2.375437E-01,+0.000000E+00),complex(-2.867837E-01,-0.000000E+00),complex(-3.173453E-02,+0.000000E+00),complex(+2.518850E-01,+0.000000E+00),complex(+2.518850E-01,+0.000000E+00),complex(-3.173453E-02,+0.000000E+00),complex(-2.867837E-01,+0.000000E+00),complex(-2.375437E-01,+0.000000E+00),complex(+6.697823E-02,+0.000000E+00),complex(+3.386067E-01,-0.000000E+00),complex(+3.688873E-01,+0.000000E+00),complex(+2.040992E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+1.178367E-01),complex(+0.000000E+00,+2.738279E-01),complex(-0.000000E+00,+3.794897E-01),complex(+0.000000E+00,+3.148835E-01),complex(-0.000000E+00,+6.697823E-02),complex(+0.000000E+00,-2.128817E-01),complex(+0.000000E+00,-3.114729E-01),complex(+0.000000E+00,-1.454259E-01),complex(+0.000000E+00,+1.454259E-01),complex(+0.000000E+00,+3.114729E-01),complex(+0.000000E+00,+2.128817E-01),complex(+0.000000E+00,-6.697823E-02),complex(-0.000000E+00,-3.148835E-01),complex(+0.000000E+00,-3.794897E-01),complex(-0.000000E+00,-2.738279E-01),complex(+0.000000E+00,-1.178367E-01)],$
            [complex(-5.660694E-02,+0.000000E+00),complex(-1.607743E-01,-0.000000E+00),complex(-2.927864E-01,+0.000000E+00),complex(-3.794897E-01,-0.000000E+00),complex(-3.386067E-01,+0.000000E+00),complex(-1.477162E-01,-0.000000E+00),complex(+1.144205E-01,+0.000000E+00),complex(+3.027281E-01,+0.000000E+00),complex(+3.027281E-01,+0.000000E+00),complex(+1.144205E-01,-0.000000E+00),complex(-1.477162E-01,+0.000000E+00),complex(-3.386067E-01,+0.000000E+00),complex(-3.794897E-01,+0.000000E+00),complex(-2.927864E-01,+0.000000E+00),complex(-1.607743E-01,+0.000000E+00),complex(-5.660694E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.139541E-02),complex(+0.000000E+00,-7.181553E-02),complex(+0.000000E+00,-1.607743E-01),complex(+0.000000E+00,-2.738279E-01),complex(+0.000000E+00,-3.688873E-01),complex(+0.000000E+00,-3.908203E-01),complex(+0.000000E+00,-3.027281E-01),complex(+0.000000E+00,-1.144205E-01),complex(+0.000000E+00,+1.144205E-01),complex(+0.000000E+00,+3.027281E-01),complex(+0.000000E+00,+3.908203E-01),complex(+0.000000E+00,+3.688873E-01),complex(+0.000000E+00,+2.738279E-01),complex(+0.000000E+00,+1.607743E-01),complex(+0.000000E+00,+7.181553E-02),complex(+0.000000E+00,+2.139541E-02)],$
            [complex(+5.524272E-03,+0.000000E+00),complex(+2.139541E-02,+0.000000E+00),complex(+5.660694E-02,+0.000000E+00),complex(+1.178367E-01,+0.000000E+00),complex(+2.040992E-01,+0.000000E+00),complex(+3.027281E-01,+0.000000E+00),complex(+3.908203E-01,+0.000000E+00),complex(+4.431485E-01,+0.000000E+00),complex(+4.431485E-01,+0.000000E+00),complex(+3.908203E-01,-0.000000E+00),complex(+3.027281E-01,+0.000000E+00),complex(+2.040992E-01,-0.000000E+00),complex(+1.178367E-01,+0.000000E+00),complex(+5.660694E-02,-0.000000E+00),complex(+2.139541E-02,+0.000000E+00),complex(+5.524272E-03,-0.000000E+00)]$
            ]
          end

      14: begin
            matrix[105:119,105:119]=[$
            [complex(-7.812500E-03,-0.000000E+00),complex(+2.923170E-02,+0.000000E+00),complex(-7.452650E-02,-0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(-2.471764E-01,-0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(-4.281222E-01,-0.000000E+00),complex(+4.576818E-01,+0.000000E+00),complex(-4.281222E-01,+0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(-2.471764E-01,+0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(-7.452650E-02,+0.000000E+00),complex(+2.923170E-02,+0.000000E+00),complex(-7.812500E-03,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.923170E-02),complex(+0.000000E+00,+9.375000E-02),complex(+0.000000E+00,-1.991805E-01),complex(+0.000000E+00,+3.186887E-01),complex(+0.000000E+00,-3.963641E-01),complex(+0.000000E+00,+3.736956E-01),complex(+0.000000E+00,-2.288409E-01),0,complex(+0.000000E+00,+2.288409E-01),complex(+0.000000E+00,-3.736956E-01),complex(+0.000000E+00,+3.963641E-01),complex(+0.000000E+00,-3.186887E-01),complex(+0.000000E+00,+1.991805E-01),complex(+0.000000E+00,-9.375000E-02),complex(+0.000000E+00,+2.923170E-02)],$
            [complex(+7.452650E-02,+0.000000E+00),complex(-1.991805E-01,+0.000000E+00),complex(+3.359375E-01,+0.000000E+00),complex(-3.906250E-01,+0.000000E+00),complex(+2.850224E-01,+0.000000E+00),complex(-3.664387E-02,+0.000000E+00),complex(-2.243970E-01,-0.000000E+00),complex(+3.358466E-01,+0.000000E+00),complex(-2.243970E-01,+0.000000E+00),complex(-3.664387E-02,+0.000000E+00),complex(+2.850224E-01,-0.000000E+00),complex(-3.906250E-01,+0.000000E+00),complex(+3.359375E-01,-0.000000E+00),complex(-1.991805E-01,+0.000000E+00),complex(+7.452650E-02,-0.000000E+00)],$
            [complex(-0.000000E+00,+1.490530E-01),complex(+0.000000E+00,-3.186887E-01),complex(-0.000000E+00,+3.906250E-01),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,-5.182226E-02),complex(+0.000000E+00,+2.931510E-01),complex(+0.000000E+00,-2.692764E-01),0,complex(+0.000000E+00,+2.692764E-01),complex(+0.000000E+00,-2.931510E-01),complex(+0.000000E+00,+5.182226E-02),complex(+0.000000E+00,+2.500000E-01),complex(-0.000000E+00,-3.906250E-01),complex(+0.000000E+00,+3.186887E-01),complex(-0.000000E+00,-1.490530E-01)],$
            [complex(-2.471764E-01,-0.000000E+00),complex(+3.963641E-01,+0.000000E+00),complex(-2.850224E-01,-0.000000E+00),complex(-5.182226E-02,+0.000000E+00),complex(+3.046875E-01,+0.000000E+00),complex(-2.099223E-01,+0.000000E+00),complex(-1.217848E-01,-0.000000E+00),complex(+3.037847E-01,+0.000000E+00),complex(-1.217848E-01,+0.000000E+00),complex(-2.099223E-01,+0.000000E+00),complex(+3.046875E-01,-0.000000E+00),complex(-5.182226E-02,+0.000000E+00),complex(-2.850224E-01,+0.000000E+00),complex(+3.963641E-01,+0.000000E+00),complex(-2.471764E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.495603E-01),complex(+0.000000E+00,+3.736956E-01),complex(+0.000000E+00,-3.664387E-02),complex(+0.000000E+00,-2.931510E-01),complex(-0.000000E+00,+2.099223E-01),complex(+0.000000E+00,+1.562500E-01),complex(+0.000000E+00,-2.870496E-01),0,complex(+0.000000E+00,+2.870496E-01),complex(+0.000000E+00,-1.562500E-01),complex(-0.000000E+00,-2.099223E-01),complex(+0.000000E+00,+2.931510E-01),complex(+0.000000E+00,+3.664387E-02),complex(+0.000000E+00,-3.736956E-01),complex(+0.000000E+00,+3.495603E-01)],$
            [complex(+4.281222E-01,+0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(-2.243970E-01,-0.000000E+00),complex(+2.692764E-01,+0.000000E+00),complex(+1.217848E-01,+0.000000E+00),complex(-2.870496E-01,+0.000000E+00),complex(-3.906250E-02,-0.000000E+00),complex(+2.923170E-01,+0.000000E+00),complex(-3.906250E-02,+0.000000E+00),complex(-2.870496E-01,+0.000000E+00),complex(+1.217848E-01,-0.000000E+00),complex(+2.692764E-01,+0.000000E+00),complex(-2.243970E-01,+0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(+4.281222E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+4.576818E-01),0,complex(+0.000000E+00,-3.358466E-01),0,complex(-0.000000E+00,+3.037847E-01),0,complex(+0.000000E+00,-2.923170E-01),0,complex(+0.000000E+00,+2.923170E-01),0,complex(-0.000000E+00,-3.037847E-01),0,complex(+0.000000E+00,+3.358466E-01),0,complex(-0.000000E+00,-4.576818E-01)],$
            [complex(-4.281222E-01,-0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(+2.243970E-01,+0.000000E+00),complex(+2.692764E-01,+0.000000E+00),complex(-1.217848E-01,-0.000000E+00),complex(-2.870496E-01,+0.000000E+00),complex(+3.906250E-02,+0.000000E+00),complex(+2.923170E-01,+0.000000E+00),complex(+3.906250E-02,-0.000000E+00),complex(-2.870496E-01,+0.000000E+00),complex(-1.217848E-01,+0.000000E+00),complex(+2.692764E-01,+0.000000E+00),complex(+2.243970E-01,-0.000000E+00),complex(-2.288409E-01,+0.000000E+00),complex(-4.281222E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.495603E-01),complex(+0.000000E+00,-3.736956E-01),complex(+0.000000E+00,-3.664387E-02),complex(+0.000000E+00,+2.931510E-01),complex(-0.000000E+00,+2.099223E-01),complex(+0.000000E+00,-1.562500E-01),complex(+0.000000E+00,-2.870496E-01),0,complex(+0.000000E+00,+2.870496E-01),complex(+0.000000E+00,+1.562500E-01),complex(-0.000000E+00,-2.099223E-01),complex(+0.000000E+00,-2.931510E-01),complex(+0.000000E+00,+3.664387E-02),complex(+0.000000E+00,+3.736956E-01),complex(+0.000000E+00,+3.495603E-01)],$
            [complex(+2.471764E-01,+0.000000E+00),complex(+3.963641E-01,+0.000000E+00),complex(+2.850224E-01,+0.000000E+00),complex(-5.182226E-02,+0.000000E+00),complex(-3.046875E-01,-0.000000E+00),complex(-2.099223E-01,+0.000000E+00),complex(+1.217848E-01,+0.000000E+00),complex(+3.037847E-01,+0.000000E+00),complex(+1.217848E-01,-0.000000E+00),complex(-2.099223E-01,+0.000000E+00),complex(-3.046875E-01,+0.000000E+00),complex(-5.182226E-02,+0.000000E+00),complex(+2.850224E-01,-0.000000E+00),complex(+3.963641E-01,+0.000000E+00),complex(+2.471764E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+1.490530E-01),complex(+0.000000E+00,+3.186887E-01),complex(-0.000000E+00,+3.906250E-01),complex(+0.000000E+00,+2.500000E-01),complex(+0.000000E+00,-5.182226E-02),complex(+0.000000E+00,-2.931510E-01),complex(+0.000000E+00,-2.692764E-01),0,complex(+0.000000E+00,+2.692764E-01),complex(+0.000000E+00,+2.931510E-01),complex(+0.000000E+00,+5.182226E-02),complex(+0.000000E+00,-2.500000E-01),complex(-0.000000E+00,-3.906250E-01),complex(+0.000000E+00,-3.186887E-01),complex(-0.000000E+00,-1.490530E-01)],$
            [complex(-7.452650E-02,-0.000000E+00),complex(-1.991805E-01,+0.000000E+00),complex(-3.359375E-01,-0.000000E+00),complex(-3.906250E-01,+0.000000E+00),complex(-2.850224E-01,-0.000000E+00),complex(-3.664387E-02,+0.000000E+00),complex(+2.243970E-01,+0.000000E+00),complex(+3.358466E-01,+0.000000E+00),complex(+2.243970E-01,-0.000000E+00),complex(-3.664387E-02,+0.000000E+00),complex(-2.850224E-01,+0.000000E+00),complex(-3.906250E-01,+0.000000E+00),complex(-3.359375E-01,+0.000000E+00),complex(-1.991805E-01,+0.000000E+00),complex(-7.452650E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.923170E-02),complex(+0.000000E+00,-9.375000E-02),complex(+0.000000E+00,-1.991805E-01),complex(+0.000000E+00,-3.186887E-01),complex(+0.000000E+00,-3.963641E-01),complex(+0.000000E+00,-3.736956E-01),complex(+0.000000E+00,-2.288409E-01),0,complex(+0.000000E+00,+2.288409E-01),complex(+0.000000E+00,+3.736956E-01),complex(+0.000000E+00,+3.963641E-01),complex(+0.000000E+00,+3.186887E-01),complex(+0.000000E+00,+1.991805E-01),complex(+0.000000E+00,+9.375000E-02),complex(+0.000000E+00,+2.923170E-02)],$
            [complex(+7.812500E-03,+0.000000E+00),complex(+2.923170E-02,+0.000000E+00),complex(+7.452650E-02,+0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(+2.471764E-01,+0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(+4.281222E-01,+0.000000E+00),complex(+4.576818E-01,+0.000000E+00),complex(+4.281222E-01,-0.000000E+00),complex(+3.495603E-01,+0.000000E+00),complex(+2.471764E-01,-0.000000E+00),complex(+1.490530E-01,+0.000000E+00),complex(+7.452650E-02,-0.000000E+00),complex(+2.923170E-02,+0.000000E+00),complex(+7.812500E-03,-0.000000E+00)]$
            ]
          end

      13: begin
            matrix[91:104,91:104]=[$
            [complex(+0.000000E+00,-1.104854E-02),complex(-0.000000E+00,+3.983609E-02),complex(+0.000000E+00,-9.757809E-02),complex(-0.000000E+00,+1.868478E-01),complex(+0.000000E+00,-2.954324E-01),complex(-0.000000E+00,+3.963641E-01),complex(+0.000000E+00,-4.576818E-01),complex(+0.000000E+00,+4.576818E-01),complex(+0.000000E+00,-3.963641E-01),complex(+0.000000E+00,+2.954324E-01),complex(+0.000000E+00,-1.868478E-01),complex(+0.000000E+00,+9.757809E-02),complex(+0.000000E+00,-3.983609E-02),complex(+0.000000E+00,+1.104854E-02)],$
            [complex(+3.983609E-02,+0.000000E+00),complex(-1.215340E-01,+0.000000E+00),complex(+2.435696E-01,+0.000000E+00),complex(-3.627558E-01,+0.000000E+00),complex(+4.096909E-01,+0.000000E+00),complex(-3.297949E-01,+0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(-3.297949E-01,+0.000000E+00),complex(+4.096909E-01,+0.000000E+00),complex(-3.627558E-01,+0.000000E+00),complex(+2.435696E-01,+0.000000E+00),complex(-1.215340E-01,+0.000000E+00),complex(+3.983609E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+9.757809E-02),complex(+0.000000E+00,-2.435696E-01),complex(+0.000000E+00,+3.756505E-01),complex(+0.000000E+00,-3.808143E-01),complex(+0.000000E+00,+2.007068E-01),complex(-0.000000E+00,+8.975879E-02),complex(+0.000000E+00,-3.109336E-01),complex(+0.000000E+00,+3.109336E-01),complex(+0.000000E+00,-8.975879E-02),complex(-0.000000E+00,-2.007068E-01),complex(+0.000000E+00,+3.808143E-01),complex(-0.000000E+00,-3.756505E-01),complex(+0.000000E+00,+2.435696E-01),complex(-0.000000E+00,-9.757809E-02)],$
            [complex(-1.868478E-01,-0.000000E+00),complex(+3.627558E-01,+0.000000E+00),complex(-3.808143E-01,-0.000000E+00),complex(+1.546796E-01,+0.000000E+00),complex(+1.746928E-01,+0.000000E+00),complex(-3.281250E-01,+0.000000E+00),complex(+1.623798E-01,+0.000000E+00),complex(+1.623798E-01,+0.000000E+00),complex(-3.281250E-01,+0.000000E+00),complex(+1.746928E-01,+0.000000E+00),complex(+1.546796E-01,-0.000000E+00),complex(-3.808143E-01,+0.000000E+00),complex(+3.627558E-01,-0.000000E+00),complex(-1.868478E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.954324E-01),complex(-0.000000E+00,+4.096909E-01),complex(+0.000000E+00,-2.007068E-01),complex(+0.000000E+00,-1.746928E-01),complex(+0.000000E+00,+3.204077E-01),complex(+0.000000E+00,-7.411588E-02),complex(+0.000000E+00,-2.567450E-01),complex(+0.000000E+00,+2.567450E-01),complex(+0.000000E+00,+7.411588E-02),complex(-0.000000E+00,-3.204077E-01),complex(+0.000000E+00,+1.746928E-01),complex(+0.000000E+00,+2.007068E-01),complex(+0.000000E+00,-4.096909E-01),complex(+0.000000E+00,+2.954324E-01)],$
            [complex(+3.963641E-01,+0.000000E+00),complex(-3.297949E-01,+0.000000E+00),complex(-8.975879E-02,-0.000000E+00),complex(+3.281250E-01,+0.000000E+00),complex(-7.411588E-02,-0.000000E+00),complex(-2.762136E-01,+0.000000E+00),complex(+1.913664E-01,+0.000000E+00),complex(+1.913664E-01,+0.000000E+00),complex(-2.762136E-01,+0.000000E+00),complex(-7.411588E-02,+0.000000E+00),complex(+3.281250E-01,-0.000000E+00),complex(-8.975879E-02,+0.000000E+00),complex(-3.297949E-01,+0.000000E+00),complex(+3.963641E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.576818E-01),complex(+0.000000E+00,-1.269381E-01),complex(+0.000000E+00,-3.109336E-01),complex(-0.000000E+00,+1.623798E-01),complex(+0.000000E+00,+2.567450E-01),complex(+0.000000E+00,-1.913664E-01),complex(+0.000000E+00,-2.209709E-01),complex(+0.000000E+00,+2.209709E-01),complex(+0.000000E+00,+1.913664E-01),complex(-0.000000E+00,-2.567450E-01),complex(+0.000000E+00,-1.623798E-01),complex(+0.000000E+00,+3.109336E-01),complex(+0.000000E+00,+1.269381E-01),complex(-0.000000E+00,-4.576818E-01)],$
            [complex(-4.576818E-01,-0.000000E+00),complex(-1.269381E-01,+0.000000E+00),complex(+3.109336E-01,+0.000000E+00),complex(+1.623798E-01,+0.000000E+00),complex(-2.567450E-01,-0.000000E+00),complex(-1.913664E-01,+0.000000E+00),complex(+2.209709E-01,+0.000000E+00),complex(+2.209709E-01,+0.000000E+00),complex(-1.913664E-01,+0.000000E+00),complex(-2.567450E-01,+0.000000E+00),complex(+1.623798E-01,-0.000000E+00),complex(+3.109336E-01,+0.000000E+00),complex(-1.269381E-01,+0.000000E+00),complex(-4.576818E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.963641E-01),complex(+0.000000E+00,-3.297949E-01),complex(+0.000000E+00,+8.975879E-02),complex(-0.000000E+00,+3.281250E-01),complex(+0.000000E+00,+7.411588E-02),complex(+0.000000E+00,-2.762136E-01),complex(+0.000000E+00,-1.913664E-01),complex(+0.000000E+00,+1.913664E-01),complex(+0.000000E+00,+2.762136E-01),complex(-0.000000E+00,-7.411588E-02),complex(+0.000000E+00,-3.281250E-01),complex(-0.000000E+00,-8.975879E-02),complex(+0.000000E+00,+3.297949E-01),complex(+0.000000E+00,+3.963641E-01)],$
            [complex(+2.954324E-01,+0.000000E+00),complex(+4.096909E-01,+0.000000E+00),complex(+2.007068E-01,+0.000000E+00),complex(-1.746928E-01,+0.000000E+00),complex(-3.204077E-01,-0.000000E+00),complex(-7.411588E-02,+0.000000E+00),complex(+2.567450E-01,+0.000000E+00),complex(+2.567450E-01,+0.000000E+00),complex(-7.411588E-02,+0.000000E+00),complex(-3.204077E-01,+0.000000E+00),complex(-1.746928E-01,+0.000000E+00),complex(+2.007068E-01,+0.000000E+00),complex(+4.096909E-01,-0.000000E+00),complex(+2.954324E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.868478E-01),complex(-0.000000E+00,+3.627558E-01),complex(+0.000000E+00,+3.808143E-01),complex(-0.000000E+00,+1.546796E-01),complex(+0.000000E+00,-1.746928E-01),complex(+0.000000E+00,-3.281250E-01),complex(+0.000000E+00,-1.623798E-01),complex(+0.000000E+00,+1.623798E-01),complex(+0.000000E+00,+3.281250E-01),complex(+0.000000E+00,+1.746928E-01),complex(+0.000000E+00,-1.546796E-01),complex(-0.000000E+00,-3.808143E-01),complex(+0.000000E+00,-3.627558E-01),complex(-0.000000E+00,-1.868478E-01)],$
            [complex(-9.757809E-02,-0.000000E+00),complex(-2.435696E-01,+0.000000E+00),complex(-3.756505E-01,-0.000000E+00),complex(-3.808143E-01,+0.000000E+00),complex(-2.007068E-01,-0.000000E+00),complex(+8.975879E-02,+0.000000E+00),complex(+3.109336E-01,+0.000000E+00),complex(+3.109336E-01,+0.000000E+00),complex(+8.975879E-02,-0.000000E+00),complex(-2.007068E-01,+0.000000E+00),complex(-3.808143E-01,+0.000000E+00),complex(-3.756505E-01,+0.000000E+00),complex(-2.435696E-01,+0.000000E+00),complex(-9.757809E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.983609E-02),complex(+0.000000E+00,-1.215340E-01),complex(+0.000000E+00,-2.435696E-01),complex(+0.000000E+00,-3.627558E-01),complex(+0.000000E+00,-4.096909E-01),complex(+0.000000E+00,-3.297949E-01),complex(+0.000000E+00,-1.269381E-01),complex(+0.000000E+00,+1.269381E-01),complex(+0.000000E+00,+3.297949E-01),complex(+0.000000E+00,+4.096909E-01),complex(+0.000000E+00,+3.627558E-01),complex(+0.000000E+00,+2.435696E-01),complex(+0.000000E+00,+1.215340E-01),complex(+0.000000E+00,+3.983609E-02)],$
            [complex(+1.104854E-02,+0.000000E+00),complex(+3.983609E-02,+0.000000E+00),complex(+9.757809E-02,+0.000000E+00),complex(+1.868478E-01,+0.000000E+00),complex(+2.954324E-01,+0.000000E+00),complex(+3.963641E-01,+0.000000E+00),complex(+4.576818E-01,+0.000000E+00),complex(+4.576818E-01,+0.000000E+00),complex(+3.963641E-01,-0.000000E+00),complex(+2.954324E-01,+0.000000E+00),complex(+1.868478E-01,-0.000000E+00),complex(+9.757809E-02,+0.000000E+00),complex(+3.983609E-02,-0.000000E+00),complex(+1.104854E-02,+0.000000E+00)]$
            ]
          end

      12: begin
            matrix[78:90,78:90]=[$
            [complex(+1.562500E-02,+0.000000E+00),complex(-5.412659E-02,-0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(-2.317562E-01,-0.000000E+00),complex(+3.476343E-01,+0.000000E+00),complex(-4.397265E-01,-0.000000E+00),complex(+4.749589E-01,+0.000000E+00),complex(-4.397265E-01,+0.000000E+00),complex(+3.476343E-01,+0.000000E+00),complex(-2.317562E-01,+0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(-5.412659E-02,+0.000000E+00),complex(+1.562500E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+5.412659E-02),complex(+0.000000E+00,-1.562500E-01),complex(+0.000000E+00,+2.931510E-01),complex(+0.000000E+00,-4.014135E-01),complex(+0.000000E+00,+4.014135E-01),complex(+0.000000E+00,-2.538762E-01),0,complex(+0.000000E+00,+2.538762E-01),complex(+0.000000E+00,-4.014135E-01),complex(+0.000000E+00,+4.014135E-01),complex(+0.000000E+00,-2.931510E-01),complex(+0.000000E+00,+1.562500E-01),complex(+0.000000E+00,-5.412659E-02)],$
            [complex(-1.269381E-01,+0.000000E+00),complex(+2.931510E-01,+0.000000E+00),complex(-4.062500E-01,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(-8.558165E-02,+0.000000E+00),complex(-2.165063E-01,-0.000000E+00),complex(+3.507804E-01,+0.000000E+00),complex(-2.165063E-01,+0.000000E+00),complex(-8.558165E-02,+0.000000E+00),complex(+3.423266E-01,-0.000000E+00),complex(-4.062500E-01,+0.000000E+00),complex(+2.931510E-01,-0.000000E+00),complex(-1.269381E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.317562E-01),complex(-0.000000E+00,+4.014135E-01),complex(+0.000000E+00,-3.423266E-01),complex(-0.000000E+00,+3.125000E-02),complex(+0.000000E+00,+2.812500E-01),complex(+0.000000E+00,-2.964635E-01),0,complex(+0.000000E+00,+2.964635E-01),complex(+0.000000E+00,-2.812500E-01),complex(-0.000000E+00,-3.125000E-02),complex(+0.000000E+00,+3.423266E-01),complex(-0.000000E+00,-4.014135E-01),complex(+0.000000E+00,+2.317562E-01)],$
            [complex(+3.476343E-01,+0.000000E+00),complex(-4.014135E-01,-0.000000E+00),complex(+8.558165E-02,+0.000000E+00),complex(+2.812500E-01,+0.000000E+00),complex(-2.656250E-01,+0.000000E+00),complex(-9.882118E-02,-0.000000E+00),complex(+3.202172E-01,+0.000000E+00),complex(-9.882118E-02,+0.000000E+00),complex(-2.656250E-01,+0.000000E+00),complex(+2.812500E-01,-0.000000E+00),complex(+8.558165E-02,+0.000000E+00),complex(-4.014135E-01,+0.000000E+00),complex(+3.476343E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.397265E-01),complex(+0.000000E+00,-2.538762E-01),complex(+0.000000E+00,-2.165063E-01),complex(-0.000000E+00,+2.964635E-01),complex(+0.000000E+00,+9.882118E-02),complex(+0.000000E+00,-3.125000E-01),0,complex(+0.000000E+00,+3.125000E-01),complex(+0.000000E+00,-9.882118E-02),complex(-0.000000E+00,-2.964635E-01),complex(+0.000000E+00,+2.165063E-01),complex(+0.000000E+00,+2.538762E-01),complex(+0.000000E+00,-4.397265E-01)],$
            [complex(-4.749589E-01,+0.000000E+00),0,complex(+3.507804E-01,+0.000000E+00),0,complex(-3.202172E-01,+0.000000E+00),0,complex(+3.125000E-01,+0.000000E+00),0,complex(-3.202172E-01,+0.000000E+00),0,complex(+3.507804E-01,+0.000000E+00),0,complex(-4.749589E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.397265E-01),complex(+0.000000E+00,-2.538762E-01),complex(+0.000000E+00,+2.165063E-01),complex(-0.000000E+00,+2.964635E-01),complex(+0.000000E+00,-9.882118E-02),complex(+0.000000E+00,-3.125000E-01),0,complex(+0.000000E+00,+3.125000E-01),complex(+0.000000E+00,+9.882118E-02),complex(-0.000000E+00,-2.964635E-01),complex(+0.000000E+00,-2.165063E-01),complex(+0.000000E+00,+2.538762E-01),complex(+0.000000E+00,+4.397265E-01)],$
            [complex(+3.476343E-01,+0.000000E+00),complex(+4.014135E-01,+0.000000E+00),complex(+8.558165E-02,+0.000000E+00),complex(-2.812500E-01,-0.000000E+00),complex(-2.656250E-01,+0.000000E+00),complex(+9.882118E-02,+0.000000E+00),complex(+3.202172E-01,+0.000000E+00),complex(+9.882118E-02,-0.000000E+00),complex(-2.656250E-01,+0.000000E+00),complex(-2.812500E-01,+0.000000E+00),complex(+8.558165E-02,+0.000000E+00),complex(+4.014135E-01,-0.000000E+00),complex(+3.476343E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.317562E-01),complex(-0.000000E+00,+4.014135E-01),complex(+0.000000E+00,+3.423266E-01),complex(-0.000000E+00,+3.125000E-02),complex(+0.000000E+00,-2.812500E-01),complex(+0.000000E+00,-2.964635E-01),0,complex(+0.000000E+00,+2.964635E-01),complex(+0.000000E+00,+2.812500E-01),complex(-0.000000E+00,-3.125000E-02),complex(+0.000000E+00,-3.423266E-01),complex(-0.000000E+00,-4.014135E-01),complex(+0.000000E+00,-2.317562E-01)],$
            [complex(-1.269381E-01,+0.000000E+00),complex(-2.931510E-01,-0.000000E+00),complex(-4.062500E-01,+0.000000E+00),complex(-3.423266E-01,-0.000000E+00),complex(-8.558165E-02,+0.000000E+00),complex(+2.165063E-01,+0.000000E+00),complex(+3.507804E-01,+0.000000E+00),complex(+2.165063E-01,-0.000000E+00),complex(-8.558165E-02,+0.000000E+00),complex(-3.423266E-01,+0.000000E+00),complex(-4.062500E-01,+0.000000E+00),complex(-2.931510E-01,+0.000000E+00),complex(-1.269381E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-5.412659E-02),complex(+0.000000E+00,-1.562500E-01),complex(+0.000000E+00,-2.931510E-01),complex(+0.000000E+00,-4.014135E-01),complex(+0.000000E+00,-4.014135E-01),complex(+0.000000E+00,-2.538762E-01),0,complex(+0.000000E+00,+2.538762E-01),complex(+0.000000E+00,+4.014135E-01),complex(+0.000000E+00,+4.014135E-01),complex(+0.000000E+00,+2.931510E-01),complex(+0.000000E+00,+1.562500E-01),complex(+0.000000E+00,+5.412659E-02)],$
            [complex(+1.562500E-02,+0.000000E+00),complex(+5.412659E-02,+0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(+2.317562E-01,+0.000000E+00),complex(+3.476343E-01,+0.000000E+00),complex(+4.397265E-01,+0.000000E+00),complex(+4.749589E-01,+0.000000E+00),complex(+4.397265E-01,-0.000000E+00),complex(+3.476343E-01,+0.000000E+00),complex(+2.317562E-01,-0.000000E+00),complex(+1.269381E-01,+0.000000E+00),complex(+5.412659E-02,-0.000000E+00),complex(+1.562500E-02,+0.000000E+00)]$
            ]
          end

      11: begin
            matrix[66:77,66:77]=[$
            [complex(-0.000000E+00,+2.209709E-02),complex(+0.000000E+00,-7.328775E-02),complex(-0.000000E+00,+1.638764E-01),complex(+0.000000E+00,-2.838422E-01),complex(-0.000000E+00,+4.014135E-01),complex(+0.000000E+00,-4.749589E-01),complex(+0.000000E+00,+4.749589E-01),complex(+0.000000E+00,-4.014135E-01),complex(+0.000000E+00,+2.838422E-01),complex(+0.000000E+00,-1.638764E-01),complex(+0.000000E+00,+7.328775E-02),complex(+0.000000E+00,-2.209709E-02)],$
            [complex(-7.328775E-02,+0.000000E+00),complex(+1.988738E-01,+0.000000E+00),complex(-3.458741E-01,+0.000000E+00),complex(+4.279082E-01,+0.000000E+00),complex(-3.630922E-01,+0.000000E+00),complex(+1.432055E-01,+0.000000E+00),complex(+1.432055E-01,+0.000000E+00),complex(-3.630922E-01,+0.000000E+00),complex(+4.279082E-01,+0.000000E+00),complex(-3.458741E-01,+0.000000E+00),complex(+1.988738E-01,+0.000000E+00),complex(-7.328775E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.638764E-01),complex(+0.000000E+00,+3.458741E-01),complex(+0.000000E+00,-4.198447E-01),complex(+0.000000E+00,+2.679129E-01),complex(-0.000000E+00,+5.412659E-02),complex(+0.000000E+00,-3.202172E-01),complex(+0.000000E+00,+3.202172E-01),complex(+0.000000E+00,-5.412659E-02),complex(-0.000000E+00,-2.679129E-01),complex(+0.000000E+00,+4.198447E-01),complex(-0.000000E+00,-3.458741E-01),complex(+0.000000E+00,+1.638764E-01)],$
            [complex(+2.838422E-01,+0.000000E+00),complex(-4.279082E-01,-0.000000E+00),complex(+2.679129E-01,+0.000000E+00),complex(+1.104854E-01,+0.000000E+00),complex(-3.437500E-01,+0.000000E+00),complex(+1.848775E-01,+0.000000E+00),complex(+1.848775E-01,+0.000000E+00),complex(-3.437500E-01,+0.000000E+00),complex(+1.104854E-01,+0.000000E+00),complex(+2.679129E-01,-0.000000E+00),complex(-4.279082E-01,+0.000000E+00),complex(+2.838422E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+4.014135E-01),complex(+0.000000E+00,-3.630922E-01),complex(+0.000000E+00,-5.412659E-02),complex(+0.000000E+00,+3.437500E-01),complex(+0.000000E+00,-1.325825E-01),complex(+0.000000E+00,-2.614563E-01),complex(+0.000000E+00,+2.614563E-01),complex(+0.000000E+00,+1.325825E-01),complex(-0.000000E+00,-3.437500E-01),complex(+0.000000E+00,+5.412659E-02),complex(+0.000000E+00,+3.630922E-01),complex(+0.000000E+00,-4.014135E-01)],$
            [complex(-4.749589E-01,+0.000000E+00),complex(+1.432055E-01,+0.000000E+00),complex(+3.202172E-01,+0.000000E+00),complex(-1.848775E-01,-0.000000E+00),complex(-2.614563E-01,+0.000000E+00),complex(+2.209709E-01,+0.000000E+00),complex(+2.209709E-01,+0.000000E+00),complex(-2.614563E-01,+0.000000E+00),complex(-1.848775E-01,+0.000000E+00),complex(+3.202172E-01,-0.000000E+00),complex(+1.432055E-01,+0.000000E+00),complex(-4.749589E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.749589E-01),complex(+0.000000E+00,-1.432055E-01),complex(-0.000000E+00,+3.202172E-01),complex(+0.000000E+00,+1.848775E-01),complex(+0.000000E+00,-2.614563E-01),complex(+0.000000E+00,-2.209709E-01),complex(+0.000000E+00,+2.209709E-01),complex(+0.000000E+00,+2.614563E-01),complex(-0.000000E+00,-1.848775E-01),complex(+0.000000E+00,-3.202172E-01),complex(+0.000000E+00,+1.432055E-01),complex(+0.000000E+00,+4.749589E-01)],$
            [complex(+4.014135E-01,+0.000000E+00),complex(+3.630922E-01,+0.000000E+00),complex(-5.412659E-02,+0.000000E+00),complex(-3.437500E-01,-0.000000E+00),complex(-1.325825E-01,+0.000000E+00),complex(+2.614563E-01,+0.000000E+00),complex(+2.614563E-01,+0.000000E+00),complex(-1.325825E-01,+0.000000E+00),complex(-3.437500E-01,+0.000000E+00),complex(-5.412659E-02,+0.000000E+00),complex(+3.630922E-01,+0.000000E+00),complex(+4.014135E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+2.838422E-01),complex(+0.000000E+00,+4.279082E-01),complex(-0.000000E+00,+2.679129E-01),complex(+0.000000E+00,-1.104854E-01),complex(+0.000000E+00,-3.437500E-01),complex(+0.000000E+00,-1.848775E-01),complex(+0.000000E+00,+1.848775E-01),complex(+0.000000E+00,+3.437500E-01),complex(+0.000000E+00,+1.104854E-01),complex(+0.000000E+00,-2.679129E-01),complex(-0.000000E+00,-4.279082E-01),complex(+0.000000E+00,-2.838422E-01)],$
            [complex(-1.638764E-01,+0.000000E+00),complex(-3.458741E-01,-0.000000E+00),complex(-4.198447E-01,+0.000000E+00),complex(-2.679129E-01,-0.000000E+00),complex(+5.412659E-02,+0.000000E+00),complex(+3.202172E-01,+0.000000E+00),complex(+3.202172E-01,+0.000000E+00),complex(+5.412659E-02,-0.000000E+00),complex(-2.679129E-01,+0.000000E+00),complex(-4.198447E-01,+0.000000E+00),complex(-3.458741E-01,+0.000000E+00),complex(-1.638764E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-7.328775E-02),complex(+0.000000E+00,-1.988738E-01),complex(+0.000000E+00,-3.458741E-01),complex(+0.000000E+00,-4.279082E-01),complex(+0.000000E+00,-3.630922E-01),complex(+0.000000E+00,-1.432055E-01),complex(+0.000000E+00,+1.432055E-01),complex(+0.000000E+00,+3.630922E-01),complex(+0.000000E+00,+4.279082E-01),complex(+0.000000E+00,+3.458741E-01),complex(+0.000000E+00,+1.988738E-01),complex(+0.000000E+00,+7.328775E-02)],$
            [complex(+2.209709E-02,+0.000000E+00),complex(+7.328775E-02,+0.000000E+00),complex(+1.638764E-01,+0.000000E+00),complex(+2.838422E-01,+0.000000E+00),complex(+4.014135E-01,+0.000000E+00),complex(+4.749589E-01,+0.000000E+00),complex(+4.749589E-01,+0.000000E+00),complex(+4.014135E-01,-0.000000E+00),complex(+2.838422E-01,+0.000000E+00),complex(+1.638764E-01,-0.000000E+00),complex(+7.328775E-02,+0.000000E+00),complex(+2.209709E-02,-0.000000E+00)]$
            ]
          end

      10: begin
            matrix[55:65,55:65]=[$
            [complex(-3.125000E-02,-0.000000E+00),complex(+9.882118E-02,+0.000000E+00),complex(-2.096314E-01,-0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(-4.528555E-01,-0.000000E+00),complex(+4.960784E-01,+0.000000E+00),complex(-4.528555E-01,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(-2.096314E-01,+0.000000E+00),complex(+9.882118E-02,+0.000000E+00),complex(-3.125000E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-9.882118E-02),complex(+0.000000E+00,+2.500000E-01),complex(+0.000000E+00,-3.977476E-01),complex(+0.000000E+00,+4.330127E-01),complex(+0.000000E+00,-2.864110E-01),0,complex(+0.000000E+00,+2.864110E-01),complex(+0.000000E+00,-4.330127E-01),complex(+0.000000E+00,+3.977476E-01),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,+9.882118E-02)],$
            [complex(+2.096314E-01,+0.000000E+00),complex(-3.977476E-01,+0.000000E+00),complex(+4.062500E-01,+0.000000E+00),complex(-1.530931E-01,+0.000000E+00),complex(-2.025231E-01,-0.000000E+00),complex(+3.697550E-01,+0.000000E+00),complex(-2.025231E-01,+0.000000E+00),complex(-1.530931E-01,+0.000000E+00),complex(+4.062500E-01,-0.000000E+00),complex(-3.977476E-01,+0.000000E+00),complex(+2.096314E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.423266E-01),complex(+0.000000E+00,-4.330127E-01),complex(-0.000000E+00,+1.530931E-01),complex(+0.000000E+00,+2.500000E-01),complex(+0.000000E+00,-3.307189E-01),0,complex(+0.000000E+00,+3.307189E-01),complex(+0.000000E+00,-2.500000E-01),complex(-0.000000E+00,-1.530931E-01),complex(+0.000000E+00,+4.330127E-01),complex(-0.000000E+00,-3.423266E-01)],$
            [complex(-4.528555E-01,-0.000000E+00),complex(+2.864110E-01,+0.000000E+00),complex(+2.025231E-01,+0.000000E+00),complex(-3.307189E-01,+0.000000E+00),complex(-6.250000E-02,-0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(-6.250000E-02,+0.000000E+00),complex(-3.307189E-01,+0.000000E+00),complex(+2.025231E-01,-0.000000E+00),complex(+2.864110E-01,+0.000000E+00),complex(-4.528555E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.960784E-01),0,complex(-0.000000E+00,+3.697550E-01),0,complex(+0.000000E+00,-3.423266E-01),0,complex(+0.000000E+00,+3.423266E-01),0,complex(-0.000000E+00,-3.697550E-01),0,complex(+0.000000E+00,+4.960784E-01)],$
            [complex(+4.528555E-01,+0.000000E+00),complex(+2.864110E-01,+0.000000E+00),complex(-2.025231E-01,-0.000000E+00),complex(-3.307189E-01,+0.000000E+00),complex(+6.250000E-02,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(+6.250000E-02,-0.000000E+00),complex(-3.307189E-01,+0.000000E+00),complex(-2.025231E-01,+0.000000E+00),complex(+2.864110E-01,+0.000000E+00),complex(+4.528555E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.423266E-01),complex(+0.000000E+00,+4.330127E-01),complex(-0.000000E+00,+1.530931E-01),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,-3.307189E-01),0,complex(+0.000000E+00,+3.307189E-01),complex(+0.000000E+00,+2.500000E-01),complex(-0.000000E+00,-1.530931E-01),complex(+0.000000E+00,-4.330127E-01),complex(-0.000000E+00,-3.423266E-01)],$
            [complex(-2.096314E-01,-0.000000E+00),complex(-3.977476E-01,+0.000000E+00),complex(-4.062500E-01,-0.000000E+00),complex(-1.530931E-01,+0.000000E+00),complex(+2.025231E-01,+0.000000E+00),complex(+3.697550E-01,+0.000000E+00),complex(+2.025231E-01,-0.000000E+00),complex(-1.530931E-01,+0.000000E+00),complex(-4.062500E-01,+0.000000E+00),complex(-3.977476E-01,+0.000000E+00),complex(-2.096314E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-9.882118E-02),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,-3.977476E-01),complex(+0.000000E+00,-4.330127E-01),complex(+0.000000E+00,-2.864110E-01),0,complex(+0.000000E+00,+2.864110E-01),complex(+0.000000E+00,+4.330127E-01),complex(+0.000000E+00,+3.977476E-01),complex(+0.000000E+00,+2.500000E-01),complex(+0.000000E+00,+9.882118E-02)],$
            [complex(+3.125000E-02,+0.000000E+00),complex(+9.882118E-02,+0.000000E+00),complex(+2.096314E-01,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(+4.528555E-01,+0.000000E+00),complex(+4.960784E-01,+0.000000E+00),complex(+4.528555E-01,-0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(+2.096314E-01,-0.000000E+00),complex(+9.882118E-02,+0.000000E+00),complex(+3.125000E-02,-0.000000E+00)]$
            ]
          end

       9: begin
            matrix[45:54,45:54]=[$
            [complex(+0.000000E+00,-4.419417E-02),complex(-0.000000E+00,+1.325825E-01),complex(+0.000000E+00,-2.651650E-01),complex(-0.000000E+00,+4.050463E-01),complex(+0.000000E+00,-4.960784E-01),complex(+0.000000E+00,+4.960784E-01),complex(+0.000000E+00,-4.050463E-01),complex(+0.000000E+00,+2.651650E-01),complex(+0.000000E+00,-1.325825E-01),complex(+0.000000E+00,+4.419417E-02)],$
            [complex(+1.325825E-01,+0.000000E+00),complex(-3.093592E-01,+0.000000E+00),complex(+4.419417E-01,+0.000000E+00),complex(-4.050463E-01,+0.000000E+00),complex(+1.653595E-01,+0.000000E+00),complex(+1.653595E-01,+0.000000E+00),complex(-4.050463E-01,+0.000000E+00),complex(+4.419417E-01,+0.000000E+00),complex(-3.093592E-01,+0.000000E+00),complex(+1.325825E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.651650E-01),complex(+0.000000E+00,-4.419417E-01),complex(+0.000000E+00,+3.535534E-01),0,complex(+0.000000E+00,-3.307189E-01),complex(+0.000000E+00,+3.307189E-01),0,complex(-0.000000E+00,-3.535534E-01),complex(+0.000000E+00,+4.419417E-01),complex(-0.000000E+00,-2.651650E-01)],$
            [complex(-4.050463E-01,-0.000000E+00),complex(+4.050463E-01,+0.000000E+00),0,complex(-3.535534E-01,+0.000000E+00),complex(+2.165063E-01,+0.000000E+00),complex(+2.165063E-01,+0.000000E+00),complex(-3.535534E-01,+0.000000E+00),0,complex(+4.050463E-01,-0.000000E+00),complex(-4.050463E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.960784E-01),complex(-0.000000E+00,+1.653595E-01),complex(+0.000000E+00,+3.307189E-01),complex(+0.000000E+00,-2.165063E-01),complex(+0.000000E+00,-2.651650E-01),complex(+0.000000E+00,+2.651650E-01),complex(+0.000000E+00,+2.165063E-01),complex(-0.000000E+00,-3.307189E-01),complex(+0.000000E+00,-1.653595E-01),complex(+0.000000E+00,+4.960784E-01)],$
            [complex(+4.960784E-01,+0.000000E+00),complex(+1.653595E-01,+0.000000E+00),complex(-3.307189E-01,-0.000000E+00),complex(-2.165063E-01,+0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(-2.165063E-01,+0.000000E+00),complex(-3.307189E-01,+0.000000E+00),complex(+1.653595E-01,-0.000000E+00),complex(+4.960784E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.050463E-01),complex(-0.000000E+00,+4.050463E-01),0,complex(+0.000000E+00,-3.535534E-01),complex(+0.000000E+00,-2.165063E-01),complex(+0.000000E+00,+2.165063E-01),complex(+0.000000E+00,+3.535534E-01),0,complex(+0.000000E+00,-4.050463E-01),complex(-0.000000E+00,-4.050463E-01)],$
            [complex(-2.651650E-01,-0.000000E+00),complex(-4.419417E-01,+0.000000E+00),complex(-3.535534E-01,-0.000000E+00),0,complex(+3.307189E-01,+0.000000E+00),complex(+3.307189E-01,+0.000000E+00),0,complex(-3.535534E-01,+0.000000E+00),complex(-4.419417E-01,+0.000000E+00),complex(-2.651650E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.325825E-01),complex(+0.000000E+00,-3.093592E-01),complex(+0.000000E+00,-4.419417E-01),complex(+0.000000E+00,-4.050463E-01),complex(+0.000000E+00,-1.653595E-01),complex(+0.000000E+00,+1.653595E-01),complex(+0.000000E+00,+4.050463E-01),complex(+0.000000E+00,+4.419417E-01),complex(+0.000000E+00,+3.093592E-01),complex(+0.000000E+00,+1.325825E-01)],$
            [complex(+4.419417E-02,+0.000000E+00),complex(+1.325825E-01,+0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(+4.050463E-01,+0.000000E+00),complex(+4.960784E-01,+0.000000E+00),complex(+4.960784E-01,+0.000000E+00),complex(+4.050463E-01,-0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(+1.325825E-01,-0.000000E+00),complex(+4.419417E-02,+0.000000E+00)]$
            ]
          end

       8: begin
            matrix[36:44,36:44]=[$
            [complex(+6.250000E-02,+0.000000E+00),complex(-1.767767E-01,-0.000000E+00),complex(+3.307189E-01,+0.000000E+00),complex(-4.677072E-01,-0.000000E+00),complex(+5.229125E-01,+0.000000E+00),complex(-4.677072E-01,+0.000000E+00),complex(+3.307189E-01,+0.000000E+00),complex(-1.767767E-01,+0.000000E+00),complex(+6.250000E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.767767E-01),complex(+0.000000E+00,-3.750000E-01),complex(+0.000000E+00,+4.677072E-01),complex(+0.000000E+00,-3.307189E-01),0,complex(+0.000000E+00,+3.307189E-01),complex(+0.000000E+00,-4.677072E-01),complex(+0.000000E+00,+3.750000E-01),complex(+0.000000E+00,-1.767767E-01)],$
            [complex(-3.307189E-01,+0.000000E+00),complex(+4.677072E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-1.767767E-01,-0.000000E+00),complex(+3.952847E-01,+0.000000E+00),complex(-1.767767E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+4.677072E-01,-0.000000E+00),complex(-3.307189E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.677072E-01),complex(-0.000000E+00,+3.307189E-01),complex(+0.000000E+00,+1.767767E-01),complex(+0.000000E+00,-3.750000E-01),0,complex(+0.000000E+00,+3.750000E-01),complex(+0.000000E+00,-1.767767E-01),complex(-0.000000E+00,-3.307189E-01),complex(+0.000000E+00,+4.677072E-01)],$
            [complex(+5.229125E-01,+0.000000E+00),0,complex(-3.952847E-01,+0.000000E+00),0,complex(+3.750000E-01,+0.000000E+00),0,complex(-3.952847E-01,+0.000000E+00),0,complex(+5.229125E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+4.677072E-01),complex(-0.000000E+00,+3.307189E-01),complex(+0.000000E+00,-1.767767E-01),complex(+0.000000E+00,-3.750000E-01),0,complex(+0.000000E+00,+3.750000E-01),complex(+0.000000E+00,+1.767767E-01),complex(-0.000000E+00,-3.307189E-01),complex(+0.000000E+00,-4.677072E-01)],$
            [complex(-3.307189E-01,+0.000000E+00),complex(-4.677072E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+1.767767E-01,+0.000000E+00),complex(+3.952847E-01,+0.000000E+00),complex(+1.767767E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-4.677072E-01,+0.000000E+00),complex(-3.307189E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.767767E-01),complex(+0.000000E+00,-3.750000E-01),complex(+0.000000E+00,-4.677072E-01),complex(+0.000000E+00,-3.307189E-01),0,complex(+0.000000E+00,+3.307189E-01),complex(+0.000000E+00,+4.677072E-01),complex(+0.000000E+00,+3.750000E-01),complex(+0.000000E+00,+1.767767E-01)],$
            [complex(+6.250000E-02,+0.000000E+00),complex(+1.767767E-01,+0.000000E+00),complex(+3.307189E-01,+0.000000E+00),complex(+4.677072E-01,+0.000000E+00),complex(+5.229125E-01,+0.000000E+00),complex(+4.677072E-01,-0.000000E+00),complex(+3.307189E-01,+0.000000E+00),complex(+1.767767E-01,-0.000000E+00),complex(+6.250000E-02,+0.000000E+00)]$
            ]
          end

       7: begin
            matrix[28:35,28:35]=[$
            [complex(-0.000000E+00,+8.838835E-02),complex(+0.000000E+00,-2.338536E-01),complex(-0.000000E+00,+4.050463E-01),complex(+0.000000E+00,-5.229125E-01),complex(+0.000000E+00,+5.229125E-01),complex(+0.000000E+00,-4.050463E-01),complex(+0.000000E+00,+2.338536E-01),complex(+0.000000E+00,-8.838835E-02)],$
            [complex(-2.338536E-01,+0.000000E+00),complex(+4.419417E-01,+0.000000E+00),complex(-4.592793E-01,+0.000000E+00),complex(+1.976424E-01,+0.000000E+00),complex(+1.976424E-01,+0.000000E+00),complex(-4.592793E-01,+0.000000E+00),complex(+4.419417E-01,+0.000000E+00),complex(-2.338536E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-4.050463E-01),complex(+0.000000E+00,+4.592793E-01),complex(+0.000000E+00,-8.838835E-02),complex(+0.000000E+00,-3.423266E-01),complex(+0.000000E+00,+3.423266E-01),complex(+0.000000E+00,+8.838835E-02),complex(-0.000000E+00,-4.592793E-01),complex(+0.000000E+00,+4.050463E-01)],$
            [complex(+5.229125E-01,+0.000000E+00),complex(-1.976424E-01,-0.000000E+00),complex(-3.423266E-01,+0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(+2.651650E-01,+0.000000E+00),complex(-3.423266E-01,+0.000000E+00),complex(-1.976424E-01,+0.000000E+00),complex(+5.229125E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+5.229125E-01),complex(+0.000000E+00,+1.976424E-01),complex(+0.000000E+00,-3.423266E-01),complex(+0.000000E+00,-2.651650E-01),complex(+0.000000E+00,+2.651650E-01),complex(+0.000000E+00,+3.423266E-01),complex(-0.000000E+00,-1.976424E-01),complex(+0.000000E+00,-5.229125E-01)],$
            [complex(-4.050463E-01,+0.000000E+00),complex(-4.592793E-01,-0.000000E+00),complex(-8.838835E-02,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(+3.423266E-01,+0.000000E+00),complex(-8.838835E-02,+0.000000E+00),complex(-4.592793E-01,+0.000000E+00),complex(-4.050463E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.338536E-01),complex(+0.000000E+00,-4.419417E-01),complex(+0.000000E+00,-4.592793E-01),complex(+0.000000E+00,-1.976424E-01),complex(+0.000000E+00,+1.976424E-01),complex(+0.000000E+00,+4.592793E-01),complex(+0.000000E+00,+4.419417E-01),complex(+0.000000E+00,+2.338536E-01)],$
            [complex(+8.838835E-02,+0.000000E+00),complex(+2.338536E-01,+0.000000E+00),complex(+4.050463E-01,+0.000000E+00),complex(+5.229125E-01,+0.000000E+00),complex(+5.229125E-01,+0.000000E+00),complex(+4.050463E-01,-0.000000E+00),complex(+2.338536E-01,+0.000000E+00),complex(+8.838835E-02,-0.000000E+00)]$
            ]
          end

       6: begin
            matrix[21:27,21:27]=[$
            [complex(-1.250000E-01,-0.000000E+00),complex(+3.061862E-01,+0.000000E+00),complex(-4.841229E-01,-0.000000E+00),complex(+5.590170E-01,+0.000000E+00),complex(-4.841229E-01,+0.000000E+00),complex(+3.061862E-01,+0.000000E+00),complex(-1.250000E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.061862E-01),complex(+0.000000E+00,+5.000000E-01),complex(+0.000000E+00,-3.952847E-01),0,complex(+0.000000E+00,+3.952847E-01),complex(+0.000000E+00,-5.000000E-01),complex(+0.000000E+00,+3.061862E-01)],$
            [complex(+4.841229E-01,+0.000000E+00),complex(-3.952847E-01,+0.000000E+00),complex(-1.250000E-01,-0.000000E+00),complex(+4.330127E-01,+0.000000E+00),complex(-1.250000E-01,+0.000000E+00),complex(-3.952847E-01,+0.000000E+00),complex(+4.841229E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+5.590170E-01),0,complex(+0.000000E+00,-4.330127E-01),0,complex(+0.000000E+00,+4.330127E-01),0,complex(-0.000000E+00,-5.590170E-01)],$
            [complex(-4.841229E-01,-0.000000E+00),complex(-3.952847E-01,+0.000000E+00),complex(+1.250000E-01,+0.000000E+00),complex(+4.330127E-01,+0.000000E+00),complex(+1.250000E-01,-0.000000E+00),complex(-3.952847E-01,+0.000000E+00),complex(-4.841229E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.061862E-01),complex(+0.000000E+00,-5.000000E-01),complex(+0.000000E+00,-3.952847E-01),0,complex(+0.000000E+00,+3.952847E-01),complex(+0.000000E+00,+5.000000E-01),complex(+0.000000E+00,+3.061862E-01)],$
            [complex(+1.250000E-01,+0.000000E+00),complex(+3.061862E-01,+0.000000E+00),complex(+4.841229E-01,+0.000000E+00),complex(+5.590170E-01,+0.000000E+00),complex(+4.841229E-01,-0.000000E+00),complex(+3.061862E-01,+0.000000E+00),complex(+1.250000E-01,-0.000000E+00)]$
            ]
          end

       5: begin
            matrix[15:20,15:20]=[$
            [complex(+0.000000E+00,-1.767767E-01),complex(-0.000000E+00,+3.952847E-01),complex(+0.000000E+00,-5.590170E-01),complex(+0.000000E+00,+5.590170E-01),complex(+0.000000E+00,-3.952847E-01),complex(+0.000000E+00,+1.767767E-01)],$
            [complex(+3.952847E-01,+0.000000E+00),complex(-5.303301E-01,+0.000000E+00),complex(+2.500000E-01,+0.000000E+00),complex(+2.500000E-01,+0.000000E+00),complex(-5.303301E-01,+0.000000E+00),complex(+3.952847E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+5.590170E-01),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,-3.535534E-01),complex(+0.000000E+00,+3.535534E-01),complex(+0.000000E+00,+2.500000E-01),complex(-0.000000E+00,-5.590170E-01)],$
            [complex(-5.590170E-01,-0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(+3.535534E-01,+0.000000E+00),complex(+3.535534E-01,+0.000000E+00),complex(-2.500000E-01,+0.000000E+00),complex(-5.590170E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.952847E-01),complex(+0.000000E+00,-5.303301E-01),complex(+0.000000E+00,-2.500000E-01),complex(+0.000000E+00,+2.500000E-01),complex(+0.000000E+00,+5.303301E-01),complex(+0.000000E+00,+3.952847E-01)],$
            [complex(+1.767767E-01,+0.000000E+00),complex(+3.952847E-01,+0.000000E+00),complex(+5.590170E-01,+0.000000E+00),complex(+5.590170E-01,+0.000000E+00),complex(+3.952847E-01,-0.000000E+00),complex(+1.767767E-01,+0.000000E+00)]$
            ]
          end

       4: begin
            matrix[10:14,10:14]=[$
            [complex(+2.500000E-01,+0.000000E+00),complex(-5.000000E-01,-0.000000E+00),complex(+6.123725E-01,+0.000000E+00),complex(-5.000000E-01,+0.000000E+00),complex(+2.500000E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+5.000000E-01),complex(+0.000000E+00,-5.000000E-01),0,complex(+0.000000E+00,+5.000000E-01),complex(+0.000000E+00,-5.000000E-01)],$
            [complex(-6.123725E-01,+0.000000E+00),0,complex(+5.000000E-01,+0.000000E+00),0,complex(-6.123725E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-5.000000E-01),complex(+0.000000E+00,-5.000000E-01),0,complex(+0.000000E+00,+5.000000E-01),complex(+0.000000E+00,+5.000000E-01)],$
            [complex(+2.500000E-01,+0.000000E+00),complex(+5.000000E-01,+0.000000E+00),complex(+6.123725E-01,+0.000000E+00),complex(+5.000000E-01,-0.000000E+00),complex(+2.500000E-01,+0.000000E+00)]$
            ]
          end

       3: begin
            matrix[6:9,6:9]=[$
            [complex(-0.000000E+00,+3.535534E-01),complex(+0.000000E+00,-6.123724E-01),complex(+0.000000E+00,+6.123724E-01),complex(+0.000000E+00,-3.535534E-01)],$
            [complex(-6.123724E-01,+0.000000E+00),complex(+3.535534E-01,+0.000000E+00),complex(+3.535534E-01,+0.000000E+00),complex(-6.123724E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-6.123724E-01),complex(+0.000000E+00,-3.535534E-01),complex(+0.000000E+00,+3.535534E-01),complex(+0.000000E+00,+6.123724E-01)],$
            [complex(+3.535534E-01,+0.000000E+00),complex(+6.123724E-01,+0.000000E+00),complex(+6.123724E-01,+0.000000E+00),complex(+3.535534E-01,-0.000000E+00)]$
            ]
          end

       2: begin
            matrix[3:5,3:5]=[$
            [complex(-5.000000E-01,-0.000000E+00),complex(+7.071068E-01,+0.000000E+00),complex(-5.000000E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-7.071068E-01),0,complex(+0.000000E+00,+7.071068E-01)],$
            [complex(+5.000000E-01,+0.000000E+00),complex(+7.071068E-01,+0.000000E+00),complex(+5.000000E-01,-0.000000E+00)]$
            ]
          end

       1: begin
            matrix[1:2,1:2]=[$
            [complex(+0.000000E+00,-7.071068E-01),complex(+0.000000E+00,+7.071068E-01)],$
            [complex(+7.071068E-01,+0.000000E+00),complex(+7.071068E-01,+0.000000E+00)]$
            ]
          end

       0: matrix[0]=complex(1.,0.)

    endswitch
  endelse
  
endif else begin
  
  ; Calculate inverse matrix, to transfer from polar shapelets to Cartesians.
  ; First, initialise an empty array
  matrix=complexarr(n_coeffs,n_coeffs)
  
  if n_max gt 20 or keyword_set(calculate) then begin
    ; Calculate matrix. It would be nice to know how to calculate this
    ; by itself, especially since it seems a simpler matrix than the
    ; forward transform. Oh well, at least this minimises the amount of
    ; independent code that has to be managed.
    matrix=la_invert(shapelets_polar_matrix(n_max,calculate=calculate))
  endif else begin
    ; Use look-up table    
    switch n_max of
  
      20: begin
            matrix[210:230,210:230]=[$
            [complex(+9.766079E-04,+0.000000E+00),complex(+0.000000E+00,-4.367333E-03),complex(-1.346101E-02,+0.000000E+00),complex(+0.000000E+00,+3.297254E-02),complex(+6.797464E-02,+0.000000E+00),complex(+0.000000E+00,-1.215968E-01),complex(-1.922613E-01,+0.000000E+00),complex(+0.000000E+00,+2.718986E-01),complex(+3.466041E-01,+0.000000E+00),complex(+0.000000E+00,-4.002238E-01),complex(-4.197583E-01,+0.000000E+00),complex(+0.000000E+00,+4.002239E-01),complex(+3.466040E-01,+0.000000E+00),complex(+0.000000E+00,-2.718985E-01),complex(-1.922614E-01,+0.000000E+00),complex(+0.000000E+00,+1.215968E-01),complex(+6.797463E-02,+0.000000E+00),complex(+0.000000E+00,-3.297250E-02),complex(-1.346102E-02,+0.000000E+00),complex(+0.000000E+00,+4.367323E-03),complex(+9.766098E-04,-0.000000E+00)],$
            [complex(-4.367294E-03,+0.000000E+00),complex(+0.000000E+00,+1.757806E-02),complex(+4.815942E-02,+0.000000E+00),complex(+0.000000E+00,-1.032204E-01),complex(-1.823951E-01,+0.000000E+00),complex(+0.000000E+00,+2.718986E-01),complex(+3.439275E-01,+0.000000E+00),complex(+0.000000E+00,-3.647903E-01),complex(-3.100121E-01,+0.000000E+00),complex(+0.000000E+00,+1.789856E-01),complex(+7.135282E-08,+0.000000E+00),complex(+0.000000E+00,+1.789856E-01),complex(+3.100120E-01,+0.000000E+00),complex(+0.000000E+00,-3.647902E-01),complex(-3.439276E-01,+0.000000E+00),complex(+0.000000E+00,+2.718985E-01),complex(+1.823951E-01,+0.000000E+00),complex(+0.000000E+00,-1.032203E-01),complex(-4.815943E-02,+0.000000E+00),complex(+0.000000E+00,+1.757816E-02),complex(+4.367352E-03,-0.000000E+00)],$
            [complex(+1.346106E-02,+0.000000E+00),complex(+0.000000E+00,-4.815952E-02),complex(-1.152344E-01,+0.000000E+00),complex(+0.000000E+00,+2.105031E-01),complex(+3.057467E-01,+0.000000E+00),complex(+0.000000E+00,-3.528621E-01),complex(-3.068583E-01,+0.000000E+00),complex(+0.000000E+00,+1.578048E-01),complex(-5.029058E-02,+0.000000E+00),complex(+0.000000E+00,+2.322823E-01),complex(+3.045247E-01,+0.000000E+00),complex(+0.000000E+00,-2.322823E-01),complex(-5.029059E-02,+0.000000E+00),complex(+0.000000E+00,-1.578048E-01),complex(-3.068583E-01,+0.000000E+00),complex(+0.000000E+00,+3.528622E-01),complex(+3.057467E-01,+0.000000E+00),complex(+0.000000E+00,-2.105030E-01),complex(-1.152343E-01,+0.000000E+00),complex(+0.000000E+00,+4.815954E-02),complex(+1.346104E-02,-0.000000E+00)],$
            [complex(-3.297254E-02,+0.000000E+00),complex(+0.000000E+00,+1.032205E-01),complex(+2.105030E-01,+0.000000E+00),complex(+0.000000E+00,-3.144532E-01),complex(-3.462765E-01,+0.000000E+00),complex(+0.000000E+00,+2.520970E-01),complex(+4.555434E-02,+0.000000E+00),complex(+0.000000E+00,+1.771647E-01),complex(+2.874347E-01,+0.000000E+00),complex(+0.000000E+00,-2.133649E-01),complex(-1.065832E-08,+0.000000E+00),complex(+0.000000E+00,-2.133649E-01),complex(-2.874347E-01,+0.000000E+00),complex(+0.000000E+00,+1.771647E-01),complex(-4.555434E-02,+0.000000E+00),complex(+0.000000E+00,+2.520969E-01),complex(+3.462765E-01,+0.000000E+00),complex(+0.000000E+00,-3.144531E-01),complex(-2.105030E-01,+0.000000E+00),complex(+0.000000E+00,+1.032204E-01),complex(+3.297254E-02,-0.000000E+00)],$
            [complex(+6.797474E-02,+0.000000E+00),complex(+0.000000E+00,-1.823952E-01),complex(-3.057468E-01,+0.000000E+00),complex(+0.000000E+00,+3.462763E-01),complex(+2.314453E-01,+0.000000E+00),complex(+0.000000E+00,+8.734811E-03),complex(+2.292573E-01,+0.000000E+00),complex(+0.000000E+00,-2.695312E-01),complex(-9.461068E-02,+0.000000E+00),complex(+0.000000E+00,-1.552459E-01),complex(-2.713720E-01,+0.000000E+00),complex(+0.000000E+00,+1.552458E-01),complex(-9.461080E-02,+0.000000E+00),complex(+0.000000E+00,+2.695312E-01),complex(+2.292573E-01,+0.000000E+00),complex(+0.000000E+00,-8.734662E-03),complex(+2.314453E-01,+0.000000E+00),complex(+0.000000E+00,-3.462764E-01),complex(-3.057467E-01,+0.000000E+00),complex(+0.000000E+00,+1.823951E-01),complex(+6.797466E-02,-0.000000E+00)],$
            [complex(-1.215968E-01,+0.000000E+00),complex(+0.000000E+00,+2.718985E-01),complex(+3.528622E-01,+0.000000E+00),complex(-0.000000E+00,-2.520969E-01),complex(+8.734666E-03,+0.000000E+00),complex(-0.000000E+00,-2.421875E-01),complex(-2.470529E-01,+0.000000E+00),complex(+0.000000E+00,+1.746923E-02),complex(-2.226905E-01,+0.000000E+00),complex(+0.000000E+00,+2.314267E-01),complex(+4.461672E-08,+0.000000E+00),complex(+0.000000E+00,+2.314268E-01),complex(+2.226905E-01,+0.000000E+00),complex(+0.000000E+00,+1.746927E-02),complex(+2.470529E-01,+0.000000E+00),complex(-0.000000E+00,-2.421875E-01),complex(-8.734640E-03,+0.000000E+00),complex(-0.000000E+00,-2.520969E-01),complex(-3.528622E-01,+0.000000E+00),complex(+0.000000E+00,+2.718985E-01),complex(+1.215967E-01,-0.000000E+00)],$
            [complex(+1.922613E-01,+0.000000E+00),complex(+0.000000E+00,-3.439276E-01),complex(-3.068582E-01,+0.000000E+00),complex(+0.000000E+00,+4.555433E-02),complex(-2.292573E-01,+0.000000E+00),complex(+0.000000E+00,+2.470530E-01),complex(-7.812463E-03,+0.000000E+00),complex(+0.000000E+00,+2.430679E-01),complex(+1.830945E-01,+0.000000E+00),complex(+0.000000E+00,+9.757794E-02),complex(+2.558520E-01,+0.000000E+00),complex(+0.000000E+00,-9.757799E-02),complex(+1.830943E-01,+0.000000E+00),complex(+0.000000E+00,-2.430679E-01),complex(-7.812504E-03,+0.000000E+00),complex(+0.000000E+00,-2.470529E-01),complex(-2.292573E-01,+0.000000E+00),complex(+0.000000E+00,-4.555430E-02),complex(-3.068581E-01,+0.000000E+00),complex(+0.000000E+00,+3.439276E-01),complex(+1.922613E-01,-0.000000E+00)],$
            [complex(-2.718983E-01,+0.000000E+00),complex(+0.000000E+00,+3.647900E-01),complex(+1.578048E-01,+0.000000E+00),complex(+0.000000E+00,+1.771647E-01),complex(+2.695312E-01,+0.000000E+00),complex(+0.000000E+00,-1.746941E-02),complex(+2.430680E-01,+0.000000E+00),complex(+0.000000E+00,-1.640625E-01),complex(+1.394264E-01,+0.000000E+00),complex(+0.000000E+00,-2.414935E-01),complex(-4.896881E-09,+0.000000E+00),complex(+0.000000E+00,-2.414934E-01),complex(-1.394263E-01,+0.000000E+00),complex(+0.000000E+00,-1.640625E-01),complex(-2.430680E-01,+0.000000E+00),complex(+0.000000E+00,-1.746932E-02),complex(-2.695313E-01,+0.000000E+00),complex(+0.000000E+00,+1.771648E-01),complex(-1.578048E-01,+0.000000E+00),complex(+0.000000E+00,+3.647903E-01),complex(+2.718985E-01,-0.000000E+00)],$
            [complex(+3.466041E-01,+0.000000E+00),complex(+0.000000E+00,-3.100121E-01),complex(+5.029054E-02,+0.000000E+00),complex(+0.000000E+00,-2.874346E-01),complex(-9.461074E-02,+0.000000E+00),complex(+0.000000E+00,-2.226904E-01),complex(-1.830944E-01,+0.000000E+00),complex(+0.000000E+00,-1.394264E-01),complex(-2.324218E-01,+0.000000E+00),complex(+0.000000E+00,-4.736078E-02),complex(-2.483620E-01,+0.000000E+00),complex(+0.000000E+00,+4.736075E-02),complex(-2.324219E-01,+0.000000E+00),complex(+0.000000E+00,+1.394264E-01),complex(-1.830944E-01,+0.000000E+00),complex(+0.000000E+00,+2.226905E-01),complex(-9.461071E-02,+0.000000E+00),complex(+0.000000E+00,+2.874348E-01),complex(+5.029058E-02,+0.000000E+00),complex(+0.000000E+00,+3.100121E-01),complex(+3.466040E-01,-0.000000E+00)],$
            [complex(-4.002237E-01,+0.000000E+00),complex(+0.000000E+00,+1.789856E-01),complex(-2.322824E-01,+0.000000E+00),complex(+0.000000E+00,+2.133650E-01),complex(-1.552458E-01,+0.000000E+00),complex(+0.000000E+00,+2.314267E-01),complex(-9.757808E-02,+0.000000E+00),complex(+0.000000E+00,+2.414934E-01),complex(-4.736071E-02,+0.000000E+00),complex(+0.000000E+00,+2.460937E-01),complex(+2.203462E-08,+0.000000E+00),complex(+0.000000E+00,+2.460937E-01),complex(+4.736078E-02,+0.000000E+00),complex(+0.000000E+00,+2.414934E-01),complex(+9.757807E-02,+0.000000E+00),complex(+0.000000E+00,+2.314268E-01),complex(+1.552458E-01,+0.000000E+00),complex(+0.000000E+00,+2.133649E-01),complex(+2.322824E-01,+0.000000E+00),complex(+0.000000E+00,+1.789856E-01),complex(+4.002238E-01,-0.000000E+00)],$
            [complex(+4.197584E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-07),complex(+3.045246E-01,+0.000000E+00),complex(+0.000000E+00,+4.470348E-08),complex(+2.713720E-01,+0.000000E+00),complex(+0.000000E+00,+4.470348E-08),complex(+2.558520E-01,+0.000000E+00),0,complex(+2.483619E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+2.460937E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+2.483619E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+2.558519E-01,+0.000000E+00),0,complex(+2.713719E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+3.045248E-01,+0.000000E+00),complex(+0.000000E+00,+8.940697E-08),complex(+4.197583E-01,-0.000000E+00)],$
            [complex(-4.002239E-01,+0.000000E+00),complex(+0.000000E+00,-1.789855E-01),complex(-2.322823E-01,+0.000000E+00),complex(+0.000000E+00,-2.133649E-01),complex(-1.552458E-01,+0.000000E+00),complex(+0.000000E+00,-2.314267E-01),complex(-9.757812E-02,+0.000000E+00),complex(+0.000000E+00,-2.414934E-01),complex(-4.736073E-02,+0.000000E+00),complex(+0.000000E+00,-2.460937E-01),complex(+1.356904E-08,+0.000000E+00),complex(+0.000000E+00,-2.460938E-01),complex(+4.736075E-02,+0.000000E+00),complex(+0.000000E+00,-2.414934E-01),complex(+9.757813E-02,+0.000000E+00),complex(+0.000000E+00,-2.314267E-01),complex(+1.552458E-01,+0.000000E+00),complex(+0.000000E+00,-2.133650E-01),complex(+2.322823E-01,+0.000000E+00),complex(+0.000000E+00,-1.789856E-01),complex(+4.002238E-01,-0.000000E+00)],$
            [complex(+3.466041E-01,+0.000000E+00),complex(+0.000000E+00,+3.100122E-01),complex(+5.029061E-02,+0.000000E+00),complex(+0.000000E+00,+2.874346E-01),complex(-9.461063E-02,+0.000000E+00),complex(+0.000000E+00,+2.226905E-01),complex(-1.830944E-01,+0.000000E+00),complex(+0.000000E+00,+1.394264E-01),complex(-2.324219E-01,+0.000000E+00),complex(+0.000000E+00,+4.736082E-02),complex(-2.483619E-01,+0.000000E+00),complex(+0.000000E+00,-4.736091E-02),complex(-2.324219E-01,+0.000000E+00),complex(+0.000000E+00,-1.394263E-01),complex(-1.830944E-01,+0.000000E+00),complex(+0.000000E+00,-2.226905E-01),complex(-9.461068E-02,+0.000000E+00),complex(+0.000000E+00,-2.874347E-01),complex(+5.029055E-02,+0.000000E+00),complex(+0.000000E+00,-3.100121E-01),complex(+3.466040E-01,-0.000000E+00)],$
            [complex(-2.718986E-01,+0.000000E+00),complex(+0.000000E+00,-3.647902E-01),complex(+1.578047E-01,+0.000000E+00),complex(+0.000000E+00,-1.771647E-01),complex(+2.695314E-01,+0.000000E+00),complex(+0.000000E+00,+1.746935E-02),complex(+2.430680E-01,+0.000000E+00),complex(+0.000000E+00,+1.640625E-01),complex(+1.394263E-01,+0.000000E+00),complex(+0.000000E+00,+2.414935E-01),complex(+1.623776E-08,+0.000000E+00),complex(+0.000000E+00,+2.414935E-01),complex(-1.394263E-01,+0.000000E+00),complex(+0.000000E+00,+1.640624E-01),complex(-2.430680E-01,+0.000000E+00),complex(+0.000000E+00,+1.746926E-02),complex(-2.695312E-01,+0.000000E+00),complex(+0.000000E+00,-1.771647E-01),complex(-1.578048E-01,+0.000000E+00),complex(+0.000000E+00,-3.647903E-01),complex(+2.718985E-01,+0.000000E+00)],$
            [complex(+1.922613E-01,+0.000000E+00),complex(+0.000000E+00,+3.439275E-01),complex(-3.068582E-01,+0.000000E+00),complex(+0.000000E+00,-4.555446E-02),complex(-2.292572E-01,+0.000000E+00),complex(+0.000000E+00,-2.470529E-01),complex(-7.812504E-03,+0.000000E+00),complex(+0.000000E+00,-2.430679E-01),complex(+1.830943E-01,+0.000000E+00),complex(+0.000000E+00,-9.757810E-02),complex(+2.558520E-01,+0.000000E+00),complex(+0.000000E+00,+9.757826E-02),complex(+1.830944E-01,+0.000000E+00),complex(-0.000000E+00,+2.430680E-01),complex(-7.812522E-03,+0.000000E+00),complex(+0.000000E+00,+2.470530E-01),complex(-2.292572E-01,+0.000000E+00),complex(+0.000000E+00,+4.555430E-02),complex(-3.068583E-01,+0.000000E+00),complex(+0.000000E+00,-3.439276E-01),complex(+1.922614E-01,-0.000000E+00)],$
            [complex(-1.215968E-01,-0.000000E+00),complex(+0.000000E+00,-2.718986E-01),complex(+3.528623E-01,+0.000000E+00),complex(+0.000000E+00,+2.520969E-01),complex(+8.734636E-03,+0.000000E+00),complex(+0.000000E+00,+2.421875E-01),complex(-2.470530E-01,-0.000000E+00),complex(+0.000000E+00,-1.746923E-02),complex(-2.226906E-01,-0.000000E+00),complex(+0.000000E+00,-2.314267E-01),complex(+7.702955E-08,+0.000000E+00),complex(+0.000000E+00,-2.314268E-01),complex(+2.226906E-01,+0.000000E+00),complex(+0.000000E+00,-1.746928E-02),complex(+2.470530E-01,+0.000000E+00),complex(+0.000000E+00,+2.421875E-01),complex(-8.734655E-03,+0.000000E+00),complex(+0.000000E+00,+2.520969E-01),complex(-3.528623E-01,+0.000000E+00),complex(+0.000000E+00,-2.718986E-01),complex(+1.215967E-01,+0.000000E+00)],$
            [complex(+6.797461E-02,+0.000000E+00),complex(+0.000000E+00,+1.823952E-01),complex(-3.057467E-01,+0.000000E+00),complex(+0.000000E+00,-3.462765E-01),complex(+2.314453E-01,+0.000000E+00),complex(+0.000000E+00,-8.734640E-03),complex(+2.292572E-01,+0.000000E+00),complex(+0.000000E+00,+2.695312E-01),complex(-9.461067E-02,+0.000000E+00),complex(+0.000000E+00,+1.552459E-01),complex(-2.713719E-01,+0.000000E+00),complex(+0.000000E+00,-1.552459E-01),complex(-9.461068E-02,+0.000000E+00),complex(+0.000000E+00,-2.695312E-01),complex(+2.292573E-01,+0.000000E+00),complex(+0.000000E+00,+8.734610E-03),complex(+2.314453E-01,+0.000000E+00),complex(+0.000000E+00,+3.462765E-01),complex(-3.057468E-01,+0.000000E+00),complex(+0.000000E+00,-1.823951E-01),complex(+6.797467E-02,-0.000000E+00)],$
            [complex(-3.297258E-02,+0.000000E+00),complex(+0.000000E+00,-1.032203E-01),complex(+2.105031E-01,+0.000000E+00),complex(+0.000000E+00,+3.144532E-01),complex(-3.462766E-01,+0.000000E+00),complex(+0.000000E+00,-2.520970E-01),complex(+4.555432E-02,+0.000000E+00),complex(+0.000000E+00,-1.771646E-01),complex(+2.874348E-01,+0.000000E+00),complex(+0.000000E+00,+2.133650E-01),complex(-1.063869E-07,+0.000000E+00),complex(+0.000000E+00,+2.133648E-01),complex(-2.874346E-01,+0.000000E+00),complex(+0.000000E+00,-1.771647E-01),complex(-4.555424E-02,+0.000000E+00),complex(+0.000000E+00,-2.520968E-01),complex(+3.462765E-01,+0.000000E+00),complex(+0.000000E+00,+3.144531E-01),complex(-2.105030E-01,+0.000000E+00),complex(+0.000000E+00,-1.032204E-01),complex(+3.297259E-02,-0.000000E+00)],$
            [complex(+1.346107E-02,+0.000000E+00),complex(+0.000000E+00,+4.815948E-02),complex(-1.152343E-01,+0.000000E+00),complex(+0.000000E+00,-2.105030E-01),complex(+3.057466E-01,+0.000000E+00),complex(+0.000000E+00,+3.528622E-01),complex(-3.068583E-01,+0.000000E+00),complex(+0.000000E+00,-1.578048E-01),complex(-5.029061E-02,+0.000000E+00),complex(+0.000000E+00,-2.322824E-01),complex(+3.045247E-01,+0.000000E+00),complex(+0.000000E+00,+2.322823E-01),complex(-5.029062E-02,+0.000000E+00),complex(+0.000000E+00,+1.578047E-01),complex(-3.068583E-01,+0.000000E+00),complex(+0.000000E+00,-3.528621E-01),complex(+3.057467E-01,+0.000000E+00),complex(+0.000000E+00,+2.105030E-01),complex(-1.152343E-01,+0.000000E+00),complex(+0.000000E+00,-4.815945E-02),complex(+1.346103E-02,-0.000000E+00)],$
            [complex(-4.367277E-03,+0.000000E+00),complex(+0.000000E+00,-1.757822E-02),complex(+4.815949E-02,+0.000000E+00),complex(+0.000000E+00,+1.032204E-01),complex(-1.823952E-01,+0.000000E+00),complex(+0.000000E+00,-2.718987E-01),complex(+3.439276E-01,+0.000000E+00),complex(+0.000000E+00,+3.647901E-01),complex(-3.100121E-01,+0.000000E+00),complex(+0.000000E+00,-1.789855E-01),complex(+3.433274E-08,+0.000000E+00),complex(+0.000000E+00,-1.789856E-01),complex(+3.100120E-01,+0.000000E+00),complex(+0.000000E+00,+3.647901E-01),complex(-3.439274E-01,+0.000000E+00),complex(+0.000000E+00,-2.718985E-01),complex(+1.823951E-01,+0.000000E+00),complex(+0.000000E+00,+1.032204E-01),complex(-4.815942E-02,+0.000000E+00),complex(+0.000000E+00,-1.757808E-02),complex(+4.367341E-03,-0.000000E+00)],$
            [complex(+9.766088E-04,+0.000000E+00),complex(+0.000000E+00,+4.367339E-03),complex(-1.346097E-02,+0.000000E+00),complex(+0.000000E+00,-3.297256E-02),complex(+6.797460E-02,+0.000000E+00),complex(+0.000000E+00,+1.215967E-01),complex(-1.922613E-01,+0.000000E+00),complex(+0.000000E+00,-2.718985E-01),complex(+3.466040E-01,+0.000000E+00),complex(+0.000000E+00,+4.002239E-01),complex(-4.197584E-01,+0.000000E+00),complex(+0.000000E+00,-4.002238E-01),complex(+3.466040E-01,+0.000000E+00),complex(+0.000000E+00,+2.718985E-01),complex(-1.922613E-01,+0.000000E+00),complex(+0.000000E+00,-1.215967E-01),complex(+6.797465E-02,+0.000000E+00),complex(+0.000000E+00,+3.297253E-02),complex(-1.346099E-02,+0.000000E+00),complex(+0.000000E+00,-4.367317E-03),complex(+9.766067E-04,-0.000000E+00)]$
            ]
          end

      19: begin
            matrix[190:209,190:209]=[$
            [complex(+0.000000E+00,-1.381070E-03),complex(-6.019983E-03,+0.000000E+00),complex(+0.000000E+00,+1.805984E-02),complex(+4.299090E-02,+0.000000E+00),complex(+0.000000E+00,-8.598191E-02),complex(-1.489250E-01,+0.000000E+00),complex(+0.000000E+00,+2.274867E-01),complex(+3.100120E-01,+0.000000E+00),complex(+0.000000E+00,-3.796857E-01),complex(-4.197583E-01,+0.000000E+00),complex(+0.000000E+00,+4.197584E-01),complex(+3.796858E-01,+0.000000E+00),complex(+0.000000E+00,-3.100120E-01),complex(-2.274867E-01,+0.000000E+00),complex(+0.000000E+00,+1.489250E-01),complex(+8.598192E-02,+0.000000E+00),complex(+0.000000E+00,-4.299096E-02),complex(-1.805978E-02,+0.000000E+00),complex(+0.000000E+00,+6.019946E-03),complex(+1.381068E-03,-0.000000E+00)],$
            [complex(+0.000000E+00,+6.019848E-03),complex(+2.347821E-02,+0.000000E+00),complex(+0.000000E+00,-6.214806E-02),complex(-1.282164E-01,+0.000000E+00),complex(+0.000000E+00,+2.169816E-01),complex(+3.074916E-01,+0.000000E+00),complex(+0.000000E+00,-3.653231E-01),complex(-3.556083E-01,+0.000000E+00),complex(+0.000000E+00,+2.613176E-01),complex(+9.629915E-02,+0.000000E+00),complex(+0.000000E+00,+9.629919E-02),complex(+2.613176E-01,+0.000000E+00),complex(+0.000000E+00,-3.556082E-01),complex(-3.653231E-01,+0.000000E+00),complex(+0.000000E+00,+3.074917E-01),complex(+2.169815E-01,+0.000000E+00),complex(+0.000000E+00,-1.282163E-01),complex(-6.214804E-02,+0.000000E+00),complex(+0.000000E+00,+2.347819E-02),complex(+6.019956E-03,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.805978E-02),complex(-6.214804E-02,+0.000000E+00),complex(+0.000000E+00,+1.422499E-01),complex(+2.465700E-01,+0.000000E+00),complex(+0.000000E+00,-3.353351E-01),complex(-3.530460E-01,+0.000000E+00),complex(+0.000000E+00,+2.609451E-01),complex(+7.112164E-02,+0.000000E+00),complex(+0.000000E+00,+1.451764E-01),complex(+2.888975E-01,+0.000000E+00),complex(+0.000000E+00,-2.888975E-01),complex(-1.451764E-01,+0.000000E+00),complex(+0.000000E+00,-7.112169E-02),complex(-2.609451E-01,+0.000000E+00),complex(+0.000000E+00,+3.530460E-01),complex(+3.353351E-01,+0.000000E+00),complex(+0.000000E+00,-2.465699E-01),complex(-1.422499E-01,+0.000000E+00),complex(+0.000000E+00,+6.214809E-02),complex(+1.805982E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.299101E-02),complex(+1.282164E-01,+0.000000E+00),complex(+0.000000E+00,-2.465700E-01),complex(-3.411238E-01,+0.000000E+00),complex(+0.000000E+00,+3.342186E-01),complex(+1.865822E-01,+0.000000E+00),complex(+0.000000E+00,+5.115540E-02),complex(+2.489754E-01,+0.000000E+00),complex(+0.000000E+00,-2.805371E-01),complex(-1.213613E-01,+0.000000E+00),complex(+0.000000E+00,-1.213611E-01),complex(-2.805370E-01,+0.000000E+00),complex(+0.000000E+00,+2.489755E-01),complex(+5.115554E-02,+0.000000E+00),complex(+0.000000E+00,+1.865822E-01),complex(+3.342184E-01,+0.000000E+00),complex(+0.000000E+00,-3.411238E-01),complex(-2.465699E-01,+0.000000E+00),complex(+0.000000E+00,+1.282163E-01),complex(+4.299093E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-8.598181E-02),complex(-2.169816E-01,+0.000000E+00),complex(+0.000000E+00,+3.353350E-01),complex(+3.342184E-01,+0.000000E+00),complex(+0.000000E+00,-1.602038E-01),complex(+1.052515E-01,+0.000000E+00),complex(+0.000000E+00,-2.777012E-01),complex(-2.190986E-01,+0.000000E+00),complex(+0.000000E+00,-2.439453E-02),complex(-2.427224E-01,+0.000000E+00),complex(+0.000000E+00,+2.427225E-01),complex(+2.439452E-02,+0.000000E+00),complex(+0.000000E+00,+2.190985E-01),complex(+2.777011E-01,+0.000000E+00),complex(+0.000000E+00,-1.052515E-01),complex(+1.602039E-01,+0.000000E+00),complex(+0.000000E+00,-3.342184E-01),complex(-3.353351E-01,+0.000000E+00),complex(+0.000000E+00,+2.169815E-01),complex(+8.598185E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.489248E-01),complex(+3.074917E-01,+0.000000E+00),complex(+0.000000E+00,-3.530459E-01),complex(-1.865823E-01,+0.000000E+00),complex(+0.000000E+00,-1.052515E-01),complex(-2.817379E-01,+0.000000E+00),complex(+0.000000E+00,+1.772078E-01),complex(-1.034972E-01,+0.000000E+00),complex(+0.000000E+00,+2.675996E-01),complex(+1.401360E-01,+0.000000E+00),complex(+0.000000E+00,+1.401360E-01),complex(+2.675995E-01,+0.000000E+00),complex(+0.000000E+00,-1.034971E-01),complex(+1.772077E-01,+0.000000E+00),complex(+0.000000E+00,-2.817379E-01),complex(-1.052515E-01,+0.000000E+00),complex(+0.000000E+00,-1.865822E-01),complex(-3.530459E-01,+0.000000E+00),complex(+0.000000E+00,+3.074916E-01),complex(+1.489250E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.274867E-01),complex(-3.653231E-01,+0.000000E+00),complex(+0.000000E+00,+2.609451E-01),complex(-5.115554E-02,+0.000000E+00),complex(+0.000000E+00,+2.777012E-01),complex(+1.772077E-01,+0.000000E+00),complex(+0.000000E+00,+1.270582E-01),complex(+2.634910E-01,+0.000000E+00),complex(+0.000000E+00,-6.454185E-02),complex(+2.140611E-01,+0.000000E+00),complex(-0.000000E+00,-2.140611E-01),complex(+6.454188E-02,+0.000000E+00),complex(+0.000000E+00,-2.634909E-01),complex(-1.270582E-01,+0.000000E+00),complex(-0.000000E+00,-1.772078E-01),complex(-2.777011E-01,+0.000000E+00),complex(+0.000000E+00,+5.115551E-02),complex(-2.609451E-01,+0.000000E+00),complex(+0.000000E+00,+3.653231E-01),complex(+2.274867E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.100121E-01),complex(+3.556082E-01,+0.000000E+00),complex(+0.000000E+00,-7.112168E-02),complex(+2.489755E-01,+0.000000E+00),complex(+0.000000E+00,-2.190986E-01),complex(+1.034971E-01,+0.000000E+00),complex(+0.000000E+00,-2.634910E-01),complex(-3.866996E-02,+0.000000E+00),complex(+0.000000E+00,-2.368038E-01),complex(-1.570778E-01,+0.000000E+00),complex(+0.000000E+00,-1.570780E-01),complex(-2.368038E-01,+0.000000E+00),complex(+0.000000E+00,-3.866991E-02),complex(-2.634909E-01,+0.000000E+00),complex(+0.000000E+00,+1.034972E-01),complex(-2.190985E-01,+0.000000E+00),complex(+0.000000E+00,+2.489756E-01),complex(-7.112166E-02,+0.000000E+00),complex(+0.000000E+00,+3.556082E-01),complex(+3.100122E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.796856E-01),complex(-2.613176E-01,+0.000000E+00),complex(+0.000000E+00,-1.451765E-01),complex(-2.805370E-01,+0.000000E+00),complex(+0.000000E+00,-2.439454E-02),complex(-2.675995E-01,+0.000000E+00),complex(+0.000000E+00,+6.454179E-02),complex(-2.368038E-01,+0.000000E+00),complex(+0.000000E+00,+1.353447E-01),complex(-1.923804E-01,+0.000000E+00),complex(+0.000000E+00,+1.923804E-01),complex(-1.353447E-01,+0.000000E+00),complex(+0.000000E+00,+2.368039E-01),complex(-6.454182E-02,+0.000000E+00),complex(+0.000000E+00,+2.675995E-01),complex(+2.439452E-02,+0.000000E+00),complex(+0.000000E+00,+2.805370E-01),complex(+1.451765E-01,+0.000000E+00),complex(+0.000000E+00,+2.613176E-01),complex(+3.796857E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.197583E-01),complex(+9.629914E-02,+0.000000E+00),complex(+0.000000E+00,+2.888974E-01),complex(+1.213612E-01,+0.000000E+00),complex(+0.000000E+00,+2.427225E-01),complex(+1.401358E-01,+0.000000E+00),complex(+0.000000E+00,+2.140611E-01),complex(+1.570779E-01,+0.000000E+00),complex(+0.000000E+00,+1.923804E-01),complex(+1.740145E-01,+0.000000E+00),complex(+0.000000E+00,+1.740146E-01),complex(+1.923802E-01,+0.000000E+00),complex(+0.000000E+00,+1.570780E-01),complex(+2.140610E-01,+0.000000E+00),complex(+0.000000E+00,+1.401359E-01),complex(+2.427225E-01,+0.000000E+00),complex(+0.000000E+00,+1.213612E-01),complex(+2.888976E-01,+0.000000E+00),complex(+0.000000E+00,+9.629908E-02),complex(+4.197583E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.197583E-01),complex(+9.629914E-02,+0.000000E+00),complex(+0.000000E+00,-2.888975E-01),complex(+1.213612E-01,+0.000000E+00),complex(+0.000000E+00,-2.427225E-01),complex(+1.401358E-01,+0.000000E+00),complex(+0.000000E+00,-2.140611E-01),complex(+1.570778E-01,+0.000000E+00),complex(+0.000000E+00,-1.923803E-01),complex(+1.740146E-01,+0.000000E+00),complex(+0.000000E+00,-1.740146E-01),complex(+1.923803E-01,+0.000000E+00),complex(+0.000000E+00,-1.570778E-01),complex(+2.140611E-01,+0.000000E+00),complex(+0.000000E+00,-1.401359E-01),complex(+2.427225E-01,+0.000000E+00),complex(+0.000000E+00,-1.213613E-01),complex(+2.888976E-01,+0.000000E+00),complex(+0.000000E+00,-9.629925E-02),complex(+4.197583E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.796858E-01),complex(-2.613177E-01,+0.000000E+00),complex(+0.000000E+00,+1.451765E-01),complex(-2.805370E-01,+0.000000E+00),complex(+0.000000E+00,+2.439452E-02),complex(-2.675996E-01,+0.000000E+00),complex(+0.000000E+00,-6.454179E-02),complex(-2.368038E-01,+0.000000E+00),complex(+0.000000E+00,-1.353446E-01),complex(-1.923803E-01,+0.000000E+00),complex(+0.000000E+00,-1.923804E-01),complex(-1.353447E-01,+0.000000E+00),complex(+0.000000E+00,-2.368039E-01),complex(-6.454185E-02,+0.000000E+00),complex(-0.000000E+00,-2.675995E-01),complex(+2.439458E-02,+0.000000E+00),complex(+0.000000E+00,-2.805371E-01),complex(+1.451765E-01,+0.000000E+00),complex(+0.000000E+00,-2.613177E-01),complex(+3.796857E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.100119E-01),complex(+3.556081E-01,+0.000000E+00),complex(+0.000000E+00,+7.112166E-02),complex(+2.489757E-01,+0.000000E+00),complex(+0.000000E+00,+2.190985E-01),complex(+1.034972E-01,+0.000000E+00),complex(+0.000000E+00,+2.634909E-01),complex(-3.866990E-02,+0.000000E+00),complex(+0.000000E+00,+2.368038E-01),complex(-1.570779E-01,+0.000000E+00),complex(+0.000000E+00,+1.570778E-01),complex(-2.368039E-01,+0.000000E+00),complex(+0.000000E+00,+3.866974E-02),complex(-2.634909E-01,+0.000000E+00),complex(-0.000000E+00,-1.034973E-01),complex(-2.190985E-01,+0.000000E+00),complex(+0.000000E+00,-2.489756E-01),complex(-7.112163E-02,+0.000000E+00),complex(+0.000000E+00,-3.556082E-01),complex(+3.100121E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.274867E-01),complex(-3.653232E-01,+0.000000E+00),complex(+0.000000E+00,-2.609451E-01),complex(-5.115546E-02,+0.000000E+00),complex(+0.000000E+00,-2.777012E-01),complex(+1.772079E-01,+0.000000E+00),complex(+0.000000E+00,-1.270582E-01),complex(+2.634910E-01,+0.000000E+00),complex(+0.000000E+00,+6.454185E-02),complex(+2.140611E-01,+0.000000E+00),complex(+0.000000E+00,+2.140610E-01),complex(+6.454182E-02,+0.000000E+00),complex(+0.000000E+00,+2.634909E-01),complex(-1.270582E-01,+0.000000E+00),complex(-0.000000E+00,+1.772078E-01),complex(-2.777011E-01,+0.000000E+00),complex(+0.000000E+00,-5.115544E-02),complex(-2.609451E-01,+0.000000E+00),complex(+0.000000E+00,-3.653231E-01),complex(+2.274867E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.489251E-01),complex(+3.074915E-01,+0.000000E+00),complex(-0.000000E+00,+3.530459E-01),complex(-1.865822E-01,-0.000000E+00),complex(-0.000000E+00,+1.052515E-01),complex(-2.817379E-01,-0.000000E+00),complex(+0.000000E+00,-1.772077E-01),complex(-1.034973E-01,+0.000000E+00),complex(+0.000000E+00,-2.675995E-01),complex(+1.401359E-01,+0.000000E+00),complex(+0.000000E+00,-1.401359E-01),complex(+2.675996E-01,+0.000000E+00),complex(+0.000000E+00,+1.034972E-01),complex(+1.772078E-01,+0.000000E+00),complex(-0.000000E+00,+2.817378E-01),complex(-1.052515E-01,+0.000000E+00),complex(+0.000000E+00,+1.865822E-01),complex(-3.530459E-01,+0.000000E+00),complex(+0.000000E+00,-3.074916E-01),complex(+1.489250E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+8.598188E-02),complex(-2.169817E-01,+0.000000E+00),complex(+0.000000E+00,-3.353353E-01),complex(+3.342184E-01,+0.000000E+00),complex(+0.000000E+00,+1.602040E-01),complex(+1.052515E-01,+0.000000E+00),complex(+0.000000E+00,+2.777011E-01),complex(-2.190985E-01,+0.000000E+00),complex(+0.000000E+00,+2.439453E-02),complex(-2.427224E-01,+0.000000E+00),complex(+0.000000E+00,-2.427224E-01),complex(+2.439458E-02,+0.000000E+00),complex(+0.000000E+00,-2.190984E-01),complex(+2.777011E-01,+0.000000E+00),complex(+0.000000E+00,+1.052515E-01),complex(+1.602038E-01,+0.000000E+00),complex(+0.000000E+00,+3.342185E-01),complex(-3.353351E-01,+0.000000E+00),complex(+0.000000E+00,-2.169815E-01),complex(+8.598190E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.299088E-02),complex(+1.282164E-01,+0.000000E+00),complex(+0.000000E+00,+2.465699E-01),complex(-3.411239E-01,+0.000000E+00),complex(+0.000000E+00,-3.342184E-01),complex(+1.865823E-01,+0.000000E+00),complex(+0.000000E+00,-5.115548E-02),complex(+2.489755E-01,+0.000000E+00),complex(+0.000000E+00,+2.805370E-01),complex(-1.213612E-01,+0.000000E+00),complex(+0.000000E+00,+1.213612E-01),complex(-2.805370E-01,+0.000000E+00),complex(+0.000000E+00,-2.489754E-01),complex(+5.115550E-02,+0.000000E+00),complex(+0.000000E+00,-1.865823E-01),complex(+3.342184E-01,+0.000000E+00),complex(+0.000000E+00,+3.411238E-01),complex(-2.465699E-01,+0.000000E+00),complex(+0.000000E+00,-1.282164E-01),complex(+4.299095E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.805981E-02),complex(-6.214803E-02,+0.000000E+00),complex(+0.000000E+00,-1.422500E-01),complex(+2.465699E-01,+0.000000E+00),complex(+0.000000E+00,+3.353352E-01),complex(-3.530459E-01,+0.000000E+00),complex(+0.000000E+00,-2.609452E-01),complex(+7.112165E-02,+0.000000E+00),complex(+0.000000E+00,-1.451764E-01),complex(+2.888975E-01,+0.000000E+00),complex(+0.000000E+00,+2.888975E-01),complex(-1.451765E-01,+0.000000E+00),complex(+0.000000E+00,+7.112169E-02),complex(-2.609450E-01,+0.000000E+00),complex(+0.000000E+00,-3.530459E-01),complex(+3.353351E-01,+0.000000E+00),complex(+0.000000E+00,+2.465699E-01),complex(-1.422500E-01,+0.000000E+00),complex(+0.000000E+00,-6.214809E-02),complex(+1.805980E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-6.019942E-03),complex(+2.347817E-02,+0.000000E+00),complex(+0.000000E+00,+6.214799E-02),complex(-1.282164E-01,+0.000000E+00),complex(+0.000000E+00,-2.169815E-01),complex(+3.074915E-01,+0.000000E+00),complex(+0.000000E+00,+3.653231E-01),complex(-3.556083E-01,+0.000000E+00),complex(+0.000000E+00,-2.613176E-01),complex(+9.629907E-02,+0.000000E+00),complex(+0.000000E+00,-9.629915E-02),complex(+2.613176E-01,+0.000000E+00),complex(+0.000000E+00,+3.556083E-01),complex(-3.653231E-01,+0.000000E+00),complex(+0.000000E+00,-3.074916E-01),complex(+2.169815E-01,+0.000000E+00),complex(+0.000000E+00,+1.282164E-01),complex(-6.214802E-02,+0.000000E+00),complex(+0.000000E+00,-2.347822E-02),complex(+6.019992E-03,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.381037E-03),complex(-6.019881E-03,+0.000000E+00),complex(+0.000000E+00,-1.805978E-02),complex(+4.299092E-02,+0.000000E+00),complex(+0.000000E+00,+8.598189E-02),complex(-1.489249E-01,+0.000000E+00),complex(+0.000000E+00,-2.274867E-01),complex(+3.100120E-01,+0.000000E+00),complex(+0.000000E+00,+3.796857E-01),complex(-4.197584E-01,+0.000000E+00),complex(+0.000000E+00,-4.197583E-01),complex(+3.796857E-01,+0.000000E+00),complex(+0.000000E+00,+3.100120E-01),complex(-2.274866E-01,+0.000000E+00),complex(+0.000000E+00,-1.489250E-01),complex(+8.598188E-02,+0.000000E+00),complex(+0.000000E+00,+4.299094E-02),complex(-1.805981E-02,+0.000000E+00),complex(+0.000000E+00,-6.019914E-03),complex(+1.381090E-03,-0.000000E+00)]$
            ]
          end

      18: begin
            matrix[171:189,171:189]=[$
            [complex(-1.953075E-03,+0.000000E+00),complex(+0.000000E+00,+8.286358E-03),complex(+2.415878E-02,+0.000000E+00),complex(+0.000000E+00,-5.579232E-02),complex(-1.080415E-01,+0.000000E+00),complex(+0.000000E+00,+1.807880E-01),complex(+2.661128E-01,+0.000000E+00),complex(+0.000000E+00,-3.484234E-01),complex(-4.085627E-01,+0.000000E+00),complex(+0.000000E+00,+4.306629E-01),complex(+4.085628E-01,+0.000000E+00),complex(+0.000000E+00,-3.484235E-01),complex(-2.661128E-01,+0.000000E+00),complex(+0.000000E+00,+1.807880E-01),complex(+1.080415E-01,+0.000000E+00),complex(+0.000000E+00,-5.579240E-02),complex(-2.415880E-02,+0.000000E+00),complex(+0.000000E+00,+8.286353E-03),complex(+1.953126E-03,-0.000000E+00)],$
            [complex(+8.286435E-03,+0.000000E+00),complex(+0.000000E+00,-3.124997E-02),complex(-7.972010E-02,+0.000000E+00),complex(+0.000000E+00,+1.578049E-01),complex(+2.546564E-01,+0.000000E+00),complex(+0.000000E+00,-3.408973E-01),complex(-3.763403E-01,+0.000000E+00),complex(+0.000000E+00,+3.284968E-01),complex(+1.925983E-01,+0.000000E+00),complex(+0.000000E+00,+3.196842E-08),complex(+1.925983E-01,+0.000000E+00),complex(+0.000000E+00,-3.284968E-01),complex(-3.763405E-01,+0.000000E+00),complex(+0.000000E+00,+3.408972E-01),complex(+2.546563E-01,+0.000000E+00),complex(+0.000000E+00,-1.578047E-01),complex(-7.972004E-02,+0.000000E+00),complex(+0.000000E+00,+3.125000E-02),complex(+8.286423E-03,+0.000000E+00)],$
            [complex(-2.415886E-02,+0.000000E+00),complex(+0.000000E+00,+7.972001E-02),complex(+1.738281E-01,+0.000000E+00),complex(+0.000000E+00,-2.841645E-01),complex(-3.581202E-01,+0.000000E+00),complex(+0.000000E+00,+3.361645E-01),complex(+1.936255E-01,+0.000000E+00),complex(+0.000000E+00,+2.816837E-02),complex(+2.312124E-01,+0.000000E+00),complex(+0.000000E+00,-3.133533E-01),complex(-2.312124E-01,+0.000000E+00),complex(+0.000000E+00,+2.816836E-02),complex(-1.936255E-01,+0.000000E+00),complex(+0.000000E+00,+3.361645E-01),complex(+3.581203E-01,+0.000000E+00),complex(+0.000000E+00,-2.841646E-01),complex(-1.738281E-01,+0.000000E+00),complex(+0.000000E+00,+7.972005E-02),complex(+2.415883E-02,-0.000000E+00)],$
            [complex(+5.579227E-02,+0.000000E+00),complex(+0.000000E+00,-1.578047E-01),complex(-2.841644E-01,+0.000000E+00),complex(+0.000000E+00,+3.593751E-01),complex(+3.025768E-01,+0.000000E+00),complex(+0.000000E+00,-1.012616E-01),complex(+1.490529E-01,+0.000000E+00),complex(+0.000000E+00,-2.927343E-01),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+4.556662E-08),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+2.927343E-01),complex(+1.490531E-01,+0.000000E+00),complex(+0.000000E+00,+1.012615E-01),complex(+3.025768E-01,+0.000000E+00),complex(+0.000000E+00,-3.593750E-01),complex(-2.841647E-01,+0.000000E+00),complex(+0.000000E+00,+1.578047E-01),complex(+5.579240E-02,-0.000000E+00)],$
            [complex(-1.080416E-01,+0.000000E+00),complex(-0.000000E+00,+2.546563E-01),complex(+3.581203E-01,+0.000000E+00),complex(+0.000000E+00,-3.025768E-01),complex(-7.031251E-02,+0.000000E+00),complex(+0.000000E+00,-1.960921E-01),complex(-2.886399E-01,+0.000000E+00),complex(-0.000000E+00,+1.259728E-01),complex(-1.477161E-01,+0.000000E+00),complex(+0.000000E+00,+2.802717E-01),complex(+1.477161E-01,+0.000000E+00),complex(+0.000000E+00,+1.259729E-01),complex(+2.886400E-01,+0.000000E+00),complex(+0.000000E+00,-1.960922E-01),complex(+7.031247E-02,+0.000000E+00),complex(+0.000000E+00,-3.025768E-01),complex(-3.581203E-01,+0.000000E+00),complex(+0.000000E+00,+2.546563E-01),complex(+1.080416E-01,+0.000000E+00)],$
            [complex(+1.807881E-01,+0.000000E+00),complex(+0.000000E+00,-3.408973E-01),complex(-3.361646E-01,+0.000000E+00),complex(+0.000000E+00,+1.012616E-01),complex(-1.960921E-01,+0.000000E+00),complex(+0.000000E+00,+2.812501E-01),complex(+6.899814E-02,+0.000000E+00),complex(+0.000000E+00,+2.107927E-01),complex(+2.471766E-01,+0.000000E+00),complex(+0.000000E+00,-2.862802E-09),complex(+2.471765E-01,+0.000000E+00),complex(+0.000000E+00,-2.107928E-01),complex(+6.899820E-02,+0.000000E+00),complex(+0.000000E+00,-2.812500E-01),complex(-1.960922E-01,+0.000000E+00),complex(+0.000000E+00,-1.012616E-01),complex(-3.361646E-01,+0.000000E+00),complex(+0.000000E+00,+3.408972E-01),complex(+1.807881E-01,-0.000000E+00)],$
            [complex(-2.661129E-01,+0.000000E+00),complex(+0.000000E+00,+3.763404E-01),complex(+1.936255E-01,+0.000000E+00),complex(+0.000000E+00,+1.490531E-01),complex(+2.886400E-01,+0.000000E+00),complex(+0.000000E+00,-6.899807E-02),complex(+2.265625E-01,+0.000000E+00),complex(+0.000000E+00,-2.148082E-01),complex(+8.396158E-02,+0.000000E+00),complex(+0.000000E+00,-2.655101E-01),complex(-8.396161E-02,+0.000000E+00),complex(+0.000000E+00,-2.148082E-01),complex(-2.265625E-01,+0.000000E+00),complex(+0.000000E+00,-6.899810E-02),complex(-2.886399E-01,+0.000000E+00),complex(+0.000000E+00,+1.490530E-01),complex(-1.936255E-01,+0.000000E+00),complex(+0.000000E+00,+3.763403E-01),complex(+2.661128E-01,-0.000000E+00)],$
            [complex(+3.484236E-01,+0.000000E+00),complex(+0.000000E+00,-3.284968E-01),complex(+2.816839E-02,+0.000000E+00),complex(+0.000000E+00,-2.927343E-01),complex(-1.259728E-01,+0.000000E+00),complex(+0.000000E+00,-2.107927E-01),complex(-2.148082E-01,+0.000000E+00),complex(-0.000000E+00,-1.093751E-01),complex(-2.565072E-01,+0.000000E+00),complex(+0.000000E+00,+5.191914E-08),complex(-2.565071E-01,+0.000000E+00),complex(+0.000000E+00,+1.093750E-01),complex(-2.148083E-01,+0.000000E+00),complex(+0.000000E+00,+2.107928E-01),complex(-1.259728E-01,+0.000000E+00),complex(+0.000000E+00,+2.927343E-01),complex(+2.816839E-02,+0.000000E+00),complex(+0.000000E+00,+3.284968E-01),complex(+3.484235E-01,-0.000000E+00)],$
            [complex(-4.085627E-01,+0.000000E+00),complex(+0.000000E+00,+1.925983E-01),complex(-2.312124E-01,+0.000000E+00),complex(+0.000000E+00,+2.288410E-01),complex(-1.477162E-01,+0.000000E+00),complex(+0.000000E+00,+2.471765E-01),complex(-8.396155E-02,+0.000000E+00),complex(+0.000000E+00,+2.565070E-01),complex(-2.734384E-02,+0.000000E+00),complex(+0.000000E+00,+2.594055E-01),complex(+2.734369E-02,+0.000000E+00),complex(+0.000000E+00,+2.565071E-01),complex(+8.396161E-02,+0.000000E+00),complex(+0.000000E+00,+2.471765E-01),complex(+1.477161E-01,+0.000000E+00),complex(+0.000000E+00,+2.288410E-01),complex(+2.312124E-01,+0.000000E+00),complex(+0.000000E+00,+1.925983E-01),complex(+4.085627E-01,-0.000000E+00)],$
            [complex(+4.306633E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+3.133534E-01,+0.000000E+00),0,complex(+2.802718E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+2.655101E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+2.594054E-01,+0.000000E+00),complex(+0.000000E+00,-1.444425E-07),complex(+2.594055E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+2.655100E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-07),complex(+2.802718E-01,+0.000000E+00),0,complex(+3.133533E-01,+0.000000E+00),complex(+0.000000E+00,+8.940697E-08),complex(+4.306629E-01,-0.000000E+00)],$
            [complex(-4.085626E-01,+0.000000E+00),complex(+0.000000E+00,-1.925983E-01),complex(-2.312124E-01,+0.000000E+00),complex(+0.000000E+00,-2.288409E-01),complex(-1.477162E-01,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(-8.396168E-02,+0.000000E+00),complex(+0.000000E+00,-2.565071E-01),complex(-2.734378E-02,+0.000000E+00),complex(+0.000000E+00,-2.594056E-01),complex(+2.734366E-02,+0.000000E+00),complex(+0.000000E+00,-2.565070E-01),complex(+8.396164E-02,+0.000000E+00),complex(+0.000000E+00,-2.471765E-01),complex(+1.477161E-01,+0.000000E+00),complex(+0.000000E+00,-2.288409E-01),complex(+2.312124E-01,+0.000000E+00),complex(+0.000000E+00,-1.925983E-01),complex(+4.085627E-01,-0.000000E+00)],$
            [complex(+3.484235E-01,+0.000000E+00),complex(+0.000000E+00,+3.284968E-01),complex(+2.816842E-02,+0.000000E+00),complex(+0.000000E+00,+2.927343E-01),complex(-1.259728E-01,+0.000000E+00),complex(+0.000000E+00,+2.107928E-01),complex(-2.148082E-01,+0.000000E+00),complex(+0.000000E+00,+1.093750E-01),complex(-2.565071E-01,+0.000000E+00),complex(+0.000000E+00,-7.685507E-09),complex(-2.565072E-01,+0.000000E+00),complex(+0.000000E+00,-1.093750E-01),complex(-2.148083E-01,+0.000000E+00),complex(+0.000000E+00,-2.107928E-01),complex(-1.259728E-01,+0.000000E+00),complex(+0.000000E+00,-2.927342E-01),complex(+2.816834E-02,+0.000000E+00),complex(+0.000000E+00,-3.284968E-01),complex(+3.484235E-01,-0.000000E+00)],$
            [complex(-2.661129E-01,+0.000000E+00),complex(+0.000000E+00,-3.763404E-01),complex(+1.936256E-01,+0.000000E+00),complex(+0.000000E+00,-1.490531E-01),complex(+2.886399E-01,+0.000000E+00),complex(+0.000000E+00,+6.899808E-02),complex(+2.265625E-01,+0.000000E+00),complex(+0.000000E+00,+2.148082E-01),complex(+8.396161E-02,+0.000000E+00),complex(+0.000000E+00,+2.655101E-01),complex(-8.396169E-02,+0.000000E+00),complex(+0.000000E+00,+2.148082E-01),complex(-2.265625E-01,-0.000000E+00),complex(+0.000000E+00,+6.899808E-02),complex(-2.886399E-01,+0.000000E+00),complex(+0.000000E+00,-1.490530E-01),complex(-1.936255E-01,+0.000000E+00),complex(+0.000000E+00,-3.763404E-01),complex(+2.661129E-01,+0.000000E+00)],$
            [complex(+1.807882E-01,+0.000000E+00),complex(+0.000000E+00,+3.408972E-01),complex(-3.361645E-01,+0.000000E+00),complex(+0.000000E+00,-1.012617E-01),complex(-1.960922E-01,+0.000000E+00),complex(+0.000000E+00,-2.812501E-01),complex(+6.899805E-02,+0.000000E+00),complex(+0.000000E+00,-2.107929E-01),complex(+2.471763E-01,+0.000000E+00),complex(+0.000000E+00,-2.862782E-09),complex(+2.471764E-01,+0.000000E+00),complex(+0.000000E+00,+2.107927E-01),complex(+6.899816E-02,+0.000000E+00),complex(+0.000000E+00,+2.812500E-01),complex(-1.960922E-01,+0.000000E+00),complex(+0.000000E+00,+1.012616E-01),complex(-3.361645E-01,+0.000000E+00),complex(+0.000000E+00,-3.408972E-01),complex(+1.807882E-01,+0.000000E+00)],$
            [complex(-1.080416E-01,+0.000000E+00),complex(+0.000000E+00,-2.546565E-01),complex(+3.581203E-01,+0.000000E+00),complex(+0.000000E+00,+3.025768E-01),complex(-7.031244E-02,+0.000000E+00),complex(+0.000000E+00,+1.960922E-01),complex(-2.886399E-01,+0.000000E+00),complex(+0.000000E+00,-1.259727E-01),complex(-1.477161E-01,+0.000000E+00),complex(+0.000000E+00,-2.802717E-01),complex(+1.477161E-01,+0.000000E+00),complex(+0.000000E+00,-1.259728E-01),complex(+2.886399E-01,+0.000000E+00),complex(+0.000000E+00,+1.960922E-01),complex(+7.031244E-02,+0.000000E+00),complex(+0.000000E+00,+3.025769E-01),complex(-3.581203E-01,+0.000000E+00),complex(+0.000000E+00,-2.546564E-01),complex(+1.080416E-01,-0.000000E+00)],$
            [complex(+5.579235E-02,+0.000000E+00),complex(+0.000000E+00,+1.578048E-01),complex(-2.841645E-01,+0.000000E+00),complex(+0.000000E+00,-3.593749E-01),complex(+3.025769E-01,+0.000000E+00),complex(+0.000000E+00,+1.012617E-01),complex(+1.490530E-01,+0.000000E+00),complex(+0.000000E+00,+2.927344E-01),complex(-2.288408E-01,+0.000000E+00),complex(+0.000000E+00,+8.313713E-09),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,-2.927343E-01),complex(+1.490531E-01,+0.000000E+00),complex(+0.000000E+00,-1.012615E-01),complex(+3.025768E-01,+0.000000E+00),complex(+0.000000E+00,+3.593750E-01),complex(-2.841646E-01,+0.000000E+00),complex(+0.000000E+00,-1.578048E-01),complex(+5.579238E-02,-0.000000E+00)],$
            [complex(-2.415886E-02,+0.000000E+00),complex(+0.000000E+00,-7.972009E-02),complex(+1.738282E-01,+0.000000E+00),complex(+0.000000E+00,+2.841645E-01),complex(-3.581203E-01,+0.000000E+00),complex(+0.000000E+00,-3.361645E-01),complex(+1.936255E-01,+0.000000E+00),complex(+0.000000E+00,-2.816837E-02),complex(+2.312124E-01,+0.000000E+00),complex(+0.000000E+00,+3.133533E-01),complex(-2.312124E-01,+0.000000E+00),complex(+0.000000E+00,-2.816834E-02),complex(-1.936256E-01,+0.000000E+00),complex(+0.000000E+00,-3.361645E-01),complex(+3.581203E-01,+0.000000E+00),complex(+0.000000E+00,+2.841646E-01),complex(-1.738282E-01,+0.000000E+00),complex(+0.000000E+00,-7.972000E-02),complex(+2.415885E-02,-0.000000E+00)],$
            [complex(+8.286430E-03,+0.000000E+00),complex(+0.000000E+00,+3.124999E-02),complex(-7.972006E-02,+0.000000E+00),complex(+0.000000E+00,-1.578049E-01),complex(+2.546563E-01,+0.000000E+00),complex(+0.000000E+00,+3.408973E-01),complex(-3.763403E-01,+0.000000E+00),complex(+0.000000E+00,-3.284969E-01),complex(+1.925982E-01,+0.000000E+00),complex(+0.000000E+00,-2.297961E-08),complex(+1.925984E-01,+0.000000E+00),complex(+0.000000E+00,+3.284968E-01),complex(-3.763404E-01,+0.000000E+00),complex(+0.000000E+00,-3.408972E-01),complex(+2.546564E-01,+0.000000E+00),complex(+0.000000E+00,+1.578047E-01),complex(-7.972008E-02,+0.000000E+00),complex(+0.000000E+00,-3.124996E-02),complex(+8.286454E-03,-0.000000E+00)],$
            [complex(-1.953087E-03,+0.000000E+00),complex(+0.000000E+00,-8.286375E-03),complex(+2.415880E-02,+0.000000E+00),complex(+0.000000E+00,+5.579235E-02),complex(-1.080416E-01,+0.000000E+00),complex(+0.000000E+00,-1.807881E-01),complex(+2.661129E-01,+0.000000E+00),complex(+0.000000E+00,+3.484235E-01),complex(-4.085629E-01,+0.000000E+00),complex(+0.000000E+00,-4.306630E-01),complex(+4.085628E-01,+0.000000E+00),complex(+0.000000E+00,+3.484235E-01),complex(-2.661128E-01,+0.000000E+00),complex(+0.000000E+00,-1.807880E-01),complex(+1.080416E-01,+0.000000E+00),complex(+0.000000E+00,+5.579237E-02),complex(-2.415884E-02,+0.000000E+00),complex(+0.000000E+00,-8.286396E-03),complex(+1.953162E-03,-0.000000E+00)]$
            ]
          end

      17: begin
            matrix[153:170,153:170]=[$
            [complex(+0.000000E+00,+2.762188E-03),complex(+1.138857E-02,+0.000000E+00),complex(+0.000000E+00,-3.221178E-02),complex(-7.202771E-02,+0.000000E+00),complex(+0.000000E+00,+1.347514E-01),complex(+2.172802E-01,+0.000000E+00),complex(+0.000000E+00,-3.072807E-01),complex(-3.851967E-01,+0.000000E+00),complex(+0.000000E+00,+4.306630E-01),complex(+4.306630E-01,+0.000000E+00),complex(+0.000000E+00,-3.851967E-01),complex(-3.072806E-01,+0.000000E+00),complex(+0.000000E+00,+2.172803E-01),complex(+1.347515E-01,+0.000000E+00),complex(+0.000000E+00,-7.202768E-02),complex(-3.221180E-02,+0.000000E+00),complex(+0.000000E+00,+1.138854E-02),complex(+2.762143E-03,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.138860E-02),complex(-4.143199E-02,+0.000000E+00),complex(+0.000000E+00,+1.015625E-01),complex(+1.921622E-01,+0.000000E+00),complex(+0.000000E+00,-2.941383E-01),complex(-3.688874E-01,+0.000000E+00),complex(+0.000000E+00,+3.726324E-01),complex(+2.802717E-01,+0.000000E+00),complex(+0.000000E+00,-1.044512E-01),complex(+1.044513E-01,+0.000000E+00),complex(+0.000000E+00,-2.802717E-01),complex(-3.726325E-01,+0.000000E+00),complex(+0.000000E+00,+3.688873E-01),complex(+2.941383E-01,+0.000000E+00),complex(+0.000000E+00,-1.921621E-01),complex(-1.015625E-01,+0.000000E+00),complex(+0.000000E+00,+4.143206E-02),complex(+1.138858E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.221174E-02),complex(+1.015624E-01,+0.000000E+00),complex(+0.000000E+00,-2.099223E-01),complex(-3.211688E-01,+0.000000E+00),complex(+0.000000E+00,+3.697549E-01),complex(+2.981060E-01,+0.000000E+00),complex(+0.000000E+00,-1.053964E-01),complex(+1.321214E-01,+0.000000E+00),complex(+0.000000E+00,-2.954324E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+1.321214E-01),complex(-1.053964E-01,+0.000000E+00),complex(+0.000000E+00,+2.981060E-01),complex(+3.697550E-01,+0.000000E+00),complex(+0.000000E+00,-3.211688E-01),complex(-2.099223E-01,+0.000000E+00),complex(+0.000000E+00,+1.015625E-01),complex(+3.221176E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-7.202780E-02),complex(-1.921622E-01,+0.000000E+00),complex(+0.000000E+00,+3.211688E-01),complex(+3.646021E-01,+0.000000E+00),complex(+0.000000E+00,-2.480392E-01),complex(-2.980232E-08,+0.000000E+00),complex(+0.000000E+00,-2.356734E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+1.321214E-01),complex(-1.321214E-01,+0.000000E+00),complex(+0.000000E+00,+2.954324E-01),complex(+2.356734E-01,+0.000000E+00),complex(+0.000000E+00,+1.024455E-07),complex(+2.480392E-01,+0.000000E+00),complex(+0.000000E+00,-3.646019E-01),complex(-3.211688E-01,+0.000000E+00),complex(+0.000000E+00,+1.921620E-01),complex(+7.202768E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.347514E-01),complex(+2.941382E-01,+0.000000E+00),complex(-0.000000E+00,-3.697551E-01),complex(-2.480392E-01,+0.000000E+00),complex(+0.000000E+00,-3.314560E-02),complex(-2.672286E-01,+0.000000E+00),complex(+0.000000E+00,+2.519456E-01),complex(-3.725290E-09,+0.000000E+00),complex(+0.000000E+00,+2.471764E-01),complex(+2.471764E-01,+0.000000E+00),complex(+0.000000E+00,+7.078052E-08),complex(+2.519455E-01,-0.000000E+00),complex(-0.000000E+00,-2.672286E-01),complex(-3.314559E-02,+0.000000E+00),complex(-0.000000E+00,-2.480392E-01),complex(-3.697549E-01,+0.000000E+00),complex(+0.000000E+00,+2.941383E-01),complex(+1.347515E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.172803E-01),complex(-3.688873E-01,+0.000000E+00),complex(+0.000000E+00,+2.981058E-01),complex(-2.980232E-08,+0.000000E+00),complex(+0.000000E+00,+2.672287E-01),complex(+2.320194E-01,+0.000000E+00),complex(+0.000000E+00,+6.250002E-02),complex(+2.742175E-01,+0.000000E+00),complex(+0.000000E+00,-1.532922E-01),complex(+1.532923E-01,+0.000000E+00),complex(+0.000000E+00,-2.742176E-01),complex(-6.249999E-02,+0.000000E+00),complex(+0.000000E+00,-2.320195E-01),complex(-2.672286E-01,+0.000000E+00),complex(+0.000000E+00,+3.725290E-08),complex(-2.981060E-01,+0.000000E+00),complex(+0.000000E+00,+3.688874E-01),complex(+2.172803E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.072806E-01),complex(+3.726325E-01,+0.000000E+00),complex(+0.000000E+00,-1.053964E-01),complex(+2.356735E-01,+0.000000E+00),complex(+0.000000E+00,-2.519457E-01),complex(+6.250007E-02,+0.000000E+00),complex(+0.000000E+00,-2.762137E-01),complex(-9.695055E-02,+0.000000E+00),complex(+0.000000E+00,-2.167879E-01),complex(-2.167881E-01,+0.000000E+00),complex(+0.000000E+00,-9.695060E-02),complex(-2.762135E-01,+0.000000E+00),complex(+0.000000E+00,+6.250004E-02),complex(-2.519455E-01,+0.000000E+00),complex(+0.000000E+00,+2.356735E-01),complex(-1.053963E-01,+0.000000E+00),complex(+0.000000E+00,+3.726325E-01),complex(+3.072807E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.851967E-01),complex(-2.802718E-01,+0.000000E+00),complex(+0.000000E+00,-1.321213E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,-7.450581E-08),complex(-2.742176E-01,+0.000000E+00),complex(+0.000000E+00,+9.695053E-02),complex(-2.320194E-01,+0.000000E+00),complex(+0.000000E+00,+1.729370E-01),complex(-1.729370E-01,+0.000000E+00),complex(+0.000000E+00,+2.320193E-01),complex(-9.695056E-02,+0.000000E+00),complex(+0.000000E+00,+2.742176E-01),complex(+7.450581E-08,+0.000000E+00),complex(+0.000000E+00,+2.954324E-01),complex(+1.321214E-01,+0.000000E+00),complex(+0.000000E+00,+2.802717E-01),complex(+3.851967E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.306630E-01),complex(+1.044511E-01,+0.000000E+00),complex(+0.000000E+00,+2.954324E-01),complex(+1.321215E-01,+0.000000E+00),complex(+0.000000E+00,+2.471763E-01),complex(+1.532923E-01,+0.000000E+00),complex(+0.000000E+00,+2.167881E-01),complex(+1.729370E-01,+0.000000E+00),complex(+0.000000E+00,+1.933496E-01),complex(+1.933496E-01,+0.000000E+00),complex(+0.000000E+00,+1.729371E-01),complex(+2.167881E-01,+0.000000E+00),complex(+0.000000E+00,+1.532923E-01),complex(+2.471765E-01,+0.000000E+00),complex(+0.000000E+00,+1.321214E-01),complex(+2.954324E-01,+0.000000E+00),complex(+0.000000E+00,+1.044512E-01),complex(+4.306630E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.306629E-01),complex(+1.044511E-01,+0.000000E+00),complex(+0.000000E+00,-2.954324E-01),complex(+1.321214E-01,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(+1.532923E-01,+0.000000E+00),complex(+0.000000E+00,-2.167880E-01),complex(+1.729371E-01,+0.000000E+00),complex(+0.000000E+00,-1.933495E-01),complex(+1.933495E-01,+0.000000E+00),complex(+0.000000E+00,-1.729371E-01),complex(+2.167881E-01,+0.000000E+00),complex(+0.000000E+00,-1.532923E-01),complex(+2.471765E-01,+0.000000E+00),complex(+0.000000E+00,-1.321213E-01),complex(+2.954324E-01,+0.000000E+00),complex(+0.000000E+00,-1.044511E-01),complex(+4.306630E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+3.851968E-01),complex(-2.802716E-01,+0.000000E+00),complex(+0.000000E+00,+1.321214E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+1.043081E-07),complex(-2.742176E-01,+0.000000E+00),complex(+0.000000E+00,-9.695059E-02),complex(-2.320194E-01,+0.000000E+00),complex(+0.000000E+00,-1.729371E-01),complex(-1.729370E-01,+0.000000E+00),complex(+0.000000E+00,-2.320194E-01),complex(-9.695050E-02,+0.000000E+00),complex(+0.000000E+00,-2.742176E-01),complex(-4.470348E-08,+0.000000E+00),complex(+0.000000E+00,-2.954323E-01),complex(+1.321212E-01,+0.000000E+00),complex(+0.000000E+00,-2.802717E-01),complex(+3.851967E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.072806E-01),complex(+3.726327E-01,+0.000000E+00),complex(+0.000000E+00,+1.053963E-01),complex(+2.356735E-01,+0.000000E+00),complex(+0.000000E+00,+2.519455E-01),complex(+6.250004E-02,+0.000000E+00),complex(+0.000000E+00,+2.762136E-01),complex(-9.695055E-02,+0.000000E+00),complex(+0.000000E+00,+2.167881E-01),complex(-2.167880E-01,+0.000000E+00),complex(+0.000000E+00,+9.695064E-02),complex(-2.762135E-01,+0.000000E+00),complex(+0.000000E+00,-6.250002E-02),complex(-2.519456E-01,+0.000000E+00),complex(+0.000000E+00,-2.356735E-01),complex(-1.053965E-01,+0.000000E+00),complex(+0.000000E+00,-3.726325E-01),complex(+3.072807E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.172803E-01),complex(-3.688874E-01,+0.000000E+00),complex(+0.000000E+00,-2.981060E-01),0,complex(+0.000000E+00,-2.672286E-01),complex(+2.320194E-01,+0.000000E+00),complex(+0.000000E+00,-6.249993E-02),complex(+2.742177E-01,+0.000000E+00),complex(+0.000000E+00,+1.532923E-01),complex(+1.532923E-01,+0.000000E+00),complex(+0.000000E+00,+2.742176E-01),complex(-6.249994E-02,+0.000000E+00),complex(+0.000000E+00,+2.320194E-01),complex(-2.672286E-01,+0.000000E+00),complex(+0.000000E+00,+7.450581E-09),complex(-2.981061E-01,+0.000000E+00),complex(+0.000000E+00,-3.688874E-01),complex(+2.172802E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.347515E-01),complex(+2.941382E-01,+0.000000E+00),complex(+0.000000E+00,+3.697549E-01),complex(-2.480392E-01,+0.000000E+00),complex(+0.000000E+00,+3.314560E-02),complex(-2.672286E-01,+0.000000E+00),complex(+0.000000E+00,-2.519456E-01),complex(+4.842877E-08,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(+2.471765E-01,+0.000000E+00),complex(+0.000000E+00,-2.235174E-08),complex(+2.519456E-01,+0.000000E+00),complex(+0.000000E+00,+2.672286E-01),complex(-3.314563E-02,+0.000000E+00),complex(+0.000000E+00,+2.480392E-01),complex(-3.697549E-01,+0.000000E+00),complex(+0.000000E+00,-2.941383E-01),complex(+1.347515E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+7.202776E-02),complex(-1.921621E-01,+0.000000E+00),complex(+0.000000E+00,-3.211690E-01),complex(+3.646020E-01,+0.000000E+00),complex(+0.000000E+00,+2.480392E-01),complex(-4.284084E-08,+0.000000E+00),complex(+0.000000E+00,+2.356735E-01),complex(-2.954324E-01,+0.000000E+00),complex(+0.000000E+00,-1.321214E-01),complex(-1.321213E-01,+0.000000E+00),complex(+0.000000E+00,-2.954324E-01),complex(+2.356735E-01,+0.000000E+00),complex(+0.000000E+00,-9.499490E-08),complex(+2.480392E-01,+0.000000E+00),complex(+0.000000E+00,+3.646020E-01),complex(-3.211688E-01,+0.000000E+00),complex(+0.000000E+00,-1.921621E-01),complex(+7.202774E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.221174E-02),complex(+1.015625E-01,+0.000000E+00),complex(+0.000000E+00,+2.099223E-01),complex(-3.211688E-01,+0.000000E+00),complex(+0.000000E+00,-3.697550E-01),complex(+2.981060E-01,+0.000000E+00),complex(+0.000000E+00,+1.053964E-01),complex(+1.321214E-01,+0.000000E+00),complex(+0.000000E+00,+2.954324E-01),complex(-2.954324E-01,+0.000000E+00),complex(+0.000000E+00,-1.321214E-01),complex(-1.053964E-01,+0.000000E+00),complex(+0.000000E+00,-2.981060E-01),complex(+3.697550E-01,+0.000000E+00),complex(+0.000000E+00,+3.211688E-01),complex(-2.099224E-01,+0.000000E+00),complex(+0.000000E+00,-1.015624E-01),complex(+3.221175E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.138859E-02),complex(-4.143208E-02,+0.000000E+00),complex(+0.000000E+00,-1.015626E-01),complex(+1.921621E-01,+0.000000E+00),complex(+0.000000E+00,+2.941385E-01),complex(-3.688875E-01,+0.000000E+00),complex(+0.000000E+00,-3.726324E-01),complex(+2.802718E-01,+0.000000E+00),complex(+0.000000E+00,+1.044512E-01),complex(+1.044511E-01,+0.000000E+00),complex(+0.000000E+00,+2.802718E-01),complex(-3.726325E-01,+0.000000E+00),complex(+0.000000E+00,-3.688873E-01),complex(+2.941382E-01,+0.000000E+00),complex(+0.000000E+00,+1.921621E-01),complex(-1.015625E-01,+0.000000E+00),complex(+0.000000E+00,-4.143200E-02),complex(+1.138858E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.762089E-03),complex(+1.138853E-02,+0.000000E+00),complex(+0.000000E+00,+3.221178E-02),complex(-7.202769E-02,+0.000000E+00),complex(+0.000000E+00,-1.347514E-01),complex(+2.172802E-01,+0.000000E+00),complex(+0.000000E+00,+3.072806E-01),complex(-3.851966E-01,+0.000000E+00),complex(+0.000000E+00,-4.306630E-01),complex(+4.306629E-01,+0.000000E+00),complex(+0.000000E+00,+3.851967E-01),complex(-3.072806E-01,+0.000000E+00),complex(+0.000000E+00,-2.172802E-01),complex(+1.347515E-01,+0.000000E+00),complex(+0.000000E+00,+7.202769E-02),complex(-3.221183E-02,+0.000000E+00),complex(+0.000000E+00,-1.138857E-02),complex(+2.762161E-03,-0.000000E+00)]$
            ]
          end

      16: begin
            matrix[136:152,136:152]=[$
            [complex(+3.906270E-03,+0.000000E+00),complex(+0.000000E+00,-1.562497E-02),complex(-4.279088E-02,+0.000000E+00),complex(+0.000000E+00,+9.243869E-02),complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,-2.581674E-01),complex(-3.495602E-01,+0.000000E+00),complex(+0.000000E+00,+4.178044E-01),complex(+4.431485E-01,+0.000000E+00),complex(+0.000000E+00,-4.178044E-01),complex(-3.495603E-01,+0.000000E+00),complex(+0.000000E+00,+2.581674E-01),complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,-9.243876E-02),complex(-4.279083E-02,+0.000000E+00),complex(+0.000000E+00,+1.562500E-02),complex(+3.906284E-03,-0.000000E+00)],$
            [complex(-1.562496E-02,+0.000000E+00),complex(+0.000000E+00,+5.468754E-02),complex(+1.283725E-01,+0.000000E+00),complex(+0.000000E+00,-2.310969E-01),complex(-3.332927E-01,+0.000000E+00),complex(+0.000000E+00,+3.872511E-01),complex(+3.495602E-01,+0.000000E+00),complex(+0.000000E+00,-2.089022E-01),complex(+4.062895E-08,+0.000000E+00),complex(+0.000000E+00,-2.089022E-01),complex(-3.495603E-01,+0.000000E+00),complex(+0.000000E+00,+3.872510E-01),complex(+3.332926E-01,+0.000000E+00),complex(+0.000000E+00,-2.310968E-01),complex(-1.283725E-01,+0.000000E+00),complex(+0.000000E+00,+5.468753E-02),complex(+1.562497E-02,-0.000000E+00)],$
            [complex(+4.279085E-02,+0.000000E+00),complex(+0.000000E+00,-1.283724E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+3.544155E-01),complex(+3.651038E-01,+0.000000E+00),complex(+0.000000E+00,-2.356734E-01),complex(+1.210719E-08,+0.000000E+00),complex(+0.000000E+00,-2.288409E-01),complex(-3.236299E-01,+0.000000E+00),complex(+0.000000E+00,+2.288409E-01),complex(+2.561137E-08,+0.000000E+00),complex(+0.000000E+00,+2.356735E-01),complex(+3.651038E-01,+0.000000E+00),complex(+0.000000E+00,-3.544154E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+1.283725E-01),complex(+4.279087E-02,-0.000000E+00)],$
            [complex(-9.243874E-02,+0.000000E+00),complex(+0.000000E+00,+2.310970E-01),complex(+3.544154E-01,+0.000000E+00),complex(+0.000000E+00,-3.515625E-01),complex(-1.690102E-01,+0.000000E+00),complex(+0.000000E+00,-1.090956E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+2.471764E-01),complex(-4.379894E-10,+0.000000E+00),complex(+0.000000E+00,+2.471765E-01),complex(+2.954323E-01,+0.000000E+00),complex(+0.000000E+00,-1.090955E-01),complex(+1.690102E-01,+0.000000E+00),complex(+0.000000E+00,-3.515626E-01),complex(-3.544155E-01,+0.000000E+00),complex(+0.000000E+00,+2.310969E-01),complex(+9.243879E-02,-0.000000E+00)],$
            [complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,-3.332926E-01),complex(-3.651038E-01,+0.000000E+00),complex(+0.000000E+00,+1.690102E-01),complex(-1.406249E-01,+0.000000E+00),complex(+0.000000E+00,+3.025768E-01),complex(+1.638764E-01,+0.000000E+00),complex(+0.000000E+00,+1.371088E-01),complex(+2.908517E-01,+0.000000E+00),complex(+0.000000E+00,-1.371088E-01),complex(+1.638764E-01,+0.000000E+00),complex(+0.000000E+00,-3.025768E-01),complex(-1.406250E-01,+0.000000E+00),complex(+0.000000E+00,-1.690103E-01),complex(-3.651038E-01,+0.000000E+00),complex(+0.000000E+00,+3.332927E-01),complex(+1.666463E-01,+0.000000E+00)],$
            [complex(-2.581674E-01,+0.000000E+00),complex(+0.000000E+00,+3.872512E-01),complex(+2.356734E-01,+0.000000E+00),complex(+0.000000E+00,+1.090955E-01),complex(+3.025770E-01,+0.000000E+00),complex(+0.000000E+00,-1.328125E-01),complex(+1.904071E-01,+0.000000E+00),complex(+0.000000E+00,-2.655101E-01),complex(+2.500546E-08,+0.000000E+00),complex(+0.000000E+00,-2.655101E-01),complex(-1.904072E-01,+0.000000E+00),complex(+0.000000E+00,-1.328125E-01),complex(-3.025769E-01,+0.000000E+00),complex(+0.000000E+00,+1.090957E-01),complex(-2.356735E-01,+0.000000E+00),complex(+0.000000E+00,+3.872511E-01),complex(+2.581674E-01,-0.000000E+00)],$
            [complex(+3.495603E-01,+0.000000E+00),complex(+0.000000E+00,-3.495603E-01),complex(-2.980232E-08,+0.000000E+00),complex(+0.000000E+00,-2.954322E-01),complex(-1.638764E-01,+0.000000E+00),complex(+0.000000E+00,-1.904071E-01),complex(-2.500001E-01,+0.000000E+00),complex(+0.000000E+00,-6.536402E-02),complex(-2.773162E-01,+0.000000E+00),complex(+0.000000E+00,+6.536414E-02),complex(-2.499999E-01,+0.000000E+00),complex(+0.000000E+00,+1.904072E-01),complex(-1.638763E-01,+0.000000E+00),complex(+0.000000E+00,+2.954324E-01),complex(+1.490116E-08,+0.000000E+00),complex(+0.000000E+00,+3.495603E-01),complex(+3.495604E-01,-0.000000E+00)],$
            [complex(-4.178044E-01,+0.000000E+00),complex(+0.000000E+00,+2.089022E-01),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+2.471766E-01),complex(-1.371088E-01,+0.000000E+00),complex(+0.000000E+00,+2.655101E-01),complex(-6.536409E-02,+0.000000E+00),complex(+0.000000E+00,+2.734375E-01),complex(+1.556443E-08,+0.000000E+00),complex(+0.000000E+00,+2.734375E-01),complex(+6.536406E-02,+0.000000E+00),complex(+0.000000E+00,+2.655101E-01),complex(+1.371088E-01,+0.000000E+00),complex(+0.000000E+00,+2.471765E-01),complex(+2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+2.089022E-01),complex(+4.178045E-01,-0.000000E+00)],$
            [complex(+4.431485E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+3.236299E-01,+0.000000E+00),complex(+0.000000E+00,+1.192093E-07),complex(+2.908517E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+2.773162E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+2.734375E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+2.773163E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+2.908518E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+3.236299E-01,+0.000000E+00),complex(+0.000000E+00,-8.940697E-08),complex(+4.431486E-01,-0.000000E+00)],$
            [complex(-4.178044E-01,+0.000000E+00),complex(+0.000000E+00,-2.089022E-01),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(-1.371089E-01,+0.000000E+00),complex(+0.000000E+00,-2.655101E-01),complex(-6.536411E-02,+0.000000E+00),complex(+0.000000E+00,-2.734375E-01),complex(-2.423982E-08,+0.000000E+00),complex(+0.000000E+00,-2.734375E-01),complex(+6.536403E-02,+0.000000E+00),complex(+0.000000E+00,-2.655101E-01),complex(+1.371089E-01,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(+2.288410E-01,+0.000000E+00),complex(+0.000000E+00,-2.089022E-01),complex(+4.178044E-01,-0.000000E+00)],$
            [complex(+3.495604E-01,+0.000000E+00),complex(+0.000000E+00,+3.495603E-01),complex(-4.470348E-08,+0.000000E+00),complex(+0.000000E+00,+2.954325E-01),complex(-1.638764E-01,+0.000000E+00),complex(+0.000000E+00,+1.904072E-01),complex(-2.500001E-01,+0.000000E+00),complex(+0.000000E+00,+6.536405E-02),complex(-2.773163E-01,+0.000000E+00),complex(+0.000000E+00,-6.536397E-02),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,-1.904072E-01),complex(-1.638763E-01,+0.000000E+00),complex(+0.000000E+00,-2.954324E-01),complex(+2.980232E-08,+0.000000E+00),complex(+0.000000E+00,-3.495602E-01),complex(+3.495603E-01,-0.000000E+00)],$
            [complex(-2.581674E-01,+0.000000E+00),complex(+0.000000E+00,-3.872510E-01),complex(+2.356734E-01,+0.000000E+00),complex(+0.000000E+00,-1.090956E-01),complex(+3.025768E-01,+0.000000E+00),complex(+0.000000E+00,+1.328125E-01),complex(+1.904071E-01,+0.000000E+00),complex(+0.000000E+00,+2.655101E-01),complex(-2.115846E-08,+0.000000E+00),complex(+0.000000E+00,+2.655101E-01),complex(-1.904071E-01,+0.000000E+00),complex(+0.000000E+00,+1.328125E-01),complex(-3.025768E-01,+0.000000E+00),complex(+0.000000E+00,-1.090957E-01),complex(-2.356734E-01,+0.000000E+00),complex(+0.000000E+00,-3.872510E-01),complex(+2.581674E-01,+0.000000E+00)],$
            [complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,+3.332928E-01),complex(-3.651039E-01,+0.000000E+00),complex(+0.000000E+00,-1.690103E-01),complex(-1.406251E-01,+0.000000E+00),complex(+0.000000E+00,-3.025768E-01),complex(+1.638764E-01,+0.000000E+00),complex(+0.000000E+00,-1.371088E-01),complex(+2.908517E-01,+0.000000E+00),complex(+0.000000E+00,+1.371089E-01),complex(+1.638764E-01,+0.000000E+00),complex(+0.000000E+00,+3.025767E-01),complex(-1.406250E-01,+0.000000E+00),complex(+0.000000E+00,+1.690102E-01),complex(-3.651037E-01,+0.000000E+00),complex(+0.000000E+00,-3.332925E-01),complex(+1.666463E-01,-0.000000E+00)],$
            [complex(-9.243876E-02,+0.000000E+00),complex(+0.000000E+00,-2.310968E-01),complex(+3.544154E-01,+0.000000E+00),complex(+0.000000E+00,+3.515625E-01),complex(-1.690102E-01,+0.000000E+00),complex(+0.000000E+00,+1.090957E-01),complex(-2.954323E-01,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(+3.906834E-08,+0.000000E+00),complex(+0.000000E+00,-2.471764E-01),complex(+2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+1.090956E-01),complex(+1.690102E-01,+0.000000E+00),complex(+0.000000E+00,+3.515624E-01),complex(-3.544155E-01,+0.000000E+00),complex(+0.000000E+00,-2.310968E-01),complex(+9.243876E-02,-0.000000E+00)],$
            [complex(+4.279077E-02,+0.000000E+00),complex(+0.000000E+00,+1.283724E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,-3.544155E-01),complex(+3.651038E-01,+0.000000E+00),complex(+0.000000E+00,+2.356736E-01),complex(-6.845221E-08,+0.000000E+00),complex(+0.000000E+00,+2.288409E-01),complex(-3.236299E-01,+0.000000E+00),complex(+0.000000E+00,-2.288411E-01),complex(-3.539026E-08,+0.000000E+00),complex(+0.000000E+00,-2.356735E-01),complex(+3.651038E-01,+0.000000E+00),complex(+0.000000E+00,+3.544155E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,-1.283725E-01),complex(+4.279094E-02,-0.000000E+00)],$
            [complex(-1.562506E-02,+0.000000E+00),complex(+0.000000E+00,-5.468757E-02),complex(+1.283724E-01,+0.000000E+00),complex(+0.000000E+00,+2.310969E-01),complex(-3.332927E-01,+0.000000E+00),complex(+0.000000E+00,-3.872510E-01),complex(+3.495603E-01,+0.000000E+00),complex(+0.000000E+00,+2.089022E-01),complex(+5.820793E-10,+0.000000E+00),complex(+0.000000E+00,+2.089022E-01),complex(-3.495603E-01,+0.000000E+00),complex(+0.000000E+00,-3.872511E-01),complex(+3.332927E-01,+0.000000E+00),complex(+0.000000E+00,+2.310969E-01),complex(-1.283724E-01,+0.000000E+00),complex(+0.000000E+00,-5.468746E-02),complex(+1.562506E-02,-0.000000E+00)],$
            [complex(+3.906293E-03,+0.000000E+00),complex(+0.000000E+00,+1.562496E-02),complex(-4.279085E-02,+0.000000E+00),complex(+0.000000E+00,-9.243858E-02),complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,+2.581674E-01),complex(-3.495603E-01,+0.000000E+00),complex(+0.000000E+00,-4.178044E-01),complex(+4.431485E-01,+0.000000E+00),complex(+0.000000E+00,+4.178045E-01),complex(-3.495603E-01,+0.000000E+00),complex(+0.000000E+00,-2.581673E-01),complex(+1.666463E-01,+0.000000E+00),complex(+0.000000E+00,+9.243878E-02),complex(-4.279085E-02,+0.000000E+00),complex(+0.000000E+00,-1.562493E-02),complex(+3.906303E-03,-0.000000E+00)]$
            ]
          end

      15: begin
            matrix[120:135,120:135]=[$
            [complex(+0.000000E+00,-5.524215E-03),complex(-2.139550E-02,+0.000000E+00),complex(+0.000000E+00,+5.660695E-02),complex(+1.178367E-01,+0.000000E+00),complex(+0.000000E+00,-2.040992E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,+3.908203E-01),complex(+4.431485E-01,+0.000000E+00),complex(+0.000000E+00,-4.431486E-01),complex(-3.908203E-01,+0.000000E+00),complex(+0.000000E+00,+3.027281E-01),complex(+2.040993E-01,+0.000000E+00),complex(+0.000000E+00,-1.178368E-01),complex(-5.660694E-02,+0.000000E+00),complex(+0.000000E+00,+2.139542E-02),complex(+5.524225E-03,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.139542E-02),complex(+7.181553E-02,+0.000000E+00),complex(+0.000000E+00,-1.607744E-01),complex(-2.738278E-01,+0.000000E+00),complex(+0.000000E+00,+3.688874E-01),complex(+3.908203E-01,+0.000000E+00),complex(+0.000000E+00,-3.027281E-01),complex(-1.144205E-01,+0.000000E+00),complex(+0.000000E+00,-1.144205E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,+3.908203E-01),complex(+3.688874E-01,+0.000000E+00),complex(+0.000000E+00,-2.738278E-01),complex(-1.607744E-01,+0.000000E+00),complex(+0.000000E+00,+7.181549E-02),complex(+2.139541E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-5.660708E-02),complex(-1.607743E-01,+0.000000E+00),complex(+0.000000E+00,+2.927863E-01),complex(+3.794898E-01,+0.000000E+00),complex(+0.000000E+00,-3.386068E-01),complex(-1.477162E-01,+0.000000E+00),complex(+0.000000E+00,-1.144204E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,+3.027281E-01),complex(+1.144204E-01,+0.000000E+00),complex(+0.000000E+00,+1.477162E-01),complex(+3.386068E-01,+0.000000E+00),complex(+0.000000E+00,-3.794897E-01),complex(-2.927864E-01,+0.000000E+00),complex(+0.000000E+00,+1.607744E-01),complex(+5.660692E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.178366E-01),complex(+2.738279E-01,+0.000000E+00),complex(+0.000000E+00,-3.794896E-01),complex(-3.148835E-01,+0.000000E+00),complex(+0.000000E+00,+6.697825E-02),complex(-2.128817E-01,+0.000000E+00),complex(+0.000000E+00,+3.114729E-01),complex(+1.454258E-01,+0.000000E+00),complex(+0.000000E+00,+1.454259E-01),complex(+3.114730E-01,+0.000000E+00),complex(+0.000000E+00,-2.128817E-01),complex(+6.697821E-02,+0.000000E+00),complex(+0.000000E+00,-3.148835E-01),complex(-3.794897E-01,+0.000000E+00),complex(+0.000000E+00,+2.738279E-01),complex(+1.178367E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.040992E-01),complex(-3.688874E-01,+0.000000E+00),complex(+0.000000E+00,+3.386068E-01),complex(+6.697830E-02,+0.000000E+00),complex(+0.000000E+00,+2.375436E-01),complex(+2.867838E-01,+0.000000E+00),complex(+0.000000E+00,-3.173452E-02),complex(+2.518850E-01,+0.000000E+00),complex(+0.000000E+00,-2.518850E-01),complex(+3.173450E-02,+0.000000E+00),complex(+0.000000E+00,-2.867837E-01),complex(-2.375437E-01,+0.000000E+00),complex(+0.000000E+00,-6.697824E-02),complex(-3.386067E-01,+0.000000E+00),complex(+0.000000E+00,+3.688874E-01),complex(+2.040992E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.027280E-01),complex(+3.908202E-01,-0.000000E+00),complex(+0.000000E+00,-1.477161E-01),complex(+2.128817E-01,+0.000000E+00),complex(-0.000000E+00,-2.867836E-01),complex(+5.524345E-03,+0.000000E+00),complex(-0.000000E+00,-2.781403E-01),complex(-1.698208E-01,+0.000000E+00),complex(-0.000000E+00,-1.698208E-01),complex(-2.781404E-01,+0.000000E+00),complex(+0.000000E+00,+5.524285E-03),complex(-2.867836E-01,+0.000000E+00),complex(+0.000000E+00,+2.128816E-01),complex(-1.477161E-01,+0.000000E+00),complex(+0.000000E+00,+3.908202E-01),complex(+3.027280E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.908203E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,-1.144204E-01),complex(-3.114729E-01,+0.000000E+00),complex(+0.000000E+00,+3.173459E-02),complex(-2.781404E-01,+0.000000E+00),complex(+0.000000E+00,+1.381068E-01),complex(-2.192377E-01,+0.000000E+00),complex(+0.000000E+00,+2.192378E-01),complex(-1.381068E-01,+0.000000E+00),complex(+0.000000E+00,+2.781404E-01),complex(-3.173442E-02,+0.000000E+00),complex(+0.000000E+00,+3.114730E-01),complex(+1.144205E-01,+0.000000E+00),complex(+0.000000E+00,+3.027280E-01),complex(+3.908203E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.431488E-01),complex(+1.144205E-01,+0.000000E+00),complex(+0.000000E+00,+3.027281E-01),complex(+1.454259E-01,+0.000000E+00),complex(+0.000000E+00,+2.518850E-01),complex(+1.698208E-01,+0.000000E+00),complex(+0.000000E+00,+2.192378E-01),complex(+1.933494E-01,+0.000000E+00),complex(+0.000000E+00,+1.933495E-01),complex(+2.192377E-01,+0.000000E+00),complex(+0.000000E+00,+1.698208E-01),complex(+2.518849E-01,+0.000000E+00),complex(+0.000000E+00,+1.454259E-01),complex(+3.027280E-01,+0.000000E+00),complex(+0.000000E+00,+1.144204E-01),complex(+4.431485E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.431485E-01),complex(+1.144205E-01,+0.000000E+00),complex(+0.000000E+00,-3.027281E-01),complex(+1.454259E-01,+0.000000E+00),complex(+0.000000E+00,-2.518850E-01),complex(+1.698210E-01,+0.000000E+00),complex(+0.000000E+00,-2.192377E-01),complex(+1.933496E-01,+0.000000E+00),complex(+0.000000E+00,-1.933496E-01),complex(+2.192378E-01,+0.000000E+00),complex(+0.000000E+00,-1.698208E-01),complex(+2.518850E-01,+0.000000E+00),complex(+0.000000E+00,-1.454259E-01),complex(+3.027280E-01,+0.000000E+00),complex(+0.000000E+00,-1.144205E-01),complex(+4.431486E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.908204E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,+1.144204E-01),complex(-3.114728E-01,+0.000000E+00),complex(+0.000000E+00,-3.173453E-02),complex(-2.781402E-01,+0.000000E+00),complex(+0.000000E+00,-1.381068E-01),complex(-2.192376E-01,+0.000000E+00),complex(+0.000000E+00,-2.192378E-01),complex(-1.381067E-01,+0.000000E+00),complex(+0.000000E+00,-2.781403E-01),complex(-3.173448E-02,+0.000000E+00),complex(+0.000000E+00,-3.114730E-01),complex(+1.144204E-01,-0.000000E+00),complex(+0.000000E+00,-3.027281E-01),complex(+3.908202E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.027281E-01),complex(+3.908202E-01,+0.000000E+00),complex(+0.000000E+00,+1.477162E-01),complex(+2.128817E-01,+0.000000E+00),complex(+0.000000E+00,+2.867837E-01),complex(+5.524248E-03,+0.000000E+00),complex(+0.000000E+00,+2.781404E-01),complex(-1.698208E-01,+0.000000E+00),complex(+0.000000E+00,+1.698208E-01),complex(-2.781405E-01,+0.000000E+00),complex(+0.000000E+00,-5.524322E-03),complex(-2.867838E-01,+0.000000E+00),complex(+0.000000E+00,-2.128816E-01),complex(-1.477161E-01,+0.000000E+00),complex(+0.000000E+00,-3.908203E-01),complex(+3.027281E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.040992E-01),complex(-3.688874E-01,+0.000000E+00),complex(+0.000000E+00,-3.386067E-01),complex(+6.697821E-02,+0.000000E+00),complex(+0.000000E+00,-2.375437E-01),complex(+2.867836E-01,+0.000000E+00),complex(+0.000000E+00,+3.173448E-02),complex(+2.518850E-01,+0.000000E+00),complex(+0.000000E+00,+2.518850E-01),complex(+3.173444E-02,+0.000000E+00),complex(+0.000000E+00,+2.867837E-01),complex(-2.375438E-01,+0.000000E+00),complex(+0.000000E+00,+6.697825E-02),complex(-3.386068E-01,+0.000000E+00),complex(+0.000000E+00,-3.688874E-01),complex(+2.040993E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-1.178367E-01),complex(+2.738278E-01,+0.000000E+00),complex(+0.000000E+00,+3.794898E-01),complex(-3.148836E-01,+0.000000E+00),complex(+0.000000E+00,-6.697829E-02),complex(-2.128818E-01,+0.000000E+00),complex(+0.000000E+00,-3.114730E-01),complex(+1.454259E-01,+0.000000E+00),complex(+0.000000E+00,-1.454258E-01),complex(+3.114728E-01,+0.000000E+00),complex(+0.000000E+00,+2.128817E-01),complex(+6.697825E-02,+0.000000E+00),complex(-0.000000E+00,+3.148834E-01),complex(-3.794898E-01,+0.000000E+00),complex(+0.000000E+00,-2.738279E-01),complex(+1.178367E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+5.660691E-02),complex(-1.607743E-01,+0.000000E+00),complex(+0.000000E+00,-2.927864E-01),complex(+3.794897E-01,+0.000000E+00),complex(+0.000000E+00,+3.386067E-01),complex(-1.477161E-01,+0.000000E+00),complex(+0.000000E+00,+1.144205E-01),complex(-3.027281E-01,+0.000000E+00),complex(+0.000000E+00,-3.027281E-01),complex(+1.144204E-01,+0.000000E+00),complex(+0.000000E+00,-1.477162E-01),complex(+3.386068E-01,+0.000000E+00),complex(-0.000000E+00,+3.794897E-01),complex(-2.927864E-01,+0.000000E+00),complex(+0.000000E+00,-1.607743E-01),complex(+5.660695E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.139549E-02),complex(+7.181545E-02,+0.000000E+00),complex(+0.000000E+00,+1.607743E-01),complex(-2.738279E-01,+0.000000E+00),complex(+0.000000E+00,-3.688875E-01),complex(+3.908203E-01,+0.000000E+00),complex(+0.000000E+00,+3.027281E-01),complex(-1.144205E-01,+0.000000E+00),complex(+0.000000E+00,+1.144205E-01),complex(-3.027280E-01,+0.000000E+00),complex(+0.000000E+00,-3.908203E-01),complex(+3.688874E-01,+0.000000E+00),complex(-0.000000E+00,+2.738279E-01),complex(-1.607744E-01,+0.000000E+00),complex(+0.000000E+00,-7.181549E-02),complex(+2.139546E-02,-0.000000E+00)],$
            [complex(-0.000000E+00,+5.524323E-03),complex(-2.139536E-02,+0.000000E+00),complex(-0.000000E+00,-5.660698E-02),complex(+1.178368E-01,+0.000000E+00),complex(+0.000000E+00,+2.040993E-01),complex(-3.027280E-01,+0.000000E+00),complex(+0.000000E+00,-3.908203E-01),complex(+4.431486E-01,+0.000000E+00),complex(+0.000000E+00,+4.431486E-01),complex(-3.908202E-01,+0.000000E+00),complex(+0.000000E+00,-3.027281E-01),complex(+2.040992E-01,+0.000000E+00),complex(-0.000000E+00,+1.178367E-01),complex(-5.660696E-02,+0.000000E+00),complex(+0.000000E+00,-2.139546E-02),complex(+5.524276E-03,-0.000000E+00)]$
            ]
          end

      14: begin
            matrix[105:119,105:119]=[$
            [complex(-7.812563E-03,+0.000000E+00),complex(+0.000000E+00,+2.923165E-02),complex(+7.452651E-02,+0.000000E+00),complex(+0.000000E+00,-1.490530E-01),complex(-2.471764E-01,+0.000000E+00),complex(+0.000000E+00,+3.495604E-01),complex(+4.281222E-01,+0.000000E+00),complex(+0.000000E+00,-4.576819E-01),complex(-4.281222E-01,+0.000000E+00),complex(+0.000000E+00,+3.495602E-01),complex(+2.471764E-01,+0.000000E+00),complex(+0.000000E+00,-1.490530E-01),complex(-7.452646E-02,+0.000000E+00),complex(+0.000000E+00,+2.923172E-02),complex(+7.812493E-03,-0.000000E+00)],$
            [complex(+2.923160E-02,+0.000000E+00),complex(+0.000000E+00,-9.375009E-02),complex(-1.991805E-01,+0.000000E+00),complex(+0.000000E+00,+3.186888E-01),complex(+3.963641E-01,+0.000000E+00),complex(+0.000000E+00,-3.736956E-01),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,-9.313226E-09),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+3.736956E-01),complex(+3.963641E-01,+0.000000E+00),complex(+0.000000E+00,-3.186888E-01),complex(-1.991804E-01,+0.000000E+00),complex(+0.000000E+00,+9.375004E-02),complex(+2.923169E-02,-0.000000E+00)],$
            [complex(-7.452656E-02,+0.000000E+00),complex(+0.000000E+00,+1.991805E-01),complex(+3.359374E-01,+0.000000E+00),complex(+0.000000E+00,-3.906249E-01),complex(-2.850223E-01,+0.000000E+00),complex(+0.000000E+00,+3.664374E-02),complex(-2.243971E-01,+0.000000E+00),complex(+0.000000E+00,+3.358467E-01),complex(+2.243969E-01,+0.000000E+00),complex(+0.000000E+00,+3.664386E-02),complex(+2.850225E-01,+0.000000E+00),complex(+0.000000E+00,-3.906251E-01),complex(-3.359375E-01,+0.000000E+00),complex(+0.000000E+00,+1.991805E-01),complex(+7.452654E-02,-0.000000E+00)],$
            [complex(+1.490530E-01,+0.000000E+00),complex(+0.000000E+00,-3.186888E-01),complex(-3.906250E-01,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(-5.182231E-02,+0.000000E+00),complex(+0.000000E+00,+2.931509E-01),complex(+2.692763E-01,+0.000000E+00),0,complex(+2.692764E-01,+0.000000E+00),complex(+0.000000E+00,-2.931510E-01),complex(-5.182227E-02,+0.000000E+00),complex(+0.000000E+00,-2.500001E-01),complex(-3.906250E-01,+0.000000E+00),complex(+0.000000E+00,+3.186888E-01),complex(+1.490531E-01,-0.000000E+00)],$
            [complex(-2.471765E-01,+0.000000E+00),complex(+0.000000E+00,+3.963639E-01),complex(+2.850223E-01,+0.000000E+00),complex(+0.000000E+00,+5.182222E-02),complex(+3.046875E-01,+0.000000E+00),complex(+0.000000E+00,-2.099224E-01),complex(+1.217849E-01,+0.000000E+00),complex(+0.000000E+00,-3.037848E-01),complex(-1.217848E-01,+0.000000E+00),complex(+0.000000E+00,-2.099223E-01),complex(-3.046875E-01,+0.000000E+00),complex(+0.000000E+00,+5.182226E-02),complex(-2.850224E-01,+0.000000E+00),complex(+0.000000E+00,+3.963641E-01),complex(+2.471764E-01,-0.000000E+00)],$
            [complex(+3.495603E-01,+0.000000E+00),complex(-0.000000E+00,-3.736957E-01),complex(-3.664391E-02,+0.000000E+00),complex(-0.000000E+00,-2.931510E-01),complex(-2.099223E-01,+0.000000E+00),complex(+0.000000E+00,-1.562500E-01),complex(-2.870496E-01,+0.000000E+00),0,complex(-2.870496E-01,+0.000000E+00),complex(+0.000000E+00,+1.562500E-01),complex(-2.099223E-01,+0.000000E+00),complex(+0.000000E+00,+2.931510E-01),complex(-3.664388E-02,+0.000000E+00),complex(+0.000000E+00,+3.736957E-01),complex(+3.495603E-01,-0.000000E+00)],$
            [complex(-4.281221E-01,+0.000000E+00),complex(+0.000000E+00,+2.288409E-01),complex(-2.243970E-01,+0.000000E+00),complex(+0.000000E+00,+2.692764E-01),complex(-1.217848E-01,+0.000000E+00),complex(+0.000000E+00,+2.870495E-01),complex(-3.906253E-02,+0.000000E+00),complex(+0.000000E+00,+2.923170E-01),complex(+3.906254E-02,+0.000000E+00),complex(+0.000000E+00,+2.870495E-01),complex(+1.217848E-01,+0.000000E+00),complex(+0.000000E+00,+2.692764E-01),complex(+2.243970E-01,+0.000000E+00),complex(+0.000000E+00,+2.288409E-01),complex(+4.281221E-01,-0.000000E+00)],$
            [complex(+4.576819E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+3.358465E-01,+0.000000E+00),0,complex(+3.037847E-01,+0.000000E+00),0,complex(+2.923169E-01,+0.000000E+00),0,complex(+2.923170E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+3.037847E-01,+0.000000E+00),0,complex(+3.358466E-01,+0.000000E+00),complex(+0.000000E+00,+4.470348E-08),complex(+4.576819E-01,-0.000000E+00)],$
            [complex(-4.281221E-01,+0.000000E+00),complex(+0.000000E+00,-2.288409E-01),complex(-2.243970E-01,+0.000000E+00),complex(+0.000000E+00,-2.692764E-01),complex(-1.217848E-01,+0.000000E+00),complex(+0.000000E+00,-2.870496E-01),complex(-3.906251E-02,+0.000000E+00),complex(+0.000000E+00,-2.923170E-01),complex(+3.906249E-02,+0.000000E+00),complex(+0.000000E+00,-2.870495E-01),complex(+1.217848E-01,+0.000000E+00),complex(+0.000000E+00,-2.692763E-01),complex(+2.243970E-01,+0.000000E+00),complex(+0.000000E+00,-2.288409E-01),complex(+4.281221E-01,-0.000000E+00)],$
            [complex(+3.495604E-01,+0.000000E+00),complex(+0.000000E+00,+3.736957E-01),complex(-3.664395E-02,+0.000000E+00),complex(+0.000000E+00,+2.931509E-01),complex(-2.099223E-01,+0.000000E+00),complex(+0.000000E+00,+1.562500E-01),complex(-2.870495E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-08),complex(-2.870496E-01,+0.000000E+00),complex(+0.000000E+00,-1.562500E-01),complex(-2.099223E-01,+0.000000E+00),complex(+0.000000E+00,-2.931510E-01),complex(-3.664388E-02,-0.000000E+00),complex(+0.000000E+00,-3.736957E-01),complex(+3.495603E-01,-0.000000E+00)],$
            [complex(-2.471764E-01,+0.000000E+00),complex(+0.000000E+00,-3.963641E-01),complex(+2.850224E-01,+0.000000E+00),complex(+0.000000E+00,-5.182227E-02),complex(+3.046875E-01,+0.000000E+00),complex(+0.000000E+00,+2.099223E-01),complex(+1.217847E-01,+0.000000E+00),complex(-0.000000E+00,+3.037848E-01),complex(-1.217849E-01,+0.000000E+00),complex(+0.000000E+00,+2.099223E-01),complex(-3.046875E-01,+0.000000E+00),complex(+0.000000E+00,-5.182222E-02),complex(-2.850225E-01,+0.000000E+00),complex(+0.000000E+00,-3.963641E-01),complex(+2.471764E-01,-0.000000E+00)],$
            [complex(+1.490530E-01,+0.000000E+00),complex(+0.000000E+00,+3.186887E-01),complex(-3.906250E-01,-0.000000E+00),complex(+0.000000E+00,-2.499999E-01),complex(-5.182232E-02,-0.000000E+00),complex(+0.000000E+00,-2.931510E-01),complex(+2.692763E-01,+0.000000E+00),complex(+0.000000E+00,+2.235174E-08),complex(+2.692764E-01,+0.000000E+00),complex(+0.000000E+00,+2.931510E-01),complex(-5.182226E-02,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(-3.906249E-01,+0.000000E+00),complex(+0.000000E+00,-3.186887E-01),complex(+1.490530E-01,+0.000000E+00)],$
            [complex(-7.452655E-02,+0.000000E+00),complex(+0.000000E+00,-1.991806E-01),complex(+3.359375E-01,+0.000000E+00),complex(+0.000000E+00,+3.906249E-01),complex(-2.850224E-01,+0.000000E+00),complex(+0.000000E+00,-3.664384E-02),complex(-2.243971E-01,+0.000000E+00),complex(+0.000000E+00,-3.358467E-01),complex(+2.243969E-01,+0.000000E+00),complex(+0.000000E+00,-3.664390E-02),complex(+2.850225E-01,+0.000000E+00),complex(-0.000000E+00,+3.906250E-01),complex(-3.359375E-01,+0.000000E+00),complex(+0.000000E+00,-1.991804E-01),complex(+7.452650E-02,-0.000000E+00)],$
            [complex(+2.923160E-02,+0.000000E+00),complex(+0.000000E+00,+9.374999E-02),complex(-1.991805E-01,+0.000000E+00),complex(+0.000000E+00,-3.186887E-01),complex(+3.963641E-01,+0.000000E+00),complex(+0.000000E+00,+3.736957E-01),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,+7.450581E-09),complex(-2.288409E-01,+0.000000E+00),complex(+0.000000E+00,-3.736957E-01),complex(+3.963641E-01,+0.000000E+00),complex(-0.000000E+00,+3.186887E-01),complex(-1.991804E-01,+0.000000E+00),complex(+0.000000E+00,-9.375001E-02),complex(+2.923170E-02,-0.000000E+00)],$
            [complex(-7.812543E-03,+0.000000E+00),complex(+0.000000E+00,-2.923160E-02),complex(+7.452649E-02,+0.000000E+00),complex(+0.000000E+00,+1.490530E-01),complex(-2.471764E-01,+0.000000E+00),complex(+0.000000E+00,-3.495603E-01),complex(+4.281222E-01,+0.000000E+00),complex(-0.000000E+00,+4.576819E-01),complex(-4.281222E-01,+0.000000E+00),complex(+0.000000E+00,-3.495603E-01),complex(+2.471764E-01,+0.000000E+00),complex(-0.000000E+00,+1.490530E-01),complex(-7.452650E-02,+0.000000E+00),complex(+0.000000E+00,-2.923170E-02),complex(+7.812484E-03,-0.000000E+00)]$
            ]
          end

      13: begin
            matrix[91:104,91:104]=[$
            [complex(+0.000000E+00,+1.104856E-02),complex(+3.983606E-02,+0.000000E+00),complex(+0.000000E+00,-9.757807E-02),complex(-1.868479E-01,+0.000000E+00),complex(+0.000000E+00,+2.954322E-01),complex(+3.963641E-01,+0.000000E+00),complex(+0.000000E+00,-4.576819E-01),complex(-4.576818E-01,+0.000000E+00),complex(+0.000000E+00,+3.963642E-01),complex(+2.954323E-01,+0.000000E+00),complex(+0.000000E+00,-1.868478E-01),complex(-9.757809E-02,+0.000000E+00),complex(+0.000000E+00,+3.983602E-02),complex(+1.104853E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,-3.983605E-02),complex(-1.215340E-01,+0.000000E+00),complex(+0.000000E+00,+2.435697E-01),complex(+3.627558E-01,+0.000000E+00),complex(+0.000000E+00,-4.096911E-01),complex(-3.297948E-01,+0.000000E+00),complex(+0.000000E+00,+1.269380E-01),complex(-1.269382E-01,+0.000000E+00),complex(+0.000000E+00,+3.297948E-01),complex(+4.096909E-01,+0.000000E+00),complex(+0.000000E+00,-3.627558E-01),complex(-2.435696E-01,+0.000000E+00),complex(+0.000000E+00,+1.215340E-01),complex(+3.983614E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+9.757815E-02),complex(+2.435696E-01,+0.000000E+00),complex(+0.000000E+00,-3.756506E-01),complex(-3.808143E-01,+0.000000E+00),complex(+0.000000E+00,+2.007068E-01),complex(-8.975875E-02,+0.000000E+00),complex(+0.000000E+00,+3.109335E-01),complex(+3.109336E-01,+0.000000E+00),complex(+0.000000E+00,-8.975876E-02),complex(+2.007068E-01,+0.000000E+00),complex(+0.000000E+00,-3.808143E-01),complex(-3.756505E-01,-0.000000E+00),complex(+0.000000E+00,+2.435697E-01),complex(+9.757811E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.868477E-01),complex(-3.627558E-01,+0.000000E+00),complex(+0.000000E+00,+3.808142E-01),complex(+1.546797E-01,+0.000000E+00),complex(+0.000000E+00,+1.746928E-01),complex(+3.281250E-01,+0.000000E+00),complex(+0.000000E+00,-1.623798E-01),complex(+1.623797E-01,+0.000000E+00),complex(+0.000000E+00,-3.281251E-01),complex(-1.746928E-01,+0.000000E+00),complex(+0.000000E+00,-1.546796E-01),complex(-3.808143E-01,+0.000000E+00),complex(+0.000000E+00,+3.627558E-01),complex(+1.868478E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.954324E-01),complex(+4.096910E-01,+0.000000E+00),complex(+0.000000E+00,-2.007068E-01),complex(+1.746929E-01,+0.000000E+00),complex(+0.000000E+00,-3.204077E-01),complex(-7.411587E-02,+0.000000E+00),complex(+0.000000E+00,-2.567449E-01),complex(-2.567450E-01,+0.000000E+00),complex(+0.000000E+00,-7.411592E-02),complex(-3.204077E-01,+0.000000E+00),complex(+0.000000E+00,+1.746928E-01),complex(-2.007067E-01,-0.000000E+00),complex(+0.000000E+00,+4.096909E-01),complex(+2.954323E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.963640E-01),complex(-3.297949E-01,+0.000000E+00),complex(+0.000000E+00,-8.975881E-02),complex(-3.281249E-01,+0.000000E+00),complex(+0.000000E+00,+7.411585E-02),complex(-2.762136E-01,+0.000000E+00),complex(+0.000000E+00,+1.913664E-01),complex(-1.913664E-01,+0.000000E+00),complex(+0.000000E+00,+2.762136E-01),complex(-7.411583E-02,+0.000000E+00),complex(+0.000000E+00,+3.281249E-01),complex(+8.975881E-02,+0.000000E+00),complex(+0.000000E+00,+3.297948E-01),complex(+3.963641E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.576818E-01),complex(+1.269380E-01,+0.000000E+00),complex(+0.000000E+00,+3.109335E-01),complex(+1.623799E-01,+0.000000E+00),complex(+0.000000E+00,+2.567450E-01),complex(+1.913663E-01,+0.000000E+00),complex(+0.000000E+00,+2.209710E-01),complex(+2.209709E-01,+0.000000E+00),complex(+0.000000E+00,+1.913664E-01),complex(+2.567450E-01,+0.000000E+00),complex(+0.000000E+00,+1.623798E-01),complex(+3.109336E-01,+0.000000E+00),complex(+0.000000E+00,+1.269381E-01),complex(+4.576819E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.576818E-01),complex(+1.269381E-01,+0.000000E+00),complex(+0.000000E+00,-3.109336E-01),complex(+1.623799E-01,+0.000000E+00),complex(+0.000000E+00,-2.567450E-01),complex(+1.913663E-01,+0.000000E+00),complex(+0.000000E+00,-2.209708E-01),complex(+2.209708E-01,+0.000000E+00),complex(+0.000000E+00,-1.913664E-01),complex(+2.567450E-01,+0.000000E+00),complex(+0.000000E+00,-1.623798E-01),complex(+3.109336E-01,+0.000000E+00),complex(+0.000000E+00,-1.269381E-01),complex(+4.576818E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.963640E-01),complex(-3.297948E-01,+0.000000E+00),complex(+0.000000E+00,+8.975881E-02),complex(-3.281249E-01,-0.000000E+00),complex(+0.000000E+00,-7.411592E-02),complex(-2.762135E-01,+0.000000E+00),complex(+0.000000E+00,-1.913664E-01),complex(-1.913664E-01,-0.000000E+00),complex(+0.000000E+00,-2.762136E-01),complex(-7.411583E-02,+0.000000E+00),complex(+0.000000E+00,-3.281250E-01),complex(+8.975883E-02,+0.000000E+00),complex(+0.000000E+00,-3.297949E-01),complex(+3.963640E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-2.954323E-01),complex(+4.096910E-01,+0.000000E+00),complex(+0.000000E+00,+2.007068E-01),complex(+1.746929E-01,+0.000000E+00),complex(+0.000000E+00,+3.204078E-01),complex(-7.411580E-02,+0.000000E+00),complex(+0.000000E+00,+2.567449E-01),complex(-2.567450E-01,+0.000000E+00),complex(+0.000000E+00,+7.411580E-02),complex(-3.204077E-01,+0.000000E+00),complex(+0.000000E+00,-1.746928E-01),complex(-2.007067E-01,+0.000000E+00),complex(+0.000000E+00,-4.096910E-01),complex(+2.954322E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.868479E-01),complex(-3.627557E-01,+0.000000E+00),complex(+0.000000E+00,-3.808143E-01),complex(+1.546797E-01,+0.000000E+00),complex(+0.000000E+00,-1.746928E-01),complex(+3.281251E-01,+0.000000E+00),complex(+0.000000E+00,+1.623797E-01),complex(+1.623798E-01,+0.000000E+00),complex(+0.000000E+00,+3.281249E-01),complex(-1.746928E-01,+0.000000E+00),complex(+0.000000E+00,+1.546795E-01),complex(-3.808143E-01,+0.000000E+00),complex(+0.000000E+00,-3.627559E-01),complex(+1.868477E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-9.757812E-02),complex(+2.435697E-01,+0.000000E+00),complex(+0.000000E+00,+3.756505E-01),complex(-3.808143E-01,+0.000000E+00),complex(+0.000000E+00,-2.007067E-01),complex(-8.975878E-02,+0.000000E+00),complex(+0.000000E+00,-3.109336E-01),complex(+3.109336E-01,+0.000000E+00),complex(+0.000000E+00,+8.975876E-02),complex(+2.007068E-01,+0.000000E+00),complex(+0.000000E+00,+3.808143E-01),complex(-3.756505E-01,+0.000000E+00),complex(+0.000000E+00,-2.435696E-01),complex(+9.757807E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.983614E-02),complex(-1.215339E-01,+0.000000E+00),complex(+0.000000E+00,-2.435697E-01),complex(+3.627559E-01,+0.000000E+00),complex(+0.000000E+00,+4.096909E-01),complex(-3.297949E-01,+0.000000E+00),complex(+0.000000E+00,-1.269381E-01),complex(-1.269381E-01,+0.000000E+00),complex(+0.000000E+00,-3.297949E-01),complex(+4.096909E-01,+0.000000E+00),complex(+0.000000E+00,+3.627558E-01),complex(-2.435697E-01,+0.000000E+00),complex(+0.000000E+00,-1.215340E-01),complex(+3.983609E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.104848E-02),complex(+3.983610E-02,+0.000000E+00),complex(+0.000000E+00,+9.757810E-02),complex(-1.868478E-01,+0.000000E+00),complex(+0.000000E+00,-2.954324E-01),complex(+3.963641E-01,+0.000000E+00),complex(+0.000000E+00,+4.576818E-01),complex(-4.576819E-01,+0.000000E+00),complex(+0.000000E+00,-3.963641E-01),complex(+2.954323E-01,+0.000000E+00),complex(+0.000000E+00,+1.868478E-01),complex(-9.757807E-02,+0.000000E+00),complex(+0.000000E+00,-3.983606E-02),complex(+1.104853E-02,-0.000000E+00)]$
            ]
          end

      12: begin
            matrix[78:90,78:90]=[$
            [complex(+1.562498E-02,+0.000000E+00),complex(+0.000000E+00,-5.412656E-02),complex(-1.269381E-01,+0.000000E+00),complex(+0.000000E+00,+2.317562E-01),complex(+3.476343E-01,+0.000000E+00),complex(+0.000000E+00,-4.397266E-01),complex(-4.749589E-01,+0.000000E+00),complex(+0.000000E+00,+4.397265E-01),complex(+3.476343E-01,+0.000000E+00),complex(+0.000000E+00,-2.317562E-01),complex(-1.269381E-01,+0.000000E+00),complex(+0.000000E+00,+5.412656E-02),complex(+1.562502E-02,+0.000000E+00)],$
            [complex(-5.412657E-02,+0.000000E+00),complex(+0.000000E+00,+1.562500E-01),complex(+2.931510E-01,+0.000000E+00),complex(+0.000000E+00,-4.014137E-01),complex(-4.014136E-01,+0.000000E+00),complex(+0.000000E+00,+2.538761E-01),complex(-2.980232E-08,+0.000000E+00),complex(+0.000000E+00,+2.538762E-01),complex(+4.014135E-01,+0.000000E+00),complex(+0.000000E+00,-4.014134E-01),complex(-2.931510E-01,+0.000000E+00),complex(+0.000000E+00,+1.562500E-01),complex(+5.412663E-02,+0.000000E+00)],$
            [complex(+1.269381E-01,+0.000000E+00),complex(+0.000000E+00,-2.931509E-01),complex(-4.062500E-01,+0.000000E+00),complex(+0.000000E+00,+3.423265E-01),complex(+8.558162E-02,+0.000000E+00),complex(+0.000000E+00,+2.165063E-01),complex(+3.507804E-01,+0.000000E+00),complex(+0.000000E+00,-2.165063E-01),complex(+8.558160E-02,+0.000000E+00),complex(+0.000000E+00,-3.423265E-01),complex(-4.062500E-01,+0.000000E+00),complex(+0.000000E+00,+2.931510E-01),complex(+1.269381E-01,+0.000000E+00)],$
            [complex(-2.317561E-01,+0.000000E+00),complex(+0.000000E+00,+4.014136E-01),complex(+3.423267E-01,+0.000000E+00),complex(+0.000000E+00,-3.125003E-02),complex(+2.812501E-01,+0.000000E+00),complex(+0.000000E+00,-2.964635E-01),complex(-2.980232E-08,+0.000000E+00),complex(+0.000000E+00,-2.964636E-01),complex(-2.812500E-01,+0.000000E+00),complex(+0.000000E+00,-3.124994E-02),complex(-3.423265E-01,+0.000000E+00),complex(+0.000000E+00,+4.014134E-01),complex(+2.317561E-01,+0.000000E+00)],$
            [complex(+3.476344E-01,+0.000000E+00),complex(-0.000000E+00,-4.014135E-01),complex(-8.558163E-02,+0.000000E+00),complex(-0.000000E+00,-2.812500E-01),complex(-2.656250E-01,+0.000000E+00),complex(+0.000000E+00,-9.882118E-02),complex(-3.202173E-01,+0.000000E+00),complex(+0.000000E+00,+9.882119E-02),complex(-2.656250E-01,+0.000000E+00),complex(+0.000000E+00,+2.812500E-01),complex(-8.558162E-02,+0.000000E+00),complex(+0.000000E+00,+4.014135E-01),complex(+3.476343E-01,+0.000000E+00)],$
            [complex(-4.397265E-01,+0.000000E+00),complex(+0.000000E+00,+2.538762E-01),complex(-2.165064E-01,+0.000000E+00),complex(+0.000000E+00,+2.964636E-01),complex(-9.882117E-02,+0.000000E+00),complex(+0.000000E+00,+3.125000E-01),complex(-5.960464E-08,+0.000000E+00),complex(+0.000000E+00,+3.125001E-01),complex(+9.882123E-02,+0.000000E+00),complex(+0.000000E+00,+2.964635E-01),complex(+2.165064E-01,+0.000000E+00),complex(+0.000000E+00,+2.538762E-01),complex(+4.397265E-01,+0.000000E+00)],$
            [complex(+4.749588E-01,+0.000000E+00),complex(+0.000000E+00,-1.788139E-07),complex(+3.507804E-01,+0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+3.202172E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+3.125000E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+3.202173E-01,+0.000000E+00),complex(+0.000000E+00,-8.940697E-08),complex(+3.507804E-01,+0.000000E+00),complex(+0.000000E+00,+1.192093E-07),complex(+4.749588E-01,+0.000000E+00)],$
            [complex(-4.397265E-01,+0.000000E+00),complex(+0.000000E+00,-2.538762E-01),complex(-2.165063E-01,+0.000000E+00),complex(+0.000000E+00,-2.964635E-01),complex(-9.882119E-02,-0.000000E+00),complex(+0.000000E+00,-3.125000E-01),0,complex(+0.000000E+00,-3.125001E-01),complex(+9.882117E-02,+0.000000E+00),complex(+0.000000E+00,-2.964636E-01),complex(+2.165064E-01,+0.000000E+00),complex(+0.000000E+00,-2.538761E-01),complex(+4.397265E-01,+0.000000E+00)],$
            [complex(+3.476343E-01,+0.000000E+00),complex(+0.000000E+00,+4.014135E-01),complex(-8.558162E-02,+0.000000E+00),complex(+0.000000E+00,+2.812499E-01),complex(-2.656250E-01,+0.000000E+00),complex(+0.000000E+00,+9.882119E-02),complex(-3.202173E-01,+0.000000E+00),complex(+0.000000E+00,-9.882128E-02),complex(-2.656250E-01,+0.000000E+00),complex(+0.000000E+00,-2.812500E-01),complex(-8.558162E-02,-0.000000E+00),complex(+0.000000E+00,-4.014135E-01),complex(+3.476343E-01,+0.000000E+00)],$
            [complex(-2.317562E-01,+0.000000E+00),complex(+0.000000E+00,-4.014137E-01),complex(+3.423268E-01,+0.000000E+00),complex(+0.000000E+00,+3.125004E-02),complex(+2.812500E-01,+0.000000E+00),complex(+0.000000E+00,+2.964636E-01),complex(+4.470348E-08,+0.000000E+00),complex(+0.000000E+00,+2.964635E-01),complex(-2.812500E-01,+0.000000E+00),complex(+0.000000E+00,+3.124994E-02),complex(-3.423266E-01,+0.000000E+00),complex(+0.000000E+00,-4.014134E-01),complex(+2.317562E-01,+0.000000E+00)],$
            [complex(+1.269381E-01,+0.000000E+00),complex(-0.000000E+00,+2.931510E-01),complex(-4.062499E-01,+0.000000E+00),complex(+0.000000E+00,-3.423266E-01),complex(+8.558162E-02,+0.000000E+00),complex(+0.000000E+00,-2.165064E-01),complex(+3.507804E-01,+0.000000E+00),complex(+0.000000E+00,+2.165064E-01),complex(+8.558164E-02,+0.000000E+00),complex(-0.000000E+00,+3.423266E-01),complex(-4.062500E-01,+0.000000E+00),complex(+0.000000E+00,-2.931510E-01),complex(+1.269381E-01,+0.000000E+00)],$
            [complex(-5.412662E-02,+0.000000E+00),complex(+0.000000E+00,-1.562500E-01),complex(+2.931510E-01,+0.000000E+00),complex(+0.000000E+00,+4.014135E-01),complex(-4.014136E-01,+0.000000E+00),complex(+0.000000E+00,-2.538763E-01),complex(+1.490116E-08,+0.000000E+00),complex(+0.000000E+00,-2.538761E-01),complex(+4.014135E-01,+0.000000E+00),complex(-0.000000E+00,+4.014135E-01),complex(-2.931509E-01,+0.000000E+00),complex(+0.000000E+00,-1.562500E-01),complex(+5.412662E-02,+0.000000E+00)],$
            [complex(+1.562494E-02,+0.000000E+00),complex(+0.000000E+00,+5.412659E-02),complex(-1.269381E-01,+0.000000E+00),complex(+0.000000E+00,-2.317562E-01),complex(+3.476343E-01,+0.000000E+00),complex(+0.000000E+00,+4.397264E-01),complex(-4.749589E-01,+0.000000E+00),complex(+0.000000E+00,-4.397264E-01),complex(+3.476343E-01,-0.000000E+00),complex(-0.000000E+00,+2.317562E-01),complex(-1.269381E-01,+0.000000E+00),complex(+0.000000E+00,-5.412659E-02),complex(+1.562501E-02,+0.000000E+00)]$
            ]
          end

      11: begin
            matrix[66:77,66:77]=[$
            [complex(+0.000000E+00,-2.209715E-02),complex(-7.328776E-02,+0.000000E+00),complex(+0.000000E+00,+1.638764E-01),complex(+2.838423E-01,+0.000000E+00),complex(+0.000000E+00,-4.014135E-01),complex(-4.749588E-01,+0.000000E+00),complex(+0.000000E+00,+4.749590E-01),complex(+4.014136E-01,+0.000000E+00),complex(+0.000000E+00,-2.838422E-01),complex(-1.638764E-01,+0.000000E+00),complex(+0.000000E+00,+7.328776E-02),complex(+2.209708E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+7.328772E-02),complex(+1.988738E-01,+0.000000E+00),complex(+0.000000E+00,-3.458741E-01),complex(-4.279082E-01,+0.000000E+00),complex(+0.000000E+00,+3.630922E-01),complex(+1.432055E-01,+0.000000E+00),complex(+0.000000E+00,+1.432056E-01),complex(+3.630922E-01,+0.000000E+00),complex(+0.000000E+00,-4.279082E-01),complex(-3.458741E-01,+0.000000E+00),complex(+0.000000E+00,+1.988738E-01),complex(+7.328776E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.638763E-01),complex(-3.458742E-01,+0.000000E+00),complex(+0.000000E+00,+4.198447E-01),complex(+2.679128E-01,+0.000000E+00),complex(+0.000000E+00,+5.412655E-02),complex(+3.202174E-01,+0.000000E+00),complex(+0.000000E+00,-3.202171E-01),complex(-5.412658E-02,+0.000000E+00),complex(+0.000000E+00,-2.679130E-01),complex(-4.198447E-01,+0.000000E+00),complex(+0.000000E+00,+3.458741E-01),complex(+1.638765E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.838423E-01),complex(+4.279081E-01,+0.000000E+00),complex(+0.000000E+00,-2.679130E-01),complex(+1.104853E-01,+0.000000E+00),complex(+0.000000E+00,-3.437501E-01),complex(-1.848777E-01,+0.000000E+00),complex(+0.000000E+00,-1.848774E-01),complex(-3.437500E-01,+0.000000E+00),complex(+0.000000E+00,+1.104855E-01),complex(-2.679129E-01,+0.000000E+00),complex(+0.000000E+00,+4.279082E-01),complex(+2.838423E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.014135E-01),complex(-3.630922E-01,+0.000000E+00),complex(+0.000000E+00,-5.412661E-02),complex(-3.437501E-01,+0.000000E+00),complex(+0.000000E+00,+1.325825E-01),complex(-2.614563E-01,+0.000000E+00),complex(+0.000000E+00,+2.614563E-01),complex(-1.325825E-01,+0.000000E+00),complex(+0.000000E+00,+3.437501E-01),complex(+5.412661E-02,+0.000000E+00),complex(+0.000000E+00,+3.630922E-01),complex(+4.014136E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.749590E-01),complex(+1.432056E-01,+0.000000E+00),complex(+0.000000E+00,+3.202171E-01),complex(+1.848774E-01,+0.000000E+00),complex(+0.000000E+00,+2.614563E-01),complex(+2.209709E-01,+0.000000E+00),complex(+0.000000E+00,+2.209709E-01),complex(+2.614563E-01,+0.000000E+00),complex(+0.000000E+00,+1.848775E-01),complex(+3.202172E-01,+0.000000E+00),complex(+0.000000E+00,+1.432055E-01),complex(+4.749588E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.749589E-01),complex(+1.432055E-01,+0.000000E+00),complex(+0.000000E+00,-3.202172E-01),complex(+1.848774E-01,+0.000000E+00),complex(+0.000000E+00,-2.614563E-01),complex(+2.209709E-01,+0.000000E+00),complex(+0.000000E+00,-2.209708E-01),complex(+2.614563E-01,+0.000000E+00),complex(+0.000000E+00,-1.848775E-01),complex(+3.202173E-01,+0.000000E+00),complex(+0.000000E+00,-1.432055E-01),complex(+4.749589E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.014135E-01),complex(-3.630922E-01,+0.000000E+00),complex(+0.000000E+00,+5.412665E-02),complex(-3.437501E-01,-0.000000E+00),complex(+0.000000E+00,-1.325825E-01),complex(-2.614562E-01,+0.000000E+00),complex(+0.000000E+00,-2.614563E-01),complex(-1.325825E-01,+0.000000E+00),complex(+0.000000E+00,-3.437500E-01),complex(+5.412659E-02,-0.000000E+00),complex(+0.000000E+00,-3.630922E-01),complex(+4.014136E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.838423E-01),complex(+4.279084E-01,+0.000000E+00),complex(+0.000000E+00,+2.679128E-01),complex(+1.104855E-01,+0.000000E+00),complex(+0.000000E+00,+3.437499E-01),complex(-1.848774E-01,+0.000000E+00),complex(+0.000000E+00,+1.848776E-01),complex(-3.437499E-01,+0.000000E+00),complex(+0.000000E+00,-1.104854E-01),complex(-2.679129E-01,+0.000000E+00),complex(+0.000000E+00,-4.279083E-01),complex(+2.838423E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+1.638764E-01),complex(-3.458741E-01,+0.000000E+00),complex(+0.000000E+00,-4.198447E-01),complex(+2.679130E-01,+0.000000E+00),complex(+0.000000E+00,-5.412655E-02),complex(+3.202172E-01,+0.000000E+00),complex(+0.000000E+00,+3.202173E-01),complex(-5.412658E-02,+0.000000E+00),complex(+0.000000E+00,+2.679129E-01),complex(-4.198447E-01,+0.000000E+00),complex(+0.000000E+00,-3.458742E-01),complex(+1.638764E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,-7.328776E-02),complex(+1.988738E-01,+0.000000E+00),complex(+0.000000E+00,+3.458740E-01),complex(-4.279084E-01,+0.000000E+00),complex(+0.000000E+00,-3.630921E-01),complex(+1.432055E-01,+0.000000E+00),complex(-0.000000E+00,-1.432054E-01),complex(+3.630922E-01,-0.000000E+00),complex(-0.000000E+00,+4.279082E-01),complex(-3.458742E-01,+0.000000E+00),complex(+0.000000E+00,-1.988738E-01),complex(+7.328777E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,+2.209707E-02),complex(-7.328773E-02,+0.000000E+00),complex(+0.000000E+00,-1.638764E-01),complex(+2.838422E-01,+0.000000E+00),complex(+0.000000E+00,+4.014136E-01),complex(-4.749589E-01,+0.000000E+00),complex(+0.000000E+00,-4.749589E-01),complex(+4.014135E-01,-0.000000E+00),complex(-0.000000E+00,+2.838422E-01),complex(-1.638763E-01,+0.000000E+00),complex(+0.000000E+00,-7.328772E-02),complex(+2.209713E-02,-0.000000E+00)]$
            ]
          end

      10: begin
            matrix[55:65,55:65]=[$
            [complex(-3.124998E-02,+0.000000E+00),complex(+0.000000E+00,+9.882118E-02),complex(+2.096314E-01,+0.000000E+00),complex(+0.000000E+00,-3.423266E-01),complex(-4.528556E-01,+0.000000E+00),complex(+0.000000E+00,+4.960784E-01),complex(+4.528555E-01,+0.000000E+00),complex(+0.000000E+00,-3.423266E-01),complex(-2.096314E-01,+0.000000E+00),complex(+0.000000E+00,+9.882114E-02),complex(+3.124998E-02,-0.000000E+00)],$
            [complex(+9.882127E-02,+0.000000E+00),complex(+0.000000E+00,-2.500000E-01),complex(-3.977476E-01,+0.000000E+00),complex(+0.000000E+00,+4.330126E-01),complex(+2.864110E-01,+0.000000E+00),complex(+0.000000E+00,-4.470348E-08),complex(+2.864109E-01,+0.000000E+00),complex(+0.000000E+00,-4.330127E-01),complex(-3.977476E-01,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(+9.882116E-02,-0.000000E+00)],$
            [complex(-2.096313E-01,+0.000000E+00),complex(+0.000000E+00,+3.977475E-01),complex(+4.062500E-01,+0.000000E+00),complex(+0.000000E+00,-1.530932E-01),complex(+2.025232E-01,+0.000000E+00),complex(+0.000000E+00,-3.697550E-01),complex(-2.025232E-01,+0.000000E+00),complex(+0.000000E+00,-1.530930E-01),complex(-4.062501E-01,+0.000000E+00),complex(+0.000000E+00,+3.977475E-01),complex(+2.096313E-01,+0.000000E+00)],$
            [complex(+3.423266E-01,+0.000000E+00),complex(+0.000000E+00,-4.330129E-01),complex(-1.530930E-01,+0.000000E+00),complex(+0.000000E+00,-2.500000E-01),complex(-3.307188E-01,+0.000000E+00),complex(+0.000000E+00,-8.940697E-08),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,+2.500001E-01),complex(-1.530930E-01,+0.000000E+00),complex(+0.000000E+00,+4.330128E-01),complex(+3.423266E-01,-0.000000E+00)],$
            [complex(-4.528555E-01,+0.000000E+00),complex(+0.000000E+00,+2.864109E-01),complex(-2.025231E-01,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(-6.250003E-02,+0.000000E+00),complex(+0.000000E+00,+3.423266E-01),complex(+6.250004E-02,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(+2.025232E-01,+0.000000E+00),complex(+0.000000E+00,+2.864110E-01),complex(+4.528555E-01,-0.000000E+00)],$
            [complex(+4.960785E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-07),complex(+3.697550E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+3.423267E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+3.423266E-01,-0.000000E+00),complex(+0.000000E+00,+2.980232E-08),complex(+3.697549E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+4.960784E-01,-0.000000E+00)],$
            [complex(-4.528554E-01,+0.000000E+00),complex(+0.000000E+00,-2.864110E-01),complex(-2.025232E-01,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(-6.249991E-02,+0.000000E+00),complex(+0.000000E+00,-3.423267E-01),complex(+6.250001E-02,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(+2.025231E-01,+0.000000E+00),complex(+0.000000E+00,-2.864110E-01),complex(+4.528555E-01,-0.000000E+00)],$
            [complex(+3.423266E-01,+0.000000E+00),complex(+0.000000E+00,+4.330128E-01),complex(-1.530931E-01,-0.000000E+00),complex(+0.000000E+00,+2.500001E-01),complex(-3.307189E-01,-0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,-2.499999E-01),complex(-1.530931E-01,+0.000000E+00),complex(+0.000000E+00,-4.330127E-01),complex(+3.423266E-01,+0.000000E+00)],$
            [complex(-2.096313E-01,+0.000000E+00),complex(+0.000000E+00,-3.977475E-01),complex(+4.062500E-01,+0.000000E+00),complex(+0.000000E+00,+1.530931E-01),complex(+2.025231E-01,+0.000000E+00),complex(+0.000000E+00,+3.697551E-01),complex(-2.025232E-01,+0.000000E+00),complex(+0.000000E+00,+1.530931E-01),complex(-4.062500E-01,+0.000000E+00),complex(+0.000000E+00,-3.977476E-01),complex(+2.096314E-01,-0.000000E+00)],$
            [complex(+9.882117E-02,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(-3.977475E-01,+0.000000E+00),complex(+0.000000E+00,-4.330127E-01),complex(+2.864110E-01,+0.000000E+00),complex(+0.000000E+00,+8.195639E-08),complex(+2.864110E-01,+0.000000E+00),complex(+0.000000E+00,+4.330127E-01),complex(-3.977475E-01,+0.000000E+00),complex(+0.000000E+00,-2.500000E-01),complex(+9.882123E-02,-0.000000E+00)],$
            [complex(-3.125006E-02,+0.000000E+00),complex(+0.000000E+00,-9.882120E-02),complex(+2.096314E-01,+0.000000E+00),complex(+0.000000E+00,+3.423267E-01),complex(-4.528555E-01,+0.000000E+00),complex(+0.000000E+00,-4.960783E-01),complex(+4.528555E-01,+0.000000E+00),complex(+0.000000E+00,+3.423266E-01),complex(-2.096313E-01,+0.000000E+00),complex(+0.000000E+00,-9.882118E-02),complex(+3.125001E-02,-0.000000E+00)]$
            ]
          end

       9: begin
            matrix[45:54,45:54]=[$
            [complex(+0.000000E+00,+4.419409E-02),complex(+1.325825E-01,+0.000000E+00),complex(+0.000000E+00,-2.651650E-01),complex(-4.050463E-01,+0.000000E+00),complex(+0.000000E+00,+4.960783E-01),complex(+4.960784E-01,+0.000000E+00),complex(+0.000000E+00,-4.050463E-01),complex(-2.651650E-01,+0.000000E+00),complex(+0.000000E+00,+1.325825E-01),complex(+4.419418E-02,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.325825E-01),complex(-3.093592E-01,+0.000000E+00),complex(+0.000000E+00,+4.419417E-01),complex(+4.050463E-01,+0.000000E+00),complex(+0.000000E+00,-1.653594E-01),complex(+1.653594E-01,+0.000000E+00),complex(+0.000000E+00,-4.050463E-01),complex(-4.419417E-01,+0.000000E+00),complex(+0.000000E+00,+3.093592E-01),complex(+1.325825E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.651650E-01),complex(+4.419417E-01,+0.000000E+00),complex(+0.000000E+00,-3.535534E-01),complex(-7.450581E-08,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-08),complex(-3.535534E-01,+0.000000E+00),complex(+0.000000E+00,+4.419418E-01),complex(+2.651651E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.050463E-01),complex(-4.050463E-01,+0.000000E+00),complex(+0.000000E+00,+4.828528E-08),complex(-3.535534E-01,+0.000000E+00),complex(+0.000000E+00,+2.165064E-01),complex(-2.165064E-01,+0.000000E+00),complex(+0.000000E+00,+3.535534E-01),complex(-2.414264E-08,+0.000000E+00),complex(+0.000000E+00,+4.050463E-01),complex(+4.050463E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.960784E-01),complex(+1.653594E-01,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(+2.165065E-01,+0.000000E+00),complex(+0.000000E+00,+2.651650E-01),complex(+2.651650E-01,+0.000000E+00),complex(+0.000000E+00,+2.165064E-01),complex(+3.307190E-01,+0.000000E+00),complex(+0.000000E+00,+1.653594E-01),complex(+4.960784E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.960784E-01),complex(+1.653595E-01,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(+2.165064E-01,+0.000000E+00),complex(+0.000000E+00,-2.651650E-01),complex(+2.651650E-01,+0.000000E+00),complex(+0.000000E+00,-2.165064E-01),complex(+3.307189E-01,+0.000000E+00),complex(+0.000000E+00,-1.653595E-01),complex(+4.960784E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+4.050463E-01),complex(-4.050461E-01,+0.000000E+00),complex(+0.000000E+00,+1.848296E-08),complex(-3.535534E-01,+0.000000E+00),complex(+0.000000E+00,-2.165063E-01),complex(-2.165064E-01,+0.000000E+00),complex(+0.000000E+00,-3.535534E-01),complex(-5.394496E-08,+0.000000E+00),complex(+0.000000E+00,-4.050464E-01),complex(+4.050463E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.651651E-01),complex(+4.419418E-01,+0.000000E+00),complex(+0.000000E+00,+3.535534E-01),complex(-1.490116E-08,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,+1.490116E-08),complex(-3.535534E-01,+0.000000E+00),complex(+0.000000E+00,-4.419418E-01),complex(+2.651651E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+1.325824E-01),complex(-3.093593E-01,+0.000000E+00),complex(+0.000000E+00,-4.419418E-01),complex(+4.050463E-01,+0.000000E+00),complex(+0.000000E+00,+1.653595E-01),complex(+1.653594E-01,+0.000000E+00),complex(+0.000000E+00,+4.050463E-01),complex(-4.419417E-01,+0.000000E+00),complex(+0.000000E+00,-3.093593E-01),complex(+1.325825E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-4.419417E-02),complex(+1.325825E-01,+0.000000E+00),complex(+0.000000E+00,+2.651650E-01),complex(-4.050463E-01,+0.000000E+00),complex(+0.000000E+00,-4.960784E-01),complex(+4.960784E-01,+0.000000E+00),complex(+0.000000E+00,+4.050463E-01),complex(-2.651651E-01,+0.000000E+00),complex(+0.000000E+00,-1.325825E-01),complex(+4.419418E-02,-0.000000E+00)]$
            ]
          end

       8: begin
            matrix[36:44,36:44]=[$
            [complex(+6.249997E-02,+0.000000E+00),complex(+0.000000E+00,-1.767768E-01),complex(-3.307190E-01,+0.000000E+00),complex(+0.000000E+00,+4.677072E-01),complex(+5.229125E-01,+0.000000E+00),complex(+0.000000E+00,-4.677072E-01),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,+1.767767E-01),complex(+6.250001E-02,-0.000000E+00)],$
            [complex(-1.767767E-01,+0.000000E+00),complex(+0.000000E+00,+3.750000E-01),complex(+4.677071E-01,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(+4.759509E-08,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(-4.677072E-01,+0.000000E+00),complex(+0.000000E+00,+3.750001E-01),complex(+1.767767E-01,+0.000000E+00)],$
            [complex(+3.307189E-01,+0.000000E+00),complex(+0.000000E+00,-4.677073E-01),complex(-2.499999E-01,+0.000000E+00),complex(+0.000000E+00,-1.767767E-01),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,+1.767767E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+4.677072E-01),complex(+3.307189E-01,-0.000000E+00)],$
            [complex(-4.677071E-01,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(-1.767766E-01,+0.000000E+00),complex(+0.000000E+00,+3.750001E-01),complex(+2.206292E-09,+0.000000E+00),complex(+0.000000E+00,+3.750000E-01),complex(+1.767767E-01,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(+4.677071E-01,-0.000000E+00)],$
            [complex(+5.229125E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+3.952848E-01,+0.000000E+00),0,complex(+3.750000E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(+3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+5.229125E-01,-0.000000E+00)],$
            [complex(-4.677070E-01,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(-1.767766E-01,-0.000000E+00),complex(+0.000000E+00,-3.750000E-01),complex(-3.721847E-08,+0.000000E+00),complex(+0.000000E+00,-3.750000E-01),complex(+1.767766E-01,+0.000000E+00),complex(+0.000000E+00,-3.307189E-01),complex(+4.677071E-01,+0.000000E+00)],$
            [complex(+3.307191E-01,+0.000000E+00),complex(+0.000000E+00,+4.677072E-01),complex(-2.499999E-01,+0.000000E+00),complex(+0.000000E+00,+1.767768E-01),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-1.767766E-01),complex(-2.500001E-01,+0.000000E+00),complex(+0.000000E+00,-4.677072E-01),complex(+3.307188E-01,-0.000000E+00)],$
            [complex(-1.767766E-01,+0.000000E+00),complex(+0.000000E+00,-3.750001E-01),complex(+4.677072E-01,+0.000000E+00),complex(+0.000000E+00,+3.307189E-01),complex(+2.891598E-09,+0.000000E+00),complex(+0.000000E+00,+3.307190E-01),complex(-4.677071E-01,+0.000000E+00),complex(+0.000000E+00,-3.750000E-01),complex(+1.767766E-01,+0.000000E+00)],$
            [complex(+6.250003E-02,+0.000000E+00),complex(+0.000000E+00,+1.767767E-01),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,-4.677072E-01),complex(+5.229126E-01,+0.000000E+00),complex(+0.000000E+00,+4.677072E-01),complex(-3.307189E-01,+0.000000E+00),complex(+0.000000E+00,-1.767767E-01),complex(+6.249999E-02,-0.000000E+00)]$
            ]
          end

       7: begin
            matrix[28:35,28:35]=[$
            [complex(+0.000000E+00,-8.838837E-02),complex(-2.338536E-01,+0.000000E+00),complex(+0.000000E+00,+4.050463E-01),complex(+5.229126E-01,+0.000000E+00),complex(+0.000000E+00,-5.229126E-01),complex(-4.050463E-01,+0.000000E+00),complex(+0.000000E+00,+2.338536E-01),complex(+8.838837E-02,+0.000000E+00)],$
            [complex(+0.000000E+00,+2.338536E-01),complex(+4.419418E-01,+0.000000E+00),complex(+0.000000E+00,-4.592794E-01),complex(-1.976424E-01,+0.000000E+00),complex(+0.000000E+00,-1.976424E-01),complex(-4.592793E-01,+0.000000E+00),complex(+0.000000E+00,+4.419417E-01),complex(+2.338536E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,-4.050463E-01),complex(-4.592793E-01,+0.000000E+00),complex(+0.000000E+00,+8.838829E-02),complex(-3.423266E-01,+0.000000E+00),complex(+0.000000E+00,+3.423266E-01),complex(-8.838829E-02,+0.000000E+00),complex(+0.000000E+00,+4.592793E-01),complex(+4.050463E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+5.229124E-01),complex(+1.976423E-01,+0.000000E+00),complex(+0.000000E+00,+3.423266E-01),complex(+2.651651E-01,+0.000000E+00),complex(+0.000000E+00,+2.651650E-01),complex(+3.423266E-01,+0.000000E+00),complex(+0.000000E+00,+1.976423E-01),complex(+5.229126E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-5.229126E-01),complex(+1.976424E-01,+0.000000E+00),complex(+0.000000E+00,-3.423266E-01),complex(+2.651651E-01,+0.000000E+00),complex(+0.000000E+00,-2.651651E-01),complex(+3.423266E-01,+0.000000E+00),complex(+0.000000E+00,-1.976424E-01),complex(+5.229126E-01,-0.000000E+00)],$
            [complex(-0.000000E+00,+4.050463E-01),complex(-4.592793E-01,+0.000000E+00),complex(+0.000000E+00,-8.838831E-02),complex(-3.423266E-01,+0.000000E+00),complex(+0.000000E+00,-3.423266E-01),complex(-8.838832E-02,+0.000000E+00),complex(+0.000000E+00,-4.592793E-01),complex(+4.050463E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-2.338536E-01),complex(+4.419417E-01,+0.000000E+00),complex(+0.000000E+00,+4.592793E-01),complex(-1.976423E-01,+0.000000E+00),complex(+0.000000E+00,+1.976424E-01),complex(-4.592793E-01,+0.000000E+00),complex(+0.000000E+00,-4.419417E-01),complex(+2.338536E-01,+0.000000E+00)],$
            [complex(+0.000000E+00,+8.838834E-02),complex(-2.338536E-01,+0.000000E+00),complex(+0.000000E+00,-4.050463E-01),complex(+5.229126E-01,-0.000000E+00),complex(+0.000000E+00,+5.229125E-01),complex(-4.050463E-01,+0.000000E+00),complex(+0.000000E+00,-2.338536E-01),complex(+8.838834E-02,-0.000000E+00)]$
            ]
          end

       6: begin
            matrix[21:27,21:27]=[$
            [complex(-1.250000E-01,+0.000000E+00),complex(+0.000000E+00,+3.061862E-01),complex(+4.841229E-01,+0.000000E+00),complex(+0.000000E+00,-5.590170E-01),complex(-4.841229E-01,-0.000000E+00),complex(+0.000000E+00,+3.061862E-01),complex(+1.250000E-01,-0.000000E+00)],$
            [complex(+3.061861E-01,+0.000000E+00),complex(+0.000000E+00,-5.000000E-01),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,+5.000000E-01),complex(+3.061863E-01,-0.000000E+00)],$
            [complex(-4.841229E-01,+0.000000E+00),complex(+0.000000E+00,+3.952847E-01),complex(-1.250000E-01,+0.000000E+00),complex(+0.000000E+00,+4.330127E-01),complex(+1.250000E-01,+0.000000E+00),complex(+0.000000E+00,+3.952847E-01),complex(+4.841229E-01,-0.000000E+00)],$
            [complex(+5.590170E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+4.330127E-01,+0.000000E+00),0,complex(+4.330127E-01,+0.000000E+00),complex(+0.000000E+00,-2.980232E-08),complex(+5.590170E-01,-0.000000E+00)],$
            [complex(-4.841229E-01,+0.000000E+00),complex(+0.000000E+00,-3.952847E-01),complex(-1.250000E-01,+0.000000E+00),complex(+0.000000E+00,-4.330127E-01),complex(+1.250000E-01,+0.000000E+00),complex(+0.000000E+00,-3.952847E-01),complex(+4.841229E-01,-0.000000E+00)],$
            [complex(+3.061862E-01,+0.000000E+00),complex(-0.000000E+00,+5.000000E-01),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-5.960464E-08),complex(-3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-5.000000E-01),complex(+3.061862E-01,-0.000000E+00)],$
            [complex(-1.250000E-01,-0.000000E+00),complex(+0.000000E+00,-3.061862E-01),complex(+4.841229E-01,+0.000000E+00),complex(+0.000000E+00,+5.590170E-01),complex(-4.841229E-01,+0.000000E+00),complex(+0.000000E+00,-3.061862E-01),complex(+1.250000E-01,+0.000000E+00)]$
            ]
          end

       5: begin
            matrix[15:20,15:20]=[$
            [complex(+0.000000E+00,+1.767767E-01),complex(+3.952847E-01,+0.000000E+00),complex(+0.000000E+00,-5.590170E-01),complex(-5.590169E-01,+0.000000E+00),complex(+0.000000E+00,+3.952847E-01),complex(+1.767767E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-3.952847E-01),complex(-5.303301E-01,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(-2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+5.303301E-01),complex(+3.952847E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+5.590169E-01),complex(+2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+3.535534E-01),complex(+3.535534E-01,+0.000000E+00),complex(+0.000000E+00,+2.500000E-01),complex(+5.590169E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-5.590170E-01),complex(+2.500000E-01,+0.000000E+00),complex(+0.000000E+00,-3.535534E-01),complex(+3.535534E-01,+0.000000E+00),complex(+0.000000E+00,-2.500000E-01),complex(+5.590170E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.952847E-01),complex(-5.303301E-01,+0.000000E+00),complex(+0.000000E+00,-2.499999E-01),complex(-2.500001E-01,-0.000000E+00),complex(+0.000000E+00,-5.303301E-01),complex(+3.952847E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-1.767767E-01),complex(+3.952847E-01,+0.000000E+00),complex(+0.000000E+00,+5.590170E-01),complex(-5.590170E-01,+0.000000E+00),complex(+0.000000E+00,-3.952847E-01),complex(+1.767767E-01,+0.000000E+00)]$
            ]
          end

       4: begin
            matrix[10:14,10:14]=[$
            [complex(+2.500000E-01,+0.000000E+00),complex(-0.000000E+00,-5.000000E-01),complex(-6.123724E-01,+0.000000E+00),complex(+0.000000E+00,+5.000000E-01),complex(+2.500000E-01,+0.000000E+00)],$
            [complex(-5.000000E-01,+0.000000E+00),complex(+0.000000E+00,+5.000000E-01),0,complex(+0.000000E+00,+5.000000E-01),complex(+5.000000E-01,+0.000000E+00)],$
            [complex(+6.123724E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+4.999999E-01,+0.000000E+00),0,complex(+6.123724E-01,+0.000000E+00)],$
            [complex(-5.000000E-01,+0.000000E+00),complex(+0.000000E+00,-5.000000E-01),0,complex(+0.000000E+00,-5.000000E-01),complex(+5.000000E-01,+0.000000E+00)],$
            [complex(+2.500000E-01,+0.000000E+00),complex(+0.000000E+00,+5.000000E-01),complex(-6.123724E-01,+0.000000E+00),complex(+0.000000E+00,-5.000000E-01),complex(+2.500000E-01,+0.000000E+00)]$
            ]
          end

       3: begin
            matrix[6:9,6:9]=[$
            [complex(+0.000000E+00,-3.535534E-01),complex(-6.123724E-01,+0.000000E+00),complex(+0.000000E+00,+6.123725E-01),complex(+3.535534E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+6.123725E-01),complex(+3.535534E-01,+0.000000E+00),complex(+0.000000E+00,+3.535534E-01),complex(+6.123725E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-6.123725E-01),complex(+3.535535E-01,+0.000000E+00),complex(+0.000000E+00,-3.535534E-01),complex(+6.123725E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,+3.535533E-01),complex(-6.123724E-01,+0.000000E+00),complex(+0.000000E+00,-6.123725E-01),complex(+3.535535E-01,+0.000000E+00)]$
            ]
          end

       2: begin
            matrix[3:5,3:5]=[$
            [complex(-4.999999E-01,+0.000000E+00),complex(+0.000000E+00,+7.071068E-01),complex(+5.000000E-01,+0.000000E+00)],$
            [complex(+7.071069E-01,+0.000000E+00),complex(+0.000000E+00,+5.960464E-08),complex(+7.071067E-01,+0.000000E+00)],$
            [complex(-5.000000E-01,+0.000000E+00),complex(+0.000000E+00,-7.071068E-01),complex(+5.000000E-01,+0.000000E+00)]$
            ]
          end

       1: begin
            matrix[1:2,1:2]=[$
            [complex(+0.000000E+00,+7.071068E-01),complex(+7.071068E-01,-0.000000E+00)],$
            [complex(+0.000000E+00,-7.071068E-01),complex(+7.071068E-01,-0.000000E+00)]$
            ]
          end

       0: matrix[0]=complex(1.,0.)
 
    endswitch
  endelse
endelse

; Code to write the matrix look-up tables used in this routine. Note that we
; can't print out the whole matrix because the read buffer isn't long enough
; to cope with entire matrices larger than n_max=4. Uncomment this to write
; out the necessary code to a temp file.
;openw,lun,"~/temp",/get_lun
;printf,lun,"pro temp, matrix=matrix"
;printf,lun,"n_max=",n_max,format='(A6,I0)'
;printf,lun,"n_coeffs=",n_coeffs,format='(A9,I0)'
;printf,lun
;printf,lun
;printf,lun,"matrix=complexarr(n_coeffs,n_coeffs)"
;printf,lun,"switch n_max of"
;printf,lun
;for n=n_max,0,-1 do begin
;  wn=where(n1+n2 eq n,n_n)
;  minn=min(wn) & maxn=max(wn) 
;  matrixn=matrix[minn:maxn,minn:maxn]
;  printf,lun,"  ",n,": begin",format='(A2,I2,A7)'
;  printf,lun,"        matrix[",minn,":",maxn,",",minn,":",maxn,"]","=[$",$
;        format='(A15,4(I0,A1),A3)'
;  for i=minn,maxn do begin
;    printf,lun,"        [",format='($,A9)'
;    for j=minn,maxn do begin
;      if matrix[j,i] eq 0 then begin
;        printf,lun,0,format='($,I1)'
;      endif else begin
;        printf,lun,"complex(",float(matrix[j,i]),",",imaginary(matrix[j,i]),")",$
;             format='($,A8,2(E+-13.6,A1))'
;             ;format='($,A8,2(F+-10.7,A1))'
;      endelse
;      if j lt maxn then printf,lun,",",format='($,A1)'
;    endfor
;    printf,lun,"]",format='($,A1)'
;  if i lt maxn then printf,lun,",",format='($,A1)'
;    printf,lun,"$",format='(A1)'
;  endfor
;  printf,lun,"        ]",format='(A9)'
;  printf,lun,"      end",format='(A9)'
;  printf,lun
;endfor
;printf,lun,"endswitch"
;printf,lun
;printf,lun
;printf,lun,"end"
;close,lun
;free_lun,lun
;print,"Now type:"
;print,".com ~/temp"
;print,"temp, matrix=matrix"

return,matrix

end
